<?php
/**
Copyright (c) 2014 - 2019 - IgniteWoo.com - All Rights Reserved
*/
class IGN_Arbitrary_Products { 

	public static function maybe_add_arbitrary_product() { 
		
		if ( empty( $_POST ) ) 
			die( 'Error' );
			
		if ( empty( $_POST['data'] ) )
			die( 'Error' );

		$data = wp_parse_args( $_POST['data'] );

		if ( empty( $data['title'] ) || empty( $data['quantity'] ) || !isset( $data['price'] ) )
			die( __( 'Error: title, price, and quantity are required. Price can be 0 (zero).', 'manual_phone_orders' ) );

		// Create a new product, set the status to publish and the catalog visibility to hidden 
		$post = array(
			'post_author' => 1,
			'post_content' => __( 'This product was temporarily created for a manual/order order. This product will be automatically deleted.', 'manual_phone_orders' ),
			'post_status' => "publish",
			'post_title' => $data['title'],
			'post_parent' => '',
			'post_type' => "product",
		);

		$post_id = wp_insert_post( $post );

		if ( empty( $post_id ) || is_wp_error( $post_id ) )
			die( __( 'Error adding arbitray product', 'manual_phone_orders' ) );
			
		// wp_set_object_terms( $post_id, 'Races', 'product_cat' );
		wp_set_object_terms($post_id, 'simple', 'product_type');

		if ( empty( $data['price'] ) ) { 
			$data['price'] = 0; 
		}
		
		if ( version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) ) { 
			
			$product = wc_get_product( $post_id );
			
			$product->set_price( $data['price'] );
			$product->set_regular_price( $data['price'] );
			
			$product->set_catalog_visibility( 'hidden' );
			$product->set_stock_status( 'instock' );
			$product->set_sku( $data['sku'] );
			
			if ( empty( $data['weight'] ) ) {
				$product->set_downloadable( 'yes' );
				$product->set_virtual( 'yes' );
			} else { 
				$product->set_downloadable( 'no' );
				$product->set_virtual( 'no' );
			}
			
			$product->set_weight( $data['weight'] );
			$product->set_length( $data['length'] );
			$product->set_width( $data['width'] );
			$product->set_height( $data['height'] );

			if ( isset( $data['tax_status'] ) ) { 
				$product->set_tax_status( $data['tax_status'] );
			}
			
			if ( isset( $data['tax_class'] ) ) { 
				$product->set_tax_class( $data['tax_class'] );
			}
			
			$product->set_purchase_note( '' );
			$product->set_featured( '' );
			
			$product->add_meta_data( '_ign_opc_arbitrary_product', time() );
			
			$product->save();
			
		} else { 
			update_post_meta($post_id, '_sku', $data['sku'] );
			
			update_post_meta( $post_id, '_visibility', 'hidden' );
			update_post_meta( $post_id, '_stock_status', 'instock');
			update_post_meta( $post_id, 'total_sales', '0');
			
			if ( empty( $data['weight'] ) ) {
				update_post_meta( $post_id, '_downloadable', 'yes');
				update_post_meta( $post_id, '_virtual', 'yes');
			} else { 
				update_post_meta( $post_id, '_downloadable', 'no');
				update_post_meta( $post_id, '_virtual', 'no');
			}
			
			update_post_meta( $post_id, '_weight', $data['weight'] );
			update_post_meta( $post_id, '_length', $data['length'] );
			update_post_meta( $post_id, '_width', $data['width'] );
			update_post_meta( $post_id, '_height', $data['height'] );
			
			update_post_meta( $post_id, '_price', $data['price'] );
			update_post_meta( $post_id, '_regular_price', $data['price'] );
			update_post_meta( $post_id, '_sale_price', "" );
			
			if ( isset( $data['tax_status'] ) ) {
				update_post_meta( $post_id, '_tax_status', $data['tax_status'] );
			}
			
			if ( isset( $data['tax_class'] ) ) { 
				update_post_meta( $post_id, '_tax_class', $data['tax_class'] );
			}
			
			update_post_meta( $post_id, '_purchase_note', "" );
			update_post_meta( $post_id, '_featured', "no" );
			
			update_post_meta( $post_id, '_product_attributes', array());
			update_post_meta( $post_id, '_sale_price_dates_from', "" );
			update_post_meta( $post_id, '_sale_price_dates_to', "" );
			update_post_meta( $post_id, '_sold_individually', "" );
			update_post_meta( $post_id, '_manage_stock', "no" );
			update_post_meta( $post_id, '_backorders', "no" );
			update_post_meta( $post_id, '_stock', "" );
						
			// Add custom post meta so we can remove these products either when the order is written
			// or later during a catchall clean up cron routine.
			// NOTE: The products are delete in the "thankyou_page_additions()" function, if the order isn't on hold
			update_post_meta( $post_id, '_ign_opc_arbitrary_product', time() );
		}

		// Set visibility 
		$terms = array( 'exclude-from-search', 'exclude-from-catalog' );

		wp_set_post_terms( $post_id, $terms, 'product_visibility', false );
		
		// Add to cart 
		$cart_item_key = IGN_OPC_Cart::add_to_cart( $post_id, $data['quantity'] );
		
		// Add this custom cart item data, this plugin uses it to know what they price should be.
		// And this, in theory, will prevent other pricing tools from overriding it.
		if ( !empty( $cart_item_key ) ) {
			WC()->cart->cart_contents[ $cart_item_key ]['_price_altered'] = $data['price'];
		}

		die( 'done' );
	}
	
	// Function runs when the order status reaches Processing or Completed
	public static function maybe_delete_arbitrary_product( $order_id ) { 

		$order = wc_get_order( $order_id );

		if ( empty( $order ) )
			return;
			
		foreach( $order->get_items( 'line_item' ) as $item_id => $item ) { 
		
			$product = $item->get_product();
			
			if ( empty( $product ) )
				continue;
			
			if ( method_exists( $product, 'get_id' ) )
				$pid = $product->get_id();
			else 
				$pid = $product->id; 
				
			if ( empty( $pid ) ) 
				continue;
			
			if ( method_exists( $product, 'get_meta' ) )
				$is_arb = $product->get_meta( '_ign_opc_arbitrary_product', true );
			else 
				$is_arb = get_post_meta( $pid, '_ign_opc_arbitrary_product', true );

			if ( empty( $is_arb ) )
				continue; 

			if ( method_exists( $product, 'delete' ) ) { 
				$product->delete();
			} else { 
				wp_delete_post( $pid, true );	
			}
		}
	}
	
	public static function add_meta_query_vars( $query, $query_vars ) {
		if ( ! empty( $query_vars['ign_opc_arbitrary_product'] ) ) {
			$query['meta_query'] = array( 
				'relation' => 'AND',
				array( 
					'key' => '_ign_opc_arbitrary_product',
					'compare' => 'EXISTS' 
				),
				array( 
					'key' => '_ign_opc_arbitrary_product',
					'value' => time() - ( HOUR_IN_SECONDS ), // 1 hour old or more
					'compare' => '<' 
				)
			);
		}
		
		unset( $query['ign_opc_arbitrary_product'] );

		return $query;
	}
	
	// CRON FUNCTION - remove arbitrary products whose expiration date has passed
	public static function remove_arbitrary_products() { 
		// Add filter to modify meta query.
		add_filter( 'woocommerce_product_data_store_cpt_get_products_query', __CLASS__ . '::add_meta_query_vars', 10, 2 );

		$args = array( 
			'limit' => -1,
			'ign_opc_arbitrary_product' => 1,
		);
		
		$products = wc_get_products( $args );

		// Remove meta query filter.
		remove_filter( 'woocommerce_product_data_store_cpt_get_products_query', __CLASS__ . '::add_meta_query_vars', 10, 2 );
		
		// Delete and products returned by the query.
		foreach( $products as $product ) { 		
			if ( !empty( $product ) && method_exists( $product, 'delete' ) ) { 
				$product->delete();
			}
		}
	}
}
