<?php
/**
Copyright (c) 2014 - 2019 - IgniteWoo.com - All Rights Reserved
*/
class IGN_OPC_Cart { 

	private static $updating_cart_in_bulk = false;
	
	public static function get_cart_item_from_session( $cart_item, $values, $key ) {

		if ( isset( $values['_price_altered'] ) ) {
			$cart_item['_price_altered'] = $values['_price_altered'];
	
			if ( '' !== $values['_price_altered'] ) { 
				$cart_item['data']->set_price( floatval( $values['_price_altered'] ) );
				// Don't do this, it will prevent the regular price showing as crossed out
				//$cart_item['data']->set_regular_price( $values['_price_altered'] );
			}
		}
	
		return $cart_item;
	}
	
	// NOTE: This is used by the ordering page AND the API endpoint
	// $api - set to TRUE to avoid sending output, used by the API 
	public static function ign_one_page_searchable_add( $args = null, $api = false ) {
		global $cart_products, $display_images, $user_ID, $ign_opc, $ign_opc_core;

		//if ( !$ign_opc_core->has_access() )
		//	return;

		if ( !empty( $args ) )
			$ids = $args;
		else if ( isset( $_POST['items'] ) )
			$ids = $_POST['items'];

		if ( !is_array( $ids ) )
			$ids = explode( ',', $ids );

		if ( empty( $ids ) )
			die( __( 'No products found', 'manual_phone_orders' ) );

		//$ign_opc->switch_user_context_to_customer( 'customer' );

		$cart_products = array();

		$products_qty_in_cart = array();

		$temp_cart = WC()->cart->get_cart();

		ob_start();
		
		foreach( $ids as $id ) {

			$pdata = $ign_opc_core->load_product( $id );

			if ( method_exists( $pdata, 'get_type' ) )
				$ptype = $pdata->get_type();
			else 
				$ptype = $pdata->product_type;
				
			// Check if the item is already in the cart, if so don't touch it, process next item
			if ( $ptype == 'variation' ) {

				$check_qty = isset( $products_qty_in_cart[ $id ] ) ? $products_qty_in_cart[ $id ] : 0;

			} else {

				$check_qty = isset( $products_qty_in_cart[ $id ] ) ? $products_qty_in_cart[ $id ] : 0;
			}

			if ( $check_qty > 0 ) {
				$item_quantity = $check_qty;
			} else {
				//$item_quantity = 1;
				$default = method_exists( $pdata, 'get_min_purchase_quantity' ) ? $pdata->get_min_purchase_quantity() : 1;
				$item_quantity = apply_filters( 'woocommerce_quantity_input_min', $default, $pdata );
			}

			if ( 'grouped' == $ptype ) {

				$children = $pdata->get_children();

				$qty = array();

				if ( !empty( $children ) ) {

					foreach( $children as $child ) {

						$check_qty = isset( $products_qty_in_cart[ $child ] ) ? $products_qty_in_cart[ $child ] : 1;

						if ( !empty( $check_qty) && $check_qty > 0 ) {
							$item_quantity = $check_qty;
						} else { 
							$item_quantity = 1;
							//$default = method_exists( $pdata, 'get_min_purchase_quantity' ) ? $pdata->get_min_purchase_quantity() : 1;
							//$item_quantity = apply_filters( 'woocommerce_quantity_input_min', $default, $pdata );
						}

						$qty[ $child ] = $item_quantity;
					}

					$args = array(
						'add-to-cart' => $id,
						'product_id' => $id,
						'qty' => $qty,
						'empty_cart' => false
					);

					self::add_to_cart_grouped( $args );
				}

			} else if ( 'variation' == $ptype ) {

				$_product = $ign_opc_core->products[ $id ];

				$variation_data = wc_get_product_variation_attributes( $id );

				$args = array(
					'product_id' => $_product['data']->parent->id,
					'variation_id' => $id,
					'variation_data' => $variation_data,
					'qty' => $item_quantity,
					'empty_cart' => false,
				);

				self::add_variation_to_cart( $args );

			} else {

				// Simple
				$args = array(
					'product_id' => $id,
					'qty' => $item_quantity,
					'empty_cart' => false
				);

				self::add_item_to_cart( $args );
			}
		}

		// Re-add any custom pricing that existed in the cart
		foreach( $temp_cart as $key => $vals ) {

			if ( empty( $vals['_price_altered'] )  )
				continue;

			WC()->cart->cart_contents[ $key ]['_price_altered'] = $vals['_price_altered'];
			WC()->cart->cart_contents[ $key ]['data']->set_price( $vals['_price_altered'] );
			WC()->cart->calculate_totals();

			$pid = isset( WC()->cart->cart_contents[ $key ]['variation_id'] ) ? WC()->cart->cart_contents[ $key ]['variation_id'] : WC()->cart->cart_contents[ $key ]['product_id'];

			foreach( $ign_opc_core->products as $id => $data ) {

				if ( $id == $pid ) {
					$ign_opc_core->products[ $id ]['_price_altered'] = $vals['_price_altered'];
				}
			}
		}

		$cart_products = $ign_opc_core->products;

		$display_images = true;

		// Set this in case a third party plugin needs it, such as tiered pricing
		$_POST['opc_checkout_flag'] = 1;
		
 		$ign_opc_core->display_cart_contents_table( $cart_products );

		//$ign_opc->switch_user_context_to_customer( 'opc_user' );

		$out = ob_get_clean();

		if ( !empty( $args ) ) {
			$out = explode( '<!--WC_END-->', $out );
		}

		// echo the last element in the array, there should not be any messages after the table
		if ( !$api ) { 
			echo end( $out );
		}

		if ( !empty( $args ) && !$api ) { 
			die();
		}
	}
	
	
	// Based on WC core code
	public static function add_to_cart( $product_id = 0, $quantity = 1, $variation_id = 0, $variation = array(), $cart_item_data = array() ) {
		global $ign_opc; 
		
		// Wrap in try catch so plugins can throw an exception to prevent adding to cart

		try {
		
			$product_id   = absint( $product_id );
			$variation_id = absint( $variation_id );

			// Ensure we don't add a variation to the cart directly by variation ID.
			if ( 'product_variation' === get_post_type( $product_id ) ) {
				$variation_id = $product_id;
				$product_id   = wp_get_post_parent_id( $variation_id );
			}

			$product_data = wc_get_product( $variation_id ? $variation_id : $product_id );

			// Sanitity check
			if ( $quantity <= 0 || ! $product_data || 'trash' === $product_data->get_status()  ) {
				if ( version_compare( WOOCOMMERCE_VERSION, '3.1', '>=' ) )
					return false;
				else 
					throw new Exception();
			}

			// Load cart item data - may be added by other plugins
			$cart_item_data = (array) apply_filters( 'woocommerce_add_cart_item_data', $cart_item_data, $product_id, $variation_id, $quantity );

			if ( !empty( $_POST['cart_item_key'] ) ) { 
				$cart_item_key = $_POST['cart_item_key'];
			}  else { 
				// Generate a ID based on product ID, variation ID, variation data, and other cart item data
				$cart_id        = WC()->cart->generate_cart_id( $product_id, $variation_id, $variation, $cart_item_data );
				// Find the cart item key in the existing cart
				$cart_item_key  = WC()->cart->find_product_in_cart( $cart_id );
			}

			
			// Force quantity to 1 if sold individually and check for exisitng item in cart
			/*
			if ( $product_data->is_sold_individually() ) {
				$quantity         = apply_filters( 'woocommerce_add_to_cart_sold_individually_quantity', 1, $quantity, $product_id, $variation_id, $cart_item_data );
				$in_cart_quantity = $cart_item_key ? WC()->cart->cart_contents[ $cart_item_key ]['quantity'] : 0;

				if ( $in_cart_quantity > 0 ) {
					throw new Exception( sprintf( '<a href="%s" class="button wc-forward">%s</a> %s', WC()->cart->get_cart_url(), __( 'View Cart', 'manual_phone_orders' ), sprintf( __( 'You cannot add another &quot;%s&quot; to your cart.', 'manual_phone_orders' ), $product_data->get_title() ) ) );
				}
			}

			// Check product is_purchasable
			if ( ! $product_data->is_purchasable() ) {
				throw new Exception( __( 'Sorry, this product cannot be purchased.', 'manual_phone_orders' ) );
			}

			// Stock check - only check if we're managing stock and backorders are not allowed
			if ( ! $product_data->is_in_stock() ) {
				throw new Exception( sprintf( __( 'You cannot add &quot;%s&quot; to the cart because the product is out of stock.', 'manual_phone_orders' ), $product_data->get_title() ) );
			}

			if ( ! $product_data->has_enough_stock( $quantity ) ) {
				throw new Exception( sprintf(__( 'You cannot add that amount of &quot;%s&quot; to the cart because there is not enough stock (%s remaining).', 'manual_phone_orders' ), $product_data->get_title(), $product_data->get_stock_quantity() ) );
			}


			// Stock check - this time accounting for whats already in-cart
			if ( $managing_stock = $product_data->managing_stock() ) {
				$products_qty_in_cart = WC()->cart->get_cart_item_quantities();

				if ( $product_data->is_type( 'variation' ) && true === $managing_stock ) {
					$check_qty = isset( $products_qty_in_cart[ $variation_id ] ) ? $products_qty_in_cart[ $variation_id ] : 0;
				} else {
					$check_qty = isset( $products_qty_in_cart[ $product_id ] ) ? $products_qty_in_cart[ $product_id ] : 0;
				}


				//Check stock based on all items in the cart

				if ( ! $product_data->has_enough_stock( $check_qty + $quantity ) ) {
					throw new Exception( sprintf(
						'<a href="%s" class="button wc-forward">%s</a> %s',
						WC()->cart->get_cart_url(),
						__( 'View Cart', 'manual_phone_orders' ),
						sprintf( __( 'You cannot add that amount to the cart &mdash; we have %s in stock and you already have %s in your cart.', 'manual_phone_orders' ), $product_data->get_stock_quantity(), $check_qty )
					) );
				}
			}
			*/

			// If cart_item_key is set, the item is already in the cart
			if ( $cart_item_key ) {

				$new_quantity = $quantity + WC()->cart->cart_contents[ $cart_item_key ]['quantity'];
				WC()->cart->set_quantity( $cart_item_key, $new_quantity, false );
			} else {
				$cart_item_key = $cart_id;

				// Add item after merging with $cart_item_data - hook to allow plugins to modify cart item
				WC()->cart->cart_contents[ $cart_item_key ] = apply_filters( 'woocommerce_add_cart_item', array_merge( $cart_item_data, array(
					'product_id'	=> $product_id,
					'variation_id'	=> $variation_id,
					'variation' 	=> $variation,
					'quantity' 		=> $quantity,
					'data'		=> $product_data
				) ), $cart_item_key );

			}

			if ( did_action( 'wp' ) ) {
				WC()->cart->set_cart_cookies( sizeof( WC()->cart->cart_contents ) > 0 );
			}

			do_action( 'woocommerce_add_to_cart', $cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data );

			return $cart_item_key;

		} catch ( Exception $e ) {
			if ( $e->getMessage() ) {
				wc_add_notice( $e->getMessage(), 'error' );
			}
			return false;
		}
	}

	
	public static function add_to_cart_donation() {

		$product_id        = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_POST['product_id'] ) );

		$quantity          = 1;

		//$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

		$passed_validation = true; // allow admin & store manager to add anyway, just like in the WP admin area

		$cik = null;

		foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {

			if ( $values['product_id'] == $product_id ) {
				$cik = $cart_item_key;
				break;
			}
		}

		if ( !empty( $cik ) )
			WC()->cart->set_quantity( $cik, 0 ); // remove so it can be re-added with the new qty

		if ( $passed_validation && self::add_to_cart( $product_id, $quantity ) ) {

			$_product = wc_get_product( $product_id );

			ob_start();

			//wc_add_notice( sprintf( __( '%s has been added to your order', 'manual_phone_orders' ), $_product->get_title() ) );

			//wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'success',
				'messages' => $messages
			);

			if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {

				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';
			} else {
				echo json_encode( $data );
			}

		} else {

			ob_start();

			wc_add_notice( __( 'Error adding donation to cart', 'manual_phone_orders' ), 'error' );

			wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'failure',
				'messages' => $messages
			);

			if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {
				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';
			} else {
				echo json_encode( $data );
			}
		}

		die();
	}

	public static function add_to_cart_gift_cert() {

		$product_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_POST['product_id'] ) );

		$quantity = empty( $_POST['quantity'] ) ? 0 : apply_filters( 'woocommerce_stock_amount', $_POST['quantity'] );

		//$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

		$passed_validation = true; // allow admin & store manager to add anyway, just like in the WP admin area

		//if ( 'true' === $_REQUEST['empty_cart'] )
		//	WC()->cart->empty_cart();

		$cik = null;

		foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {

			if ( $values['product_id'] == $product_id ) {
				$cik = $cart_item_key;
				break;
			}
		}

		if ( !empty( $cik ) )
			WC()->cart->set_quantity( $cik, 0 ); // remove so it can be re-added with the new qty

		if ( $passed_validation && self::add_to_cart( $product_id, $quantity ) ) {

			$_product = wc_get_product( $product_id );

			ob_start();

			//wc_add_notice( sprintf( __( '%s has been added to your order', 'manual_phone_orders' ), $_product->get_title() ) );

			//wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'success',
				'messages' => $messages
			);

			if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {
				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';
			} else {
				echo json_decode( $data );
			}

		} else {

			ob_start();

			wc_add_notice( __( 'Error adding item to cart', 'manual_phone_orders' ), 'error' );

			wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'failure',
				'messages' => $messages
			);

			if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {
				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';
			} else {
				echo json_encode( $data );
			}
		}

		die();
	}

	public static function add_to_cart_product_remove() {

		//$product_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_POST['product_id'] ) );
		$product_id = absint( $_POST['product_id'] );

		if ( empty( $product_id ) ) {
			die;
		}

		$variation_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_POST['variation_id'] ) );

		foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {

			if ( !empty( $variation_id ) ) {

				if ( $values['variation_id'] == $variation_id && $values['product_id'] == $product_id )
					WC()->cart->set_quantity( $cart_item_key, 0 );

			} else if ( $values['product_id'] == $product_id ) {

				WC()->cart->set_quantity( $cart_item_key, 0 );
				
				// Delete an arbitrary product if removed from the cart.
				// 
				// This only applies to simple product since arbitrary products can only be simple products
					
				$product = $values['data'];
				
				if ( method_exists( $product, 'get_id' ) )
					$pid = $product->get_id();
				else 
					$pid = $product->id; 
					
				if ( empty( $pid ) ) 
					continue;
					
				if ( method_exists( $product, 'get_meta' ) )
					$is_arb = $product->get_meta( '_ign_opc_arbitrary_product', true );
				else 
					$is_arb = get_post_meta( $product_id, '_ign_opc_arbitrary_product', true );

				if ( !empty( $is_arb ) )
					wp_delete_post( $product_id, true );
			}
		}

		$out = array();

		if ( !empty( $variation_id ) )
			$out['sel'] = $variation_id;
		else
			$out['sel'] = $product_id;

		die( '<!--WC_START-->' . json_encode( $out ) . '<!--WC_END-->' );
	}

	public static function standardize_number( $value ) {

		//using OPC as a temp holder so that decimals and thousands don't end up the same
		$value = str_replace(
			array( get_option( 'woocommerce_price_thousand_sep' ), get_option( 'woocommerce_price_decimal_sep' ), 'NYP' ),
			array( 'OPC', '.', '' ),
			$value
		);

		if ( function_exists( 'wc_format_decimal' ) )
			return wc_format_decimal( $value );
		else
			return woocommerce_format_total ( $value );

	}

	public static function maybe_save_new_price( $cart_item_key, $product_id = null, $quantity= null, $variation_id= null, $variation= null ) {

		if ( isset( $_REQUEST['price'] ) && isset( $_REQUEST['action'] ) && in_array( $_REQUEST['action'], array( 'woocommerce_add_to_cart_opc', 'woocommerce_add_to_cart_opc_variable' ) ) ) {
		
			WC()->cart->cart_contents[ $cart_item_key ]['_price_altered'] = self::standardize_number( $_REQUEST['price'] );
			WC()->cart->set_session();
		}
	}

	public static function maybe_calculate_new_prices( $cart_object ) {

		if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
			//if ( !empty( $ign_opc ) ) { 
			//	$ign_opc->switch_user_context_to_customer( 'customer' );
			//}
			return;
		}

		foreach ( $cart_object->get_cart() as $cart_item_key => $value ) {

			if ( !isset( WC()->cart->cart_contents[ $cart_item_key ]['_price_altered'] ) )
				continue;

			//if ( !WC()->session->__isset( $cart_item_key . '_price_altered' ) )
			//	continue;

			$price = WC()->cart->cart_contents[ $cart_item_key ]['_price_altered'];

			$value['data']->price = $price;
		}
	}

	public static function maybe_return_price( $price = '', $_product = false ) {
		global $ign_opc_core;

//var_dump( 'price' );
		if ( !$ign_opc_core->opc )
			return $price;
//var_dump( 'price2' );
		if ( function_exists( 'get_current_screen' ) ) {
			$current_screen = get_current_screen();

			if ( !empty( $current_screen ) && 'edit-product' == $current_screen->id )
				return $price;
		}

		$cart = WC()->cart;

		if ( empty( $cart ) )
			return $price;

		//$product_id = self::get_product_id( $_product );

		if ( $_product->is_type( 'variation' ) ) {
		
			if ( version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) )
				$variation_id = $_product->get_id();
			else 
				$variation_id = $_product->variation_id;

			foreach( $cart->cart_contents as $k => $v ) {

				if ( $v['variation_id'] == $variation_id && isset( $v['_price_altered'] ) ) {
					$price = $v['_price_altered'];
				}
			}

		} else {
			if ( version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) )
				$product_id = $_product->get_id();
			else 
				$product_id = $_product->id;
				
			foreach( $cart->cart_contents as $k => $v ) {

				if ( $v['product_id'] == $product_id && isset( $v['_price_altered'] ) ) {
					$price = $v['_price_altered'];
				}
			}
		}

		return $price;
	}

	// Our custom exit function.
	// Used to prevents all the add to cart functions from exiting if the 
	// cart is being updated in bulk via the Update the Cart button
	public static function exit_ajax( $code = false ) { 
		if ( !empty( self::$updating_cart_in_bulk ) )
			return;
			
		if ( !empty( $code ) )
			die( $code );
		else 
			die;
	}
	
	public static function clear_cart() { 
		global $cart_products, $display_images, $ign_opc, $ign_opc_core;
		
		WC()->cart->empty_cart();
		
		// Ensure that the session cart data is cleared:
		WC()->cart->get_cart_from_session(); // Load users session
		WC()->cart->empty_cart( true ); // Empty the cart
		WC()->session->set( 'cart', array() ); // Empty the session cart data

		$cart_products = false;

		//$ign_opc->switch_user_context_to_customer( 'customer' );

		$template_name = 'one_page_searchable_items_table.php';

		$ign_opc_core->load_template( $template_name );
		
		die;
	}
	
	public static function update_items_to_cart() {
		global $ign_opc, $ign_opc_core;

		$ign_opc_core->is_tax_exempt();
		
		if ( empty( $_REQUEST['items'] ) ) {
			die;
		}
			
		$items = $_REQUEST['items'];
		
		$_REQUEST = array();

		self::$updating_cart_in_bulk = true;

		ob_start();
		
		foreach( $items as $item ) { 
			$_REQUEST = $item;

			if ( !empty( $item['variation_id'] ) )
				self::add_variation_to_cart();
			else 
				self::add_item_to_cart();
		}
		
		ob_get_clean();
		
		self::$updating_cart_in_bulk = false;

		if ( !empty( $GLOBALS['ign_super_coupons'] ) && method_exists( $GLOBALS['ign_super_coupons'], 'maybe_auto_add_coupon' ) ) { 
			$ign_opc->switch_user_context_to_customer( 'customer' );
			$GLOBALS['ign_super_coupons']->maybe_auto_add_coupon();
			$ign_opc->switch_user_context_to_customer( 'opc_user' );
		}
		
		die( json_encode( array( 'result' => 'success', 'messages' => 'false' ) ) );
	}
	
	public static function is_item_in_cart( $args ) { 
	
		$cik = false; 
		
		if ( !empty( $args['cart_item_key'] ) ) { 
		
			foreach( WC()->cart->get_cart() as $cart_item_key => $values ) { 

				if ( $args['cart_item_key'] == $cart_item_key ) {
					$cik = $cart_item_key;
					break;
				}
			}
		} else { 
			foreach( WC()->cart->get_cart() as $cart_item_key => $values ) { 

				if ( $values['product_id'] == $args['product_id'] && empty( $args['variation_id'] ) ) {
					$cik = $cart_item_key;
					break;
				} else if ( $values['product_id'] == $args['product_id'] && $values['variation_id'] == $args['variation_id'] ) {
					$cik = $cart_item_key;
					break;
				}
			}
		}

		return $cik;
	}
	
	// Based on WC core
	// $args is passed only when using the "Searchable" page template style
	public static function add_item_to_cart( $args = null ) {

		if ( !empty( $args ) ) {
			$_REQUEST = $args;
		}

		if ( isset( $_REQUEST['type'] ) && 'grouped' == $_REQUEST['type'] ) {
			self::add_to_cart_grouped();
			return;
		}

		$product_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_REQUEST['product_id'] ) );

		$quantity = empty( $_REQUEST['qty'] ) ? 0 : apply_filters( 'woocommerce_stock_amount', $_REQUEST['qty'] );

		if ( absint( $quantity ) <= 0 ) {

			$_product = wc_get_product( $product_id );

			ob_start();

			wc_add_notice( sprintf( __( '%s has been removed from your order', 'manual_phone_orders' ), $_product->get_title() ) );

			wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'failure',
				'messages' => $messages
			);

			echo '<!--WC_START-->' . json_encode( $data ) . '<!--WC_END-->';

			//self::exit_ajax();
		}
		
		//$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );
		$passed_validation = true; // allow admin & store manager to add anyway, just like in the WP admin area

		//if ( 'true' === $_REQUEST['empty_cart'] )
		//	WC()->cart->empty_cart();

		$price = null;

		if ( isset( $_REQUEST['price'] ) ) {
			// Remove thousands sep first. 
			$th = wc_get_price_thousand_separator();
			
			$_REQUEST['price'] = str_replace( $th, '', $_REQUEST['price'] );
			
			$price = wc_format_decimal( $_REQUEST['price'] );
			//$price = floatval( $_REQUEST['price'] );
		}

		if ( isset( $_REQUEST['opc_price'] ) && '' !== $_REQUEST['opc_price'] ) {
			// Remove thousands sep first. 
			$th = wc_get_price_thousand_separator();
			
			$_REQUEST['opc_price'] = str_replace( $th, '', $_REQUEST['opc_price'] );
			
			$opc_price = wc_format_decimal( $_REQUEST['opc_price'] );
			//$price = floatval( $_REQUEST['price'] );
		}
		
		if ( !empty( $_POST['cart_item_key'] ) ) { 
			$cart_item_key = $_POST['cart_item_key'];
		} else { 
			$cart_item_key = false; 
		}

		if ( !empty( $_REQUEST['cart_item_key'] ) ) { 
			$cik = $_REQUEST['cart_item_key'];
		}  else { 
			$cik = self::is_item_in_cart( array( 'product_id' => $product_id, 'cart_item_key' => $cart_item_key ) );
		}

		if ( !empty( $cik ) ) { 

			if ( null !== $opc_price ) {
				WC()->cart->cart_contents[ $cik ]['_price_altered'] = $opc_price;
				WC()->cart->cart_contents[ $cik ]['data']->price_altered = $opc_price;
			} else { 
				unset( WC()->cart->cart_contents[ $cik ]['_price_altered'] );
				unset( WC()->cart->cart_contents[ $cik ]['data']->price_altered );
			}

			if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
				WC()->cart->set_quantity( $cik, $_REQUEST['qty'] );
				WC()->cart->cart_contents[ $cik ]['data']->set_price( $price );
				WC()->cart->cart_contents[ $cik ]['data']->set_regular_price( $price );
				WC()->cart->cart_contents[ $cik ]['data']->set_sale_price( '' );
				WC()->cart->set_session();
			} else {
				WC()->cart->cart_contents[ $cik ]['quantity'] = $_REQUEST['qty'];
				WC()->cart->cart_contents[ $cik ]['data']->price = $price;
				WC()->cart->cart_contents[ $cik ]['data']->regular_price = $price;
				WC()->cart->cart_contents[ $cik ]['data']->sale_price = $price;
			}
			
			
			//ob_start();

			//wc_add_notice( sprintf( __( '%s has been added to your order', 'manual_phone_orders' ), $_product->get_title() ) );

			//wc_print_notices();

			//$messages = ob_get_clean();
			
			$messages = false; 

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'success',
				'messages' => $messages
			);

			// Do not exit, there may be other products to process if we're running an Ajax request
			echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';
			
		} else { 
		
			if ( $passed_validation && self::add_to_cart( $product_id, $quantity ) ) {

				//do_action( 'woocommerce_ajax_added_to_cart', $product_id );

				$cik = false; 
				
				$_product = wc_get_product( $product_id );
				
				foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {

					if ( $values['product_id'] == $product_id ) {
						$cik = $cart_item_key;
						break;
					}
				}
				
				if ( !empty( $cik ) && null !== $price ) {
					WC()->cart->cart_contents[ $cik ]['_price_altered'] = $price;
					WC()->cart->cart_contents[ $cik ]['data']->price_altered = $price;

					if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
						WC()->cart->cart_contents[ $cik ]['data']->set_price( $price );
						WC()->cart->cart_contents[ $cik ]['data']->set_regular_price( $price );
						WC()->cart->cart_contents[ $cik ]['data']->set_sale_price( '' );
						WC()->cart->set_session();
					} else {
						WC()->cart->cart_contents[ $cik ]['data']->price = $price;
						WC()->cart->cart_contents[ $cik ]['data']->regular_price = $price;
						WC()->cart->cart_contents[ $cik ]['data']->sale_price = $price;
					}
				}
				
				ob_start();

				//wc_add_notice( sprintf( __( '%s has been added to your order', 'manual_phone_orders' ), $_product->get_title() ) );

				//wc_print_notices();

				$messages = ob_get_clean();

				// If there was an error adding to the cart, redirect to the product page to show any errors
				$data = array(
					'result' => 'success',
					'messages' => $messages
				);

				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';

			} else {

				ob_start();

				wc_add_notice( __( 'Error adding item to cart', 'manual_phone_orders' ), 'error' );

				wc_print_notices();

				$messages = ob_get_clean();

				// If there was an error adding to the cart, redirect to the product page to show any errors
				$data = array(
					'result' => 'failure',
					'messages' => $messages
				);

				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';
			}

			if ( empty( $args ) )
				self::exit_ajax();
		}
	}

	public static function add_to_cart_grouped( $args = null ) {

		if ( !empty( $args ) )
			$_REQUEST['args'] = $args;

		if ( empty( $_REQUEST['args'] ) )
			self::exit_ajax();

		$args = wp_parse_args( $_REQUEST['args'] );

		if ( empty( $args['qty'] ) )
			self::exit_ajax();

		$product_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $args['add-to-cart'] ) );

		$removed_from_cart = $added_to_cart = array();

		unset( $_REQUEST['qty'] );

		foreach( $args['qty'] as $pid => $qty )
			$_REQUEST['qty'][ $pid ] = $qty;

		if ( ! empty( $_REQUEST['qty'] ) && is_array( $_REQUEST['qty'] ) ) {

			$quantity_set = false;

			foreach ( $_REQUEST['qty'] as $item => $quantity ) {

				if ( isset( $_REQUEST['adjust_qty'] ) && 'true' == $_REQUEST['adjust_qty'] ) {

					$cik = null;

					foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {

						if ( $values['product_id'] == $item ) {
							$cik = $cart_item_key;
							break;
						}

					}

					if ( !empty( $cik ) )
						WC()->cart->set_quantity( $cik, 0 ); // remove so it can be re-added with the new qty

				}

				if ( $quantity <= 0 ) {
					$removed_from_cart[] = $item;
					continue;
				}

				$quantity_set = true;

				// Add to cart validation
				$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $item, $quantity );

				if ( $passed_validation ) {
					if ( self::add_to_cart( $item, $quantity ) ) {
						$was_added_to_cart = true;
						$added_to_cart[] = $item;
					}
				}
			}

			$messages = '';

			/*
			if ( $was_added_to_cart && !empty( $added_to_cart ) ) {

				ob_start();

				foreach( $added_to_cart as $pid ) {

					//$_product = wc_get_product( $pid );

					//wc_add_notice( sprintf( __( '%s has been added to your order', 'manual_phone_orders' ), $_product->get_title() ) );

				}

				//wc_print_notices();

				$messages .= ob_get_clean();

			}
			*/

			if ( !empty( $removed_from_cart ) ) {

				ob_start();

				foreach( $removed_from_cart as $pid ) {

					$_product = wc_get_product( $pid );

					wc_add_notice( sprintf( __( '%s has been removed from your order', 'manual_phone_orders' ), $_product->get_title() ) );

				}

				wc_print_notices();

				$messages .= ob_get_clean();

			}

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'success',
				'messages' => $messages
			);

			echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';

			if ( empty( $args ) )
				die;

			/*
			if ( ! $was_added_to_cart && ! $quantity_set ) {

				ob_start();

				wc_add_notice( __( 'Please choose the quantity of items you wish to add to your cart&hellip;', 'manual_phone_orders' ), 'error' );

				wc_print_notices();

				$messages = ob_get_clean();

				// If there was an error adding to the cart, redirect to the product page to show any errors
				$data = array(
					'result' => 'failure',
					'messages' => $messages
				

				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';

				self::exit_ajax();
			}
			*/

		} elseif ( $product_id ) {

			ob_start();

			wc_add_notice( __( 'Please choose a product to add to your cart&hellip;', 'manual_phone_orders' ), 'error' );

			wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'failure',
				'messages' => $messages
			);

			echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';

			if ( empty( $args ) )
				self::exit_ajax();
		}

	}

	// Based on WC core - REQUIRES 2.4.6 or newer
	// $args is passed only when using the "Searchable" page template style
	public static function add_variation_to_cart( $args = null ) {
		$was_added_to_cart = false;

		$added_to_cart = array();

		if ( !empty( $args ) )
			$_REQUEST = $args;

		if ( isset( $_REQUEST['args'] ) ) {

			$args = wp_parse_args( $_REQUEST['args'] );

			unset( $args['add-to-cart'] );

			$quantity = empty( $args['qty'] ) ? 0 : apply_filters( 'woocommerce_stock_amount', $args['qty'] );

			unset( $args['qty'] );

			$product_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $args['product_id'] ) );

			unset( $args['product_id'] );

			$adding_to_cart = wc_get_product( $product_id );

			//$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

			$variation_id = empty( $args['variation_id'] ) ? '' : absint( $args['variation_id'] );

			unset( $args['variation_id'] );

			// Put the remaining args, which should be only attribute selections, in the $_REQUEST array for processing
			foreach( $args as $k => $d )
				$_REQUEST[ $k ] = $d;

		} else {

			$product_id = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_REQUEST['product_id'] ) );

			$adding_to_cart = wc_get_product( $product_id );

			$quantity = empty( $_REQUEST['qty'] ) ? 0 : apply_filters( 'woocommerce_stock_amount', $_REQUEST['qty'] );

			//$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

			$variation_id = empty( $_REQUEST['variation_id'] ) ? '' : absint( $_REQUEST['variation_id'] );

			$variation_data = empty( $_REQUEST['variation_data'] ) ? '' : $_REQUEST['variation_data'];

			if ( !empty( $variation_data ) ) {

				$data = wp_parse_args( $variation_data );

				foreach( $data as $k => $d ) {
					$_REQUEST[ $k ] = $d;
				}
			}
		}

		if ( !empty( $_REQUEST['cart_item_key'] ) ) { 
			$cik = $_REQUEST['cart_item_key'];
		}  else { 
			$cik = self::is_item_in_cart( array( 'product_id' => $_REQUEST['product_id'], 'variation_id' => $_REQUEST['variation_id'] ) );
		}

		// If the item is in the cart then adjust the values
		if ( !empty( $cik ) ) { 
			if ( isset( $_REQUEST['adjust_qty'] ) && 'true' == $_REQUEST['adjust_qty'] ) {
			
				if ( isset( $_REQUEST['price'] ) && '' !== $_REQUEST['price'] ) { 
					$price = self::standardize_number( $_REQUEST['price'] );
				}
				
				if ( null !== $price ) {
					WC()->cart->cart_contents[ $cik ]['_price_altered'] = $price;
					@WC()->cart->cart_contents[ $cik ]['data']->price_altered = $price;
					
					if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
						WC()->cart->set_quantity( $cik, $_REQUEST['qty'] );
						WC()->cart->cart_contents[ $cik ]['data']->set_price( $price );
						WC()->cart->cart_contents[ $cik ]['data']->set_regular_price( $price );
						WC()->cart->cart_contents[ $cik ]['data']->set_sale_price( '' );
						WC()->cart->set_session();
					} else {
						WC()->cart->cart_contents[ $cik ]['quantity'] = $_REQUEST['qty'];
						WC()->cart->cart_contents[ $cik ]['data']->price = $price;
						WC()->cart->cart_contents[ $cik ]['data']->regular_price = $price;
						WC()->cart->cart_contents[ $cik ]['data']->sale_price = $price;
					}
				}
			}
			
			if ( empty( $args ) ) {
				self::exit_ajax( '1' );
			} 
			
		// Not in the cart, so add it
		} else { 
			
			try {
				$variation_id       = empty( $_REQUEST['variation_id'] ) ? '' : absint( $_REQUEST['variation_id'] );
				$quantity           = empty( $_REQUEST['qty'] ) ? 1 : $_REQUEST['qty'];
				$missing_attributes = array();
				$variations         = array();
				$adding_to_cart     = wc_get_product( $product_id );

				if ( ! $adding_to_cart ) {
					return false;
				}

				// If the $product_id was in fact a variation ID, update the variables.
				if ( $adding_to_cart->is_type( 'variation' ) ) {
					$variation_id   = $product_id;
					$product_id     = $adding_to_cart->get_parent_id();
					$adding_to_cart = wc_get_product( $product_id );

					if ( ! $adding_to_cart ) {
						return false;
					}
				}

				// If no variation ID is set, attempt to get a variation ID from posted attributes.
				if ( empty( $variation_id ) ) {
					$data_store   = WC_Data_Store::load( 'product' );
					$variation_id = $data_store->find_matching_product_variation( $adding_to_cart, array_map( 'sanitize_title', wp_unslash( $_REQUEST ) ) );
				}

				// Validate the attributes.
				if ( empty( $variation_id ) ) {
					throw new Exception( __( 'Please choose product options&hellip;', 'manual_phone_orders' ) );
				}

				$variation_data = wc_get_product_variation_attributes( $variation_id );

				foreach ( $adding_to_cart->get_attributes() as $attribute ) {
					if ( ! $attribute['is_variation'] ) {
						continue;
					}

					if ( version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) )
						$taxonomy = sanitize_title( $attribute->get_name() );
					else 
						$taxonomy = sanitize_title( $attribute['name'] );

					$taxonomy = 'attribute_' . sanitize_title( $taxonomy );

					if ( isset( $variation_data[ $taxonomy ] ) ) {
					
						if ( $attribute['is_taxonomy'] ) {
							// Don't use wc_clean as it destroys sanitized characters.
							$value = sanitize_title( wp_unslash( $variation_data[ $taxonomy ] ) );
						} else {
							$value = wc_clean( wp_unslash( $variation_data[ $taxonomy ] ) );
						}

						// Get valid value from variation data.
						$valid_value = isset( $variation_data[ $taxonomy ] ) ? $variation_data[ $taxonomy ] : '';

						// Allow if valid or show error.
						if ( $valid_value === $value ) {
							$variations[ $taxonomy ] = $value;
						} elseif ( '' === $valid_value && in_array( $value, $attribute->get_slugs() ) ) {
							// If valid values are empty, this is an 'any' variation so get all possible values.
							$variations[ $taxonomy ] = $value;
						} else {
							throw new Exception( sprintf( __( 'Invalid value posted for %s', 'manual_phone_orders' ), wc_attribute_label( $attribute['name'] ) ) );
						}
					} else {
						$missing_attributes[] = wc_attribute_label( $attribute['name'] );
					}
				}

				if ( ! empty( $missing_attributes ) ) {
					throw new Exception( sprintf( _n( '%s is a required field', '%s are required fields', count( $missing_attributes ), 'manual_phone_orders' ), wc_format_list_of_items( $missing_attributes ) ) );
				}
			} catch ( Exception $e ) {
				wc_add_notice( $e->getMessage(), 'error' );
				return false;
			}
			
			
			$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity, $variation_id, $variations );

			
			
			if ( $quantity <= 0 ) {

				$_product = wc_get_product( $product_id );

				ob_start();
				wc_add_notice( sprintf( __( '%s has been removed from your order', 'manual_phone_orders' ), $_product->get_title() ) );
				wc_print_notices();

				$messages = ob_get_clean();

				// If there was an error adding to the cart, redirect to the product page to show any errors
				$data = array(
					'result' => 'failure',
					'messages' => $messages
				);

				echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';

				self::exit_ajax();
			}

			if ( $passed_validation && false !== self::add_to_cart( $product_id, $quantity, $variation_id, $variations ) ) {
				//wc_add_to_cart_message( array( $product_id => $quantity ), true );
			}

			if ( isset( $_REQUEST['adjust_qty'] ) && 'true' == $_REQUEST['adjust_qty'] ) {
			
				if ( isset( $_REQUEST['price'] ) )
					$price = self::standardize_number( $_REQUEST['price'] );

				if ( null !== $price ) {
					WC()->cart->cart_contents[ $cik ]['_price_altered'] = $price;
					@WC()->cart->cart_contents[ $cik ]['data']->price_altered = $price;
					if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
						WC()->cart->cart_contents[ $cik ]['data']->set_price( $price );
						WC()->cart->cart_contents[ $cik ]['data']->set_regular_price( $price );
						WC()->cart->cart_contents[ $cik ]['data']->set_sale_price( '' );
						WC()->cart->set_session();
					} else {
						WC()->cart->cart_contents[ $cik ]['data']->price = $price;
						WC()->cart->cart_contents[ $cik ]['data']->regular_price = $price;
						WC()->cart->cart_contents[ $cik ]['data']->sale_price = $price;
					}
				}
			}
			
			if ( empty( $args ) ) {
				self::exit_ajax( '1' );
			} 
		}
	}

	public static function coupon_is_valid( $valid, $coupon, $wc_discounts = null ) {
		global $ign_opc; 
		
		if ( empty( $ign_opc ) ) { 
			return $valid;
		}
		
		// This is Ok because switching checks for a specific request var before adjusting 
		// the user account. 
		$ign_opc->switch_user_context_to_customer( 'customer' );
		
		return $valid; 
	}
}
