<?php
/**
Copyright (c) 2014 - 2019 - IgniteWoo.com - All Rights Reserved
*/
class IGN_OPC_Checkout { 

	/**
	 * Load the thankyou.php template file to alter order customer ID checking added in WC 7.8.1 in the "order_received" function.
	 * Required because WC now checks to the see if the logged in user's ID matches the customer ID assigned to the order when 
	 * rendering the "Thank you" page.
	 */
	public static function maybe_redirect_thankyou_page( $order_id ) { 
		if ( empty( $_GET['manual_phone_order'] ) ) {
			return $order_id;
		}

		if ( ! is_user_logged_in() ) { 
			return $order_id;
		}
		
		// Add a filter to override the order's customer ID so that it can be spoofed to the current logged in user ID
		// and thus allow WC to display the order details. This became a hurdle in WC 7.8.1.
		add_filter( 'woocommerce_order_get_customer_id', __CLASS__ . '::override_order_customer_id', 10, 2 );
		
		return $order_id;
	}
	
	public static function override_order_customer_id( $value, $order ) {
		if ( ! is_user_logged_in() ) { 
			return $value;
		}
		return get_current_user_id();
	}
	
	// Based on WC core code
	public static function process_checkout() {
		global $ign_opc;

		$woocommerce_checkout = WC()->checkout();

		try {

			if ( ! defined( 'WOOCOMMERCE_CHECKOUT' ) ) {
				define( 'WOOCOMMERCE_CHECKOUT', true );
			}

			// Prevent timeout
			@set_time_limit(0);

			do_action( 'woocommerce_before_checkout_process' );

			if ( 0 === sizeof( WC()->cart->get_cart() ) ) {
				throw new Exception( sprintf( __( 'Sorry, your session has expired. <a href="%s" class="wc-backward">Return to homepage</a>', 'manual_phone_orders' ), home_url() ) );
			}

			do_action( 'woocommerce_checkout_process' );

			// Checkout fields (not defined in checkout_fields)
			$_POST['terms']                     = isset( $_POST['terms'] ) ? 1 : 0;
			$_POST['createaccount']             = isset( $_POST['createaccount'] ) && ! empty( $_POST['createaccount'] ) ? 1 : 0;
			$_POST['payment_method']            = isset( $_POST['payment_method'] ) ? stripslashes( $_POST['payment_method'] ) : '';
			$_POST['shipping_method']           = isset( $_POST['shipping_method'] ) ? $_POST['shipping_method'] : '';
			$_POST['ship_to_different_address'] = isset( $_POST['ship_to_different_address'] ) ? true : false;

			$username = isset( $_POST['account_username'] ) ? $_POST['account_username'] : '';
			$password = isset( $_POST['account_password'] ) ? $_POST['account_password'] : '';

			
			if ( isset( $_POST['shiptobilling'] ) ) {
				_deprecated_argument( 'WC_Checkout::process_checkout()', '2.1', 'The "shiptobilling" field is deprecated. The template files are out of date' );

				$_POST['ship_to_different_address'] = $_POST['shiptobilling'] ? false : true;
			}

			// Update customer shipping and payment method to posted method
			$chosen_shipping_methods = WC()->session->get( 'chosen_shipping_methods' );

			if ( isset( $_POST['shipping_method'] ) && is_array( $_POST['shipping_method'] ) ) {
				foreach ( $_POST['shipping_method'] as $i => $value ) {
					$chosen_shipping_methods[ $i ] = wc_clean( $value );
				}
			}

			WC()->session->set( 'chosen_shipping_methods', $chosen_shipping_methods );
			WC()->session->set( 'chosen_payment_method', $_POST['payment_method'] );

			// Note if we skip shipping
			$skipped_shipping = false;
			// Get posted checkout_fields and do validation
			foreach ( $woocommerce_checkout->checkout_fields as $fieldset_key => $fieldset ) {

				// Shipping needed, but "ship to a different address" is not selected
				if ( $fieldset_key == 'shipping' && ( WC()->cart->needs_shipping() && false == $_POST['ship_to_different_address'] ) ) { 

					foreach( $fieldset as $ship_key => $ship_val ) { 
						$billing_key = str_replace( 'shipping_', 'billing_', $ship_key );
						$_POST[ $ship_key ] = $_POST[ $billing_key ]; 
					}

				// Skip shipping if not needed
				} else if ( $fieldset_key == 'shipping' && ( false == $_POST['ship_to_different_address'] || ! WC()->cart->needs_shipping() ) ) {
					$skipped_shipping = true;
					continue;
				
				}

				// Skip account if not needed
				if ( $fieldset_key == 'account' && ( is_user_logged_in() || ( $woocommerce_checkout->must_create_account == false && empty( $_POST['createaccount'] ) ) ) ) {
					continue;
				}

				foreach ( $fieldset as $key => $field ) {

					if ( !isset( $field['type'] ) || empty( $field['type'] ) ) {
 						$field['type'] = 'text';
						//continue; 
					}

					// Get Value
					switch ( $field['type'] ) {
						case "checkbox" :
							$_POST[ $key ] = isset( $_POST[ $key ] ) ? 1 : 0;
						break;
						case "multiselect" :
							$_POST[ $key ] = isset( $_POST[ $key ] ) ? implode( ', ', array_map( 'wc_clean', $_POST[ $key ] ) ) : '';
						break;
						case "textarea" :
							$_POST[ $key ] = isset( $_POST[ $key ] ) ? wp_strip_all_tags( wp_check_invalid_utf8( stripslashes( $_POST[ $key ] ) ) ) : '';
						break;
						default :
							$_POST[ $key ] = isset( $_POST[ $key ] ) ? ( is_array( $_POST[ $key ] ) ? array_map( 'wc_clean', $_POST[ $key ] ) : wc_clean( $_POST[ $key ] ) ) : '';
						break;
					}

					// Hooks to allow modification of value
					if ( !empty( $field['type'] ) ) { 
						$_POST[ $key ] = apply_filters( 'woocommerce_process_checkout_' . sanitize_title( $field['type'] ) . '_field', $_POST[ $key ] );
					}
					
					$_POST[ $key ] = apply_filters( 'woocommerce_process_checkout_field_' . $key, $_POST[ $key ] );

					// Validation: Required fields
					if ( isset( $field['required'] ) && $field['required'] && empty( $_POST[ $key ] ) ) {
						wc_add_notice( '<strong>' . $field['label'] . '</strong> ' . __( 'is a required field.', 'manual_phone_orders' ), 'error' );
					}

					if ( ! empty( $_POST[ $key ] ) ) {

						// Validation rules
						if ( ! empty( $field['validate'] ) && is_array( $field['validate'] ) ) {
							foreach ( $field['validate'] as $rule ) {
								switch ( $rule ) {
									case 'postcode' :
										$_POST[ $key ] = strtoupper( str_replace( ' ', '', $_POST[ $key ] ) );

										if ( ! WC_Validation::is_postcode( $_POST[ $key ], $_POST[ $fieldset_key . '_country' ] ) ) :
											wc_add_notice( __( 'Please enter a valid postcode/ZIP.', 'manual_phone_orders' ), 'error' );
										else :
											$_POST[ $key ] = wc_format_postcode( $_POST[ $key ], $_POST[ $fieldset_key . '_country' ] );
										endif;
									break;
									case 'phone' :
										$_POST[ $key ] = wc_format_phone_number( $_POST[ $key ] );

										if ( ! WC_Validation::is_phone( $_POST[ $key ] ) )
											wc_add_notice( '<strong>' . $field['label'] . '</strong> ' . __( 'is not a valid phone number.', 'manual_phone_orders' ), 'error' );
									break;
									case 'email' :
										$_POST[ $key ] = strtolower( $_POST[ $key ] );

										if ( ! is_email( $_POST[ $key ] ) )
											wc_add_notice( '<strong>' . $field['label'] . '</strong> ' . __( 'is not a valid email address.', 'manual_phone_orders' ), 'error' );
									break;
									case 'state' :
										// Get valid states
										$valid_states = WC()->countries->get_states( isset( $_POST[ $fieldset_key . '_country' ] ) ? $_POST[ $fieldset_key . '_country' ] : ( 'billing' === $fieldset_key ? WC()->customer->get_country() : WC()->customer->get_shipping_country() ) );

										if ( ! empty( $valid_states ) && is_array( $valid_states ) ) {
											$valid_state_values = array_flip( array_map( 'strtolower', $valid_states ) );

											// Convert value to key if set
											if ( isset( $valid_state_values[ strtolower( $_POST[ $key ] ) ] ) ) {
												 $_POST[ $key ] = $valid_state_values[ strtolower( $_POST[ $key ] ) ];
											}
										}

										// Only validate if the country has specific state options
										if ( ! empty( $valid_states ) && is_array( $valid_states ) && sizeof( $valid_states ) > 0 ) {
											if ( ! in_array( $_POST[ $key ], array_keys( $valid_states ) ) ) {
												wc_add_notice( '<strong>' . $field['label'] . '</strong> ' . __( 'is not valid. Please enter one of the following:', 'manual_phone_orders' ) . ' ' . implode( ', ', $valid_states ), 'error' );
											}
										}
									break;
								}
								
							}
						}
					}
				}
			}

			// Update customer location to posted location so we can correctly check available shipping methods
			if ( isset( $_POST['billing_country'] ) ) {
				WC()->customer->set_billing_country( $_POST['billing_country'] );
			}
			if ( isset( $_POST['billing_state'] ) ) {
				WC()->customer->set_billing_state( $_POST['billing_state'] );
			}
			if ( isset( $_POST['billing_postcode'] ) ) {
				WC()->customer->set_billing_postcode( $_POST['billing_postcode'] );
			}

			// Shipping Information
			if ( ! $skipped_shipping ) {

				// Update customer location to posted location so we can correctly check available shipping methods
				if ( isset( $_POST['shipping_country'] ) ) {
					WC()->customer->set_shipping_country( $_POST['shipping_country'] );
				}
				if ( isset( $_POST['shipping_state'] ) ) {
					WC()->customer->set_shipping_state( $_POST['shipping_state'] );
				}
				if ( isset( $_POST['shipping_postcode'] ) ) {
					WC()->customer->set_shipping_postcode( $_POST['shipping_postcode'] );
				}

			} else {

				// Update customer location to posted location so we can correctly check available shipping methods
				if ( isset( $_POST['billing_country'] ) ) {
					WC()->customer->set_shipping_country( $_POST['billing_country'] );
				}
				if ( isset( $_POST['billing_state'] ) ) {
					WC()->customer->set_shipping_state( $_POST['billing_state'] );
				}
				if ( isset( $_POST['billing_postcode'] ) ) {
					WC()->customer->set_shipping_postcode( $_POST['billing_postcode'] );
				}

			}

			// Update cart totals now we have customer address
			//$ign_opc->switch_user_context_to_customer( 'customer' );

			if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {

				foreach ( WC()->cart->get_cart() as $cart_item_key => $values ) {

					$price = isset( $values['_price_altered'] ) ? $values['_price_altered'] : null;

					if ( null !== $price ) {
						WC()->cart->cart_contents[ $cart_item_key ]['data']->set_price( $price );
						WC()->cart->cart_contents[ $cart_item_key ]['data']->set_regular_price( $price );
						WC()->cart->cart_contents[ $cart_item_key ]['data']->set_sale_price( '' );
						WC()->cart->set_session();
					}
				}
			}

			// Avatax "verify address" button has already calculated taxes into the total
			// so do not calc totals again because that reverts taxes to the non-validated address amounts
			//if ( true !== WC()->session->get( 'wc_avatax_address_validated' ) ) {
				WC()->cart->calculate_totals();
			//}

			//$ign_opc->switch_user_context_to_customer( 'opc_user' );

			// Terms
			if ( ! isset( $_POST['woocommerce_checkout_update_totals'] ) && empty( $_POST['terms'] ) && apply_filters( 'woocommerce_checkout_show_terms', wc_get_page_id( 'terms' ) > 0 ) ) {
				wc_add_notice( __( 'You must accept our Terms &amp; Conditions.', 'manual_phone_orders' ), 'error' );
			}

			if ( WC()->cart->needs_shipping() ) {

				if ( ! in_array( WC()->customer->get_shipping_country(), array_keys( WC()->countries->get_shipping_countries() ) ) ) {
					wc_add_notice( sprintf( __( 'Unfortunately <strong>we do not ship %s</strong>. Please enter an alternative shipping address.', 'manual_phone_orders' ), WC()->countries->shipping_to_prefix() . ' ' . WC()->customer->get_shipping_country() ), 'error' );
				}

				// Validate Shipping Methods
				$packages               = WC()->shipping->get_packages();
				$woocommerce_checkout->shipping_methods = WC()->session->get( 'chosen_shipping_methods' );

				foreach ( $packages as $i => $package ) {

					if ( ! isset( $package['rates'][ $woocommerce_checkout->shipping_methods[ $i ] ] ) ) {
						wc_add_notice( __( 'No shipping method has been selected. Please double check your address, or contact us if you need any help.', 'manual_phone_orders' ), 'error' );
						$woocommerce_checkout->shipping_methods[ $i ] = '';
					}
				}
			}

			if ( WC()->cart->needs_payment() ) {
				// Payment Method
				$available_gateways = WC()->payment_gateways->get_available_payment_gateways();

				if ( ! isset( $available_gateways[ $_POST['payment_method'] ] ) ) {
					$woocommerce_checkout->order_payment_method = '';
					wc_add_notice( __( 'Invalid payment method.', 'manual_phone_orders' ), 'error' );
				} else {
					// IGNITEWOO MOD TO OVERRIDE PRIVATE VAR

					$method = isset( $_POST['payment_method'] ) ? $_POST['payment_method'] : '';
					$woocommerce_checkout->order_payment_method = $available_gateways[ $method ];

					if ( method_exists( $available_gateways[ $method ], 'validate_fields' ) )
						$available_gateways[ $method ]->validate_fields();
				}
			} else {
				$available_gateways = array();
			}

			// Action after validation
			do_action( 'woocommerce_after_checkout_validation', $_POST, false );

			$new_customer = false; 
			
			if ( ! isset( $_POST['woocommerce_checkout_update_totals'] ) && wc_notice_count( 'error' ) == 0 ) {

				// IGNITEWOO MODS IN THIS "IF" WRAPPER AREA - Customer accounts

				// CUSTOM CUSTOMER_ID VAR SINCE IT IS PRIVATE IN THE CLASS
				$woocommerce_checkout->order_customer_id = isset( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ? $_REQUEST['ignitewoo_one_page_checkout_customer_id'] : null;

				$woocommerce_checkout->order_customer_id = apply_filters( 'woocommerce_checkout_customer_id', $woocommerce_checkout->order_customer_id );

				//$woocommerce_checkout->must_create_account = $woocommerce_checkout->enable_guest_checkout ? false : true;

				//if ( ! is_user_logged_in() && ( $woocommerce_checkout->must_create_account || ! empty( $_POST['createaccount'] ) ) ) {

				if ( ( empty( $_POST['ignitewoo_one_page_checkout_customer_id'] ) && $woocommerce_checkout->is_registration_required() ) || ( ( !empty( $_POST['createaccount'] ) && empty( $_POST['ignitewoo_one_page_checkout_customer_id'] ) ) ) ) { 
				//if ( ( $woocommerce_checkout->must_create_account || ( !empty( $_POST['createaccount'] ) ) && empty( $woocommerce_checkout->order_customer_id ) ) ) {

					$new_customer = wc_create_new_customer( $_POST['billing_email'], $username, $password );

					if ( is_wp_error( $new_customer ) ) {
						throw new Exception( $new_customer->get_error_message() );
					}

					$woocommerce_checkout->order_customer_id = $new_customer;

					$_REQUEST['ignitewoo_one_page_checkout_customer_id'] = $new_customer;

					// wc_set_customer_auth_cookie( $woocommerce_checkout->customer_id );

					// As we are now logged in, checkout will need to refresh to show logged in data
					// WC()->session->set( 'reload_checkout', true );

					// Also, recalculate cart totals to reveal any role-based discounts that were unavailable before registering

					//$ign_opc->switch_user_context_to_customer( 'customer' );
					
					// Avatax "verify address" button has already calculated taxes into the total
					// so do not calc totals again because that reverts taxes to the non-validated address amounts
					//if ( true !== WC()->session->get( 'wc_avatax_address_validated' ) ) {
						WC()->cart->calculate_totals();
					//}

					// Add customer info from other billing fields

					if ( $_POST['billing_first_name'] && apply_filters( 'woocommerce_checkout_update_customer_data', true, $woocommerce_checkout ) ) {
						$userdata = array(
							'ID'           => $woocommerce_checkout->order_customer_id,
							'first_name'   => !empty( $_POST['billing_first_name'] ) ? $_POST['billing_first_name'] : '',
							'last_name'    => !empty( $_POST['billing_last_name'] ) ? $_POST['billing_last_name'] : '',
							'display_name' => !empty( $_POST['billing_first_name'] ) ? $_POST['billing_first_name'] : ''
						);
						wp_update_user( apply_filters( 'woocommerce_checkout_customer_userdata', $userdata, $woocommerce_checkout ) );
					}

					// $ign_opc->switch_user_context_to_customer( 'opc_user' );

				}

				// Do a final stock check at this point
				// This function calls an action "woocommerce_check_cart_items" which in turn
				// causes the cart class to check stock, availability, and coupons.
				// THIS PLUGIN doesn't need to do that, because it allows admin to add whatever they want to the cart
				// exactly as if manually creating an order in the admin area.
				// $woocommerce_checkout->check_cart_items();

				// Abort if errors are present
				if ( wc_notice_count( 'error' ) > 0 )
					throw new Exception();

				if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>' ) ) {

					//if ( !empty( $woocommerce_checkout->order_customer_id ) )
					//	$_POST['ignitewoo_one_page_checkout_customer_id'] = $woocommerce_checkout->order_customer_id;

					// Update customer address info, etc, in their account
					self::process_customer( $_POST );

					$order_id = self::create_order_gt_27( $_POST );

					if ( is_wp_error( $order_id ) ) {
						throw new Exception( $order->get_error_message() );
					}

					// No account being created, no customer assigned to the order, registration not required,
					// so set the customer ID to 0.
					if ( empty( $data['create_account'] ) && !$woocommerce_checkout->is_registration_required() && empty( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ) {
						$o = wc_get_order( $order_id );
						$o->set_customer_id( 0 );
						$o->save();
					}

					//do_action( 'woocommerce_checkout_order_processed', $order_id, $posted_data, $order );

				} else {

					$order_id = self::create_order( null, $woocommerce_checkout );

				}

				
				if ( empty( $order ) && !empty( $order_id ) ) {
					$order = wc_get_order( $order_id );
				}
				
				// Deprecated: This is now done via a "new order" hook along with a metabox to display it in the order.
				// IGNITEWOO MOD -- save user who is submitting this order to the order post meta data
				global $current_user;
				$order->update_meta_data( 'store_clerk', '( #' . $current_user->data->ID . ' ) - ' . $current_user->data->user_login );
				$order->update_meta_data( '_store_clerk_id', $current_user->data->ID );

				if ( is_wp_error( $order_id ) ) {
					throw new Exception( $order_id->get_error_message() );
				}

				do_action( 'woocommerce_checkout_order_processed', $order_id, $_POST, $order );

				// IGNITEWOO MOD - unset this
				unset( WC()->session->order_awaiting_payment );

				// Process payment
				if ( WC()->cart->needs_payment() ) {

					// Store Order ID in session so it can be re-used after payment failure
					WC()->session->order_awaiting_payment = $order_id;

					// Switch to customer user so that stored card can be used if a payment gateway supports that feature
					//$ign_opc->switch_user_context_to_customer( 'customer' );

					// Process Payment
					$result = $available_gateways[ $_POST['payment_method'] ]->process_payment( $order_id );

					// Switch back to OPC user now that payment has been processed to whatever extent
					// $ign_opc->switch_user_context_to_customer( 'opc_user' );

					// Redirect to success/confirmation/payment page
					if ( $result['result'] == 'success' ) {

						WC()->session->set( 'phone_manual_order_loaded_id', '' );
					
						$result = apply_filters( 'woocommerce_payment_successful_result', $result, $order_id );
						
						$result['redirect'] = add_query_arg('manual_phone_order', 'true', $result['redirect'] );

						if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {
							echo '<!--WC_START-->' . json_encode( $result ) . '<!--WC_END-->';
							exit;
						} else if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '>=' ) ) {
							echo json_encode( $result );
							exit;
						} else {
							wp_redirect( $result['redirect'] );
							exit;
						}
					}

				} else {

					if ( empty( $order ) ) {
						$order = wc_get_order( $order_id );
					}

					// No payment was required for order
					$order->payment_complete();

					// Empty the Cart
					WC()->cart->empty_cart();
					
					WC()->session->set( 'phone_manual_order_loaded_id', '' );

					// Get redirect
					$return_url = $order->get_checkout_order_received_url();
					
					$return_url = add_query_arg('manual_phone_order', 'true', $return_url );
					
					// Redirect to success/confirmation/payment page
					if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {
						echo '<!--WC_START-->' . json_encode(
							array(
								'result' 	=> 'success',
								'redirect'  => apply_filters( 'woocommerce_checkout_no_payment_needed_redirect', $return_url, $order )
							)
						) . '<!--WC_END-->';
						exit;
					} else if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '>=' ) ) {
						echo json_encode(
							array(
								'result' 	=> 'success',
								'redirect'  => apply_filters( 'woocommerce_checkout_no_payment_needed_redirect', $return_url, $order )
							)
						);
						exit;
					} else {
						wp_safe_redirect(
							apply_filters( 'woocommerce_checkout_no_payment_needed_redirect', $return_url, $order )
						);
						exit;
					}

				}

			}

		} catch ( Exception $e ) {
			if ( ! empty( $e ) ) {
				wc_add_notice( $e->getMessage(), 'error' );
			}
		}

		// If we reached this point then there were errors
		if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '<' ) ) {

			// only print notices if not reloading the checkout, otherwise they're lost in the page reload
			if ( ! isset( WC()->session->reload_checkout ) ) {
				ob_start();
				wc_print_notices();
				$messages = ob_get_clean();
			}

			echo '<!--WC_START-->' . json_encode(
				array(
					'result'	=> 'failure',
					'messages' 	=> isset( $messages ) ? $messages : '',
					'refresh' 	=> isset( WC()->session->refresh_totals ) ? 'true' : 'false',
					'reload'    => isset( WC()->session->reload_checkout ) ? 'true' : 'false'
				)
			) . '<!--WC_END-->';

			unset( WC()->session->refresh_totals, WC()->session->reload_checkout );
			exit;

		} else if ( wp_doing_ajax() && version_compare( WOOCOMMERCE_VERSION, '2.4', '>=' ) ) {

			// only print notices if not reloading the checkout, otherwise they're lost in the page reload
			if ( ! isset( WC()->session->reload_checkout ) ) {
				ob_start();
				wc_print_notices();
				$messages = ob_get_clean();
			}

			echo json_encode(
				array(
					'result'	=> 'failure',
					'messages' 	=> isset( $messages ) ? $messages : '',
					'refresh' 	=> isset( WC()->session->refresh_totals ) ? 'true' : 'false',
					'reload'    => isset( WC()->session->reload_checkout ) ? 'true' : 'false'
				)
			);

			unset( WC()->session->refresh_totals, WC()->session->reload_checkout );
			exit;
		}
	}
	
	private static function process_customer( $data ) {
		global $ign_opc, $user_ID;

		// Do not process any customer data unless a customer is assigned to the order? 
		// This var will be empty for guest checkout, but not empty if the box to create a new customer is checked 
		// since the var is set (in process_checkout) after the new customer account is created.
		if ( empty( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ) { 
			return; 
		}

		$checkout = WC()->checkout();

		/*
		if ( !empty( $data['createaccount'] ) ) {
			$username    = ! empty( $data['account_username'] ) ? $data['account_username'] : '';
			$password    = ! empty( $data['account_password'] ) ? $data['account_password'] : '';
			$customer_id = wc_create_new_customer( $data['billing_email'], $username, $password );

			if ( is_wp_error( $customer_id ) ) {
				throw new Exception( $customer_id->get_error_message() );
			}

			wp_set_current_user( $customer_id );
			wc_set_customer_auth_cookie( $customer_id );

			// As we are now logged in, checkout will need to refresh to show logged in data
			WC()->session->set( 'reload_checkout', true );

			// Also, recalculate cart totals to reveal any role-based discounts that were unavailable before registering
			WC()->cart->calculate_totals();
		}
		*/
		
		// if registration is not required, and the checkbox to create an account is not checked,
		// and there's no customer assigned to the order, then return to avoid overwriting the logged
		// in user's account info
		//if ( empty( $data['create_account'] ) && !$checkout->is_registration_required() && empty( $_POST['ignitewoo_one_page_checkout_customer_id'] ) ) {
		//	return;
		//}

		$ign_opc->switch_user_context_to_customer( 'customer' );

		if ( 0 == $user_ID ) {
			return;
		}

		$customer_id = get_current_user_id();

		// Add customer info from other fields. but do not call the filter because this is a singleton class and filters might expect an object
		if ( $customer_id /*&& apply_filters( 'woocommerce_checkout_update_customer_data', true, self )*/ ) {
			$customer = new WC_Customer( $customer_id );

			if ( ! empty( $data['billing_first_name'] ) ) {
				$customer->set_first_name( $data['billing_first_name'] );
			}

			if ( ! empty( $data['billing_last_name'] ) ) {
				$customer->set_last_name( $data['billing_last_name'] );
			}

			foreach ( $data as $key => $value ) {
				// Use setters where available.
				if ( is_callable( array( $customer, "set_{$key}" ) ) ) {
					$customer->{"set_{$key}"}( $value );

				// Store custom fields prefixed with wither shipping_ or billing_.
				} elseif ( 0 === stripos( $key, 'billing_' ) || 0 === stripos( $key, 'shipping_' ) ) {
					$customer->update_meta_data( $key, $value );
				}
			}

			// Do this because when an email address is numeric (767@asdfasdf.com) it seems that WC may get confused 
			// and set a customer ID of 767 causing data to be written to the wrong customer account
			$customer->set_id( $customer_id );
			
			/**
			 * Action hook to adjust customer before save.
			 * @since 2.7.0
			 */
			do_action( 'woocommerce_checkout_update_customer', $customer, $data );
			
			$customer->save();
		}

		do_action( 'woocommerce_checkout_update_user_meta', $customer_id, $data );

		//$ign_opc->switch_user_context_to_customer( 'opc_user' );
	}

	// Based on WC core code
	private static function create_order_gt_27( $data ) {
		global $ign_opc, $ign_opc_core;

		//if ( !$ign_opc_core->has_access() )
		//	return;

		// Disallow other plugins from intercepting the order to create their own
		remove_all_filters( 'woocommerce_create_order' );
		//add_action( 'woocommerce_create_order', array( $ign_opc_core, 'maybe_alter_order_status' ), 2, -1 );

		// Give plugins the opportunity to create an order themselves.
		if ( $order_id = apply_filters( 'woocommerce_create_order', null, $ign_opc_core ) ) {
			return $order_id;
		}

		try {
			$order_id           = absint( WC()->session->get( 'order_awaiting_payment' ) );
			$cart_hash          = md5( json_encode( wc_clean( WC()->cart->get_cart_for_session() ) ) . WC()->cart->total );
			$available_gateways = WC()->payment_gateways->get_available_payment_gateways();

			// IgniteWoo mod: Switch to opc user
			//$ign_opc->switch_user_context_to_customer( 'customer' );

			/**
			 * If there is an order pending payment, we can resume it here so
			 * long as it has not changed. If the order has changed, i.e.
			 * different items or cost, create a new order. We use a hash to
			 * detect changes which is based on cart items + order total.
			 */
			 
			 // Disable cart hash checking since the cart contents etc might have been modified
			if ( $order_id && ( $order = wc_get_order( $order_id ) ) /*&& $order->has_cart_hash( $cart_hash )*/ && $order->has_status( array( 'pending', 'failed' ) ) ) {
				// Action for 3rd parties.
				do_action( 'woocommerce_resume_order', $order_id );

				// Remove all items - we will re-add them later.
				$order->remove_order_items();
			} else {
				$order = new WC_Order();
			}

			foreach ( $data as $key => $value ) {
				if ( is_callable( array( $order, "set_{$key}" ) ) ) {
					$order->{"set_{$key}"}( $value );

				// Store custom fields prefixed with wither shipping_ or billing_. This is for backwards compatibility with 2.6.x.
				// TODO: Fix conditional to only include shipping/billing address fields in a smarter way without str(i)pos.
				} elseif ( ( 0 === stripos( $key, 'billing_' ) || 0 === stripos( $key, 'shipping_' ) )
					&& ! in_array( $key, array( 'shipping_method', 'shipping_total', 'shipping_tax' ) ) ) {
					$order->update_meta_data( '_' . $key, $value );
				}
			}

			$order->set_created_via( 'checkout' );
			$order->set_cart_hash( $cart_hash );
			$order->set_customer_id( apply_filters( 'woocommerce_checkout_customer_id', get_current_user_id() ) );
			$order->set_currency( get_woocommerce_currency() );
			$order->set_prices_include_tax( 'yes' === get_option( 'woocommerce_prices_include_tax' ) );
			$order->set_customer_ip_address( WC_Geolocation::get_ip_address() );
			$order->set_customer_user_agent( wc_get_user_agent() );
			$order->set_customer_note( isset( $data['order_comments'] ) ? $data['order_comments'] : '' );
			$order->set_payment_method( isset( $available_gateways[ $data['payment_method'] ] ) ? $available_gateways[ $data['payment_method'] ]  : $data['payment_method'] );
			
			if ( version_compare( WOOCOMMERCE_VERSION, '3.2', '<' ) ) {
				$order->set_shipping_total( WC()->cart->get_cart_shipping_total() );
				$order->set_discount_total( WC()->cart->get_cart_discount_total() );
				$order->set_discount_tax( WC()->cart->get_cart_discount_tax_total() );
				$order->set_cart_tax( WC()->cart->tax_total );
				$order->set_shipping_tax( WC()->cart->shipping_tax_total );
				$order->set_total( WC()->cart->total );
			} else { 
				$order->set_shipping_total( WC()->cart->get_shipping_total() );	
				$order->set_discount_total( WC()->cart->get_discount_total() );
				$order->set_discount_tax( WC()->cart->get_discount_tax() );
				$order->set_cart_tax( WC()->cart->get_cart_contents_tax() + WC()->cart->get_fee_tax() );
				$order->set_shipping_tax( WC()->cart->get_shipping_tax() );
				$order->set_total( WC()->cart->get_total( 'edit' ) );
			}
						
			$checkout = new WC_Checkout();
			
			$checkout->create_order_line_items( $order, WC()->cart );
			$checkout->create_order_fee_lines( $order, WC()->cart );

			// IgniteWoo mod: Switch to opc user to obtain session vars 
			// $ign_opc->switch_user_context_to_customer( 'opc_user' );
			
			// If Avatax validated the address then the shipping packages are without rates, so get the package with rates
			// otherwise no shipping line item will be added to the order.
			if ( true === WC()->session->get( 'wc_avatax_address_validated' ) ) {
				$packages = WC()->shipping->calculate_shipping( WC()->cart->get_shipping_packages() );
			} else { 
				$packages = WC()->shipping->get_packages();
			}
			
			$checkout->create_order_shipping_lines( $order, WC()->session->get( 'chosen_shipping_methods' ), $packages );

			// IgniteWoo mod: Switch back to custom
			// $ign_opc->switch_user_context_to_customer( 'customer' );

			$checkout->create_order_tax_lines( $order, WC()->cart );
			$checkout->create_order_coupon_lines( $order, WC()->cart );

			// Reset VAT exemption to false, WC saves this in the session data so it must be reset
			// otherwise the next order will use the same exemption setting from the previous order.
			WC()->customer->set_is_vat_exempt( false );
			WC()->session->set( 'is_vat_exempt', false );
			WC()->session->save_data();
			
			/**
			 * Action hook to adjust order before save.
			 * @since 3.0.0
			 */
			do_action( 'woocommerce_checkout_create_order', $order, $data );

			// Save the order.
			$order_id = $order->save();

			do_action( 'woocommerce_checkout_update_order_meta', $order_id, $data );

			//$ign_opc->switch_user_context_to_customer( 'opc_user' );

			return $order_id;
			
		} catch ( Exception $e ) {
			//$ign_opc->switch_user_context_to_customer( 'opc_user' );
			return new WP_Error( 'checkout-error', $e->getMessage() );
		}

	}

	// Based on WC core code
	private static function create_order( $null, $checkout ) {
		global $wpdb, $ign_opc_core;

		//if ( !$ign_opc_core->has_access() )
		//	return;

		try {
			// Start transaction if available
			$wpdb->query( 'START TRANSACTION' );

			$order_data = array(
				'status'        => apply_filters( 'woocommerce_default_order_status', 'pending' ),
				'customer_id'   => $checkout->order_customer_id,
				'customer_note' => isset( $checkout->posted['order_comments'] ) ? $checkout->posted['order_comments'] : ''
			);

			// Insert or update the post data
			$order_id = absint( WC()->session->order_awaiting_payment );

			// Resume the unpaid order if its pending
			if ( $order_id > 0 && ( $order = wc_get_order( $order_id ) ) && $order->has_status( array( 'pending', 'failed' ) ) ) {

				$order_data['order_id'] = $order_id;
				$order                  = wc_update_order( $order_data );

				if ( is_wp_error( $order ) ) {
					throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 401 ) );
				} else {
					$order->remove_order_items();
					do_action( 'woocommerce_resume_order', $order_id );
				}

			} else {

				$order = wc_create_order( $order_data );

				if ( is_wp_error( $order ) ) {
					throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 400 ) );
				} else {

					$order_id = $ign_opc_core->get_order_id( $order );
					do_action( 'woocommerce_new_order', $order_id );
				}

				/*
				// IGNITEWOO MOD - update the post author to be the customer user if set
				if ( !empty( $checkout->order_customer_id ) ) {
					$order_data['post_author'] = $checkout->order_customer_id;

					$order_id = wp_update_post( $order_data );
				}
				*/
			}

			// Store the line items to the new/resumed order
			foreach ( WC()->cart->get_cart() as $cart_item_key => $values ) {
				$item_id = $order->add_product(
					$values['data'],
					$values['quantity'],
					array(
						'variation' => $values['variation'],
						'totals'    => array(
							'subtotal'     => $values['line_subtotal'],
							'subtotal_tax' => $values['line_subtotal_tax'],
							'total'        => $values['line_total'],
							'tax'          => $values['line_tax'],
							'tax_data'     => $values['line_tax_data'] // Since 2.2
						)
					)
				);

				if ( ! $item_id ) {
					throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 402 ) );
				}

				// Allow plugins to add order item meta
				do_action( 'woocommerce_add_order_item_meta', $item_id, $values, $cart_item_key );
			}

			// Stores
			foreach ( WC()->cart->get_fees() as $fee_key => $fee ) {
				$item_id = $order->add_fee( $fee );

				if ( ! $item_id ) {
					throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 403 ) );
				}

				// Allow plugins to add order item meta to fees
				do_action( 'woocommerce_add_order_fee_meta', $order_id, $item_id, $fee, $fee_key );
			}

			// Store shipping for all packages
			foreach ( WC()->shipping->get_packages() as $package_key => $package ) {
				if ( isset( $package['rates'][ $checkout->shipping_methods[ $package_key ] ] ) ) {
					$item_id = $order->add_shipping( $package['rates'][ $checkout->shipping_methods[ $package_key ] ] );

					if ( ! $item_id ) {
						throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 404 ) );
					}

					// Allows plugins to add order item meta to shipping
					do_action( 'woocommerce_add_shipping_order_item', $order_id, $item_id, $package_key );
				}
			}

			// Store tax rows
			foreach ( array_keys( WC()->cart->taxes + WC()->cart->shipping_taxes ) as $tax_rate_id ) {
				if ( $tax_rate_id && ! $order->add_tax( $tax_rate_id, WC()->cart->get_tax_amount( $tax_rate_id ), WC()->cart->get_shipping_tax_amount( $tax_rate_id ) ) && apply_filters( 'woocommerce_cart_remove_taxes_zero_rate_id', 'zero-rated' ) !== $tax_rate_id ) {
					throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 405 ) );
				}
			}

			// Store coupons
			foreach ( WC()->cart->get_coupons() as $code => $coupon ) {
				if ( ! $order->add_coupon( $code, WC()->cart->get_coupon_discount_amount( $code ), WC()->cart->get_coupon_discount_tax_amount( $code ) ) ) {
					throw new Exception( sprintf( __( 'Error %d: Unable to create order. Please try again.', 'manual_phone_orders' ), 406 ) );
				}
			}

			// Billing address
			$billing_address = array();
			if ( $checkout->checkout_fields['billing'] ) {
				foreach ( array_keys( $checkout->checkout_fields['billing'] ) as $field ) {
					$field_name = str_replace( 'billing_', '', $field );
					$billing_address[ $field_name ] = $checkout->get_posted_address_data( $field_name );
				}
			}

			// Shipping address.
			$shipping_address = array();
			if ( $checkout->checkout_fields['shipping'] ) {
				foreach ( array_keys( $checkout->checkout_fields['shipping'] ) as $field ) {
					$field_name = str_replace( 'shipping_', '', $field );
					$shipping_address[ $field_name ] = $checkout->get_posted_address_data( $field_name, 'shipping' );
				}
			}

			$order->set_address( $billing_address, 'billing' );
			$order->set_address( $shipping_address, 'shipping' );
			// IGNITEWOO MOD - USE OUR OWN VAR FOR PAYMENT SINCE IT IS PRIVATE IN THE CLASS AND CANNOT BE SET IN OUR CHECKOUT CODE
			$order->set_payment_method( $checkout->order_payment_method );
			$order->set_total( WC()->cart->shipping_total, 'shipping' );
			$order->set_total( WC()->cart->get_cart_discount_total(), 'cart_discount' );
			$order->set_total( WC()->cart->get_cart_discount_tax_total(), 'cart_discount_tax' );
			$order->set_total( WC()->cart->tax_total, 'tax' );
			$order->set_total( WC()->cart->shipping_tax_total, 'shipping_tax' );
			$order->set_total( WC()->cart->total );

			// Update user meta
			if ( $checkout->order_customer_id ) {
				if ( apply_filters( 'woocommerce_checkout_update_customer_data', true, $checkout ) ) {
					foreach ( $billing_address as $key => $value ) {
						update_user_meta( $checkout->order_customer_id, 'billing_' . $key, $value );
					}
					if ( WC()->cart->needs_shipping() ) {
						foreach ( $shipping_address as $key => $value ) {
							update_user_meta( $checkout->order_customer_id, 'shipping_' . $key, $value );
						}
					}
				}
				do_action( 'woocommerce_checkout_update_user_meta', $checkout->order_customer_id, $checkout->posted );
			}

			// Let plugins add meta
			do_action( 'woocommerce_checkout_update_order_meta', $order_id, $checkout->posted );

			// If we got here, the order was created without problems!
			$wpdb->query( 'COMMIT' );

		} catch ( Exception $e ) {
			// There was an error adding order data!
			$wpdb->query( 'ROLLBACK' );
			return new WP_Error( 'checkout-error', $e->getMessage() );
		}

		return $order_id;
	}

}
