<?php
/**
Copyright (c) 2014 - 2021 - IgniteWoo.com - All Rights Reserved
*/

class IGN_Manual_Phone_Orders_Core {

	public $opc = false;

	public $style = '';

	public $products = array();

	public $settings = false;

	function __construct() {
	
		// For debugging: 
		//@ini_set( 'display_errors', true );
		//@ini_set( 'error_reporting', E_ALL  );

		$this->shortcode = 'woocommerce_manual_phone_order';

		add_action( 'init', array( &$this, 'remove_hooks' ), 10 );

		add_action( 'init', array( &$this, 'maybe_set_user' ), -999 );
		
		// For ajax requests
		add_action( 'admin_init', array( &$this, 'maybe_set_user' ), 1 );
		
		add_action( 'shutdown', array( &$this, 'shutdown_maybe_set_user' ), 1 );
		
		add_action( 'init', array( &$this, 'init' ), 5 );

		add_action( 'wp_loaded', array( &$this, 'adjust_hooks' ), 5 );

		add_shortcode( $this->shortcode, array( &$this, 'one_page' ), 1 ,1 );
		
		add_action( 'template_redirect', array( $this, 'global_checkout_template_redirect' ), -99 );
		
		add_filter( 'woocommerce_thankyou_order_id', 'IGN_OPC_Checkout::maybe_redirect_thankyou_page', 1 );

	}

	public function global_checkout_template_redirect() { 

		// Remove all actions for "template_redirect" to prevent 3rd party plugins such 
		// as from CartFlows from intercepting [due to a call to is_checkout() ] and redirecting
		// away from the order page
		if ( !empty( $this->settings['manual_order_page'] ) && is_page( $this->settings['manual_order_page'] ) ) { 
			remove_all_actions( 'template_redirect' );
		}
	}
	
	public static function plugin_url() {
		return untrailingslashit( plugins_url( '/', __FILE__ ) );
	}

	public static function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

	public function init() {

		if ( !$this->has_access() )
			return;

		add_action( 'wp', array( &$this, 'check_for_shortcode' ), 1 );

		add_action( 'wp_head', array( &$this, 'no_cache' ), -99 );
		add_action( 'wp_head', array( &$this, 'wp_head' ), 1 );

		add_action( 'init', array( &$this, 'is_tax_exempt' ), 99999999 );
		add_action( 'woocommerce_init', array( &$this, 'is_tax_exempt' ), 99999999 );

		add_action( 'woocommerce_before_checkout_form', array( &$this, 'before_checkout_form' ), 99999999 );

		add_action( 'woocommerce_checkout_order_review', array( &$this, 'checkout_after_order_review' ), 1 );

		//add_action( 'woocommerce_checkout_before_order_review', array( &$this, 'checkout_before_order_review' ), 1 );

		add_action( 'woocommerce_before_checkout_process', array( &$this, 'before_checkout_process' ), -99 );

		//add_filter( 'woocommerce_is_checkout', array( &$this, 'is_checkout' ), 999, 9999999 );
		//add_action( 'wp_enqueue_scripts', array( $this, 'maybe_add_checkout_hook' ), -99 );

		add_filter( 'woocommerce_get_cart_item_from_session', 'IGN_OPC_Cart::get_cart_item_from_session', -99, 3 );
		
		add_action( 'woocommerce_add_to_cart', 'IGN_OPC_Cart::maybe_save_new_price', 1, 5 );

		add_action( 'woocommerce_before_calculate_totals', 'IGN_OPC_Cart::maybe_calculate_new_prices', 1, 1 );

		add_filter( 'woocommerce_coupon_is_valid', 'IGN_OPC_Cart::coupon_is_valid', -9, 3 );
		add_filter( 'woocommerce_coupon_is_valid', 'IGN_OPC_Cart::coupon_is_valid', PHP_INT_MAX, 3 );
		
		if ( version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) ) {
		
			// Ensure we have switched the customer account if assigned 
			add_filter( 'woocommerce_product_get_price', array( &$this, 'maybe_set_user_customer' ), 20, 2 );
			
			add_filter( 'woocommerce_product_get_price', 'IGN_OPC_Cart::maybe_return_price', 9999999, 2 );
			add_filter( 'woocommerce_get_product_price', 'IGN_OPC_Cart::maybe_return_price', 9999, 2 );
			add_filter( 'woocommerce_product_variation_get_price', 'IGN_OPC_Cart::maybe_return_price', 9999, 2 );	
			
			// Ensure we have switched back to the logged in user account if customer is assigned 
			add_filter( 'woocommerce_product_get_price', array( &$this, 'maybe_set_user_admin' ), PHP_INT_MAX, 2 );
			
		} else {
			add_filter( 'woocommerce_get_price', 'IGN_OPC_Cart::maybe_return_price', 9999999, 2 );
		}

		add_action( 'wp_ajax_woocommerce_add_to_cart_opc_variable', 'IGN_OPC_Cart::add_variation_to_cart', 1 );
		add_action( 'wp_ajax_nopriv_woocommerce_add_to_cart_opc_variable', 'IGN_OPC_Cart::add_variation_to_cart', 1 );

		add_action( 'wp_ajax_woocommerce_add_to_cart_opc', 'IGN_OPC_Cart::add_item_to_cart', 1 );
		add_action( 'wp_ajax_nopriv_woocommerce_add_to_cart_opc', 'IGN_OPC_Cart::add_item_to_cart', 1 );
		
		// Updates multiple items in the cart with one Ajax call via the Update the Cart button
		add_action( 'wp_ajax_woocommerce_update_the_cart_opc', 'IGN_OPC_Cart::update_items_to_cart', 1 );
		add_action( 'wp_ajax_nopriv_woocommerce_update_the_cart_opc', 'IGN_OPC_Cart::update_items_to_cart', 1 );
		
		add_action( 'wp_ajax_woocommerce_add_to_cart_opc_donation', 'IGN_OPC_Cart::add_to_cart_donation', 1 );
		add_action( 'wp_ajax_nopriv_woocommerce_add_to_cart_opc_donation', 'IGN_OPC_Cart::add_to_cart_donation', 1 );

		add_action( 'wp_ajax_woocommerce_add_to_cart_opc_gift_cert', 'IGN_OPC_Cart::add_to_cart_gift_cert', 1 );
		add_action( 'wp_ajax_nopriv_woocommerce_add_to_cart_opc_gift_cert', 'IGN_OPC_Cart::add_to_cart_gift_cert', 1 );

		add_action( 'wp_ajax_woocommerce_add_to_cart_opc_product_remove', 'IGN_OPC_Cart::add_to_cart_product_remove', 1 );
		add_action( 'wp_ajax_nopriv_woocommerce_add_to_cart_opc_product_remove', 'IGN_OPC_Cart::add_to_cart_product_remove', 1 );

		//add_action( 'wp_ajax_woocommerce_add_to_cart_opc_product_remove', array( &$this, 'add_to_cart_product_remove' ), 1 );
		//add_action( 'wp_ajax_nopriv_woocommerce_add_to_cart_opc_product_remove', array( &$this, 'add_to_cart_product_remove' ), 1 );

		//add_action( 'wp_ajax_woocommerce_update_order_review', array( &$this, 'update_order_review' ), -1 );
		//add_action( 'wp_ajax_nopriv_update_order_review', array( &$this, 'update_order_review' ), -1 );

		add_action( 'wp_ajax_get_refreshed_fragments', array( &$this, 'get_refreshed_fragments' ), -1 );
		add_action( 'wp_ajax_nopriv_get_refreshed_fragments', array( &$this, 'get_refreshed_fragments' ), -1 );

		add_action( 'wp_ajax_ign_one_page_searchable_add', 'IGN_OPC_Cart::ign_one_page_searchable_add', -1 );
		add_action( 'wp_ajax_nopriv_ign_one_page_searchable_add', 'IGN_OPC_Cart::ign_one_page_searchable_add', -1 );

		add_action( 'wp_ajax_opc_json_search_products_and_variations', array( &$this, 'json_search_products_and_variations' ), -1 );
		add_action( 'wp_ajax_nopriv_opc_json_search_products_and_variations', array( &$this, 'json_search_products_and_variations' ), -1 );

		add_action( 'wp_ajax_opc_add_fee', array( &$this, 'maybe_check_fee' ), -1 );
		add_action( 'wp_ajax_nopriv_opc_add_fee', array( &$this, 'maybe_check_fee' ), -1 );
		
		add_action( 'woocommerce_cart_calculate_fees', array( &$this, 'maybe_add_fee' ) );
		
		add_action( 'wp_ajax_add_arbitrary_product', 'IGN_Arbitrary_Products::maybe_add_arbitrary_product', -1 );
		add_action( 'wp_ajax_nopriv_add_arbitrary_product', 'IGN_Arbitrary_Products::maybe_add_arbitrary_product', -1 );
		
		// Remove arbitrary products after the order is processing or completed
		add_action( 'woocommerce_order_status_completed', 'IGN_Arbitrary_Products::maybe_delete_arbitrary_product', 9999999, 1 );
		add_action( 'woocommerce_order_status_processing', 'IGN_Arbitrary_Products::maybe_delete_arbitrary_product', 9999999, 1 );

		//add_action( 'pre_user_query', array( &$this, 'customer_query_meta' ), -1 );

		// Load and work with an existing order
		add_action( 'wp_ajax_opc_json_search_orders', array( &$this, 'json_search_orders' ), -1 );
		add_action( 'wp_ajax_nopriv_opc_json_search_orders', array( &$this, 'json_search_orders' ), -1 );
		
		
		/*
		add_action( 'wp_ajax_opc_load_order', array( &$this, 'load_order' ), -1 );
		add_action( 'wp_ajax_nopriv_opc_load_order', array( &$this, 'load_order' ), -1 );
		*/
		add_action( 'wp_loaded', array( &$this, 'load_order' ), 11 );
		
		//add_action( 'woocommerce_create_order', array( &$this, 'maybe_alter_order_status' ), 2, -1 );

		add_action( 'wp_ajax_ign_opc_clear_cart', 'IGN_OPC_Cart::clear_cart', -1 );
		add_action( 'wp_ajax_nopriv_ign_opc_clear_cart', 'IGN_OPC_Cart::clear_cart', -1 );
		
		// Configure product popup stuff
		add_action( 'wp_ajax_ign_one_page_configure_product_iframe_tag', array( &$this, 'configure_product_iframe_tag' ), -1 );
		// not implemented yet
		//add_action( 'wp_ajax_ign_one_page_check_product_type', array( &$this, 'check_product_type' ), -1 );
		
		add_action( 'wp_ajax_ign_one_page_update_items_table', array( &$this, 'update_items_table' ), -1 );
		add_action( 'wp_ajax_nopriv_ign_one_page_update_items_table', array( &$this, 'update_items_table' ), -1 );

		add_action( 'woocommerce_thankyou', array( &$this, 'thankyou_page_additions' ), 50 );

		// DEPRECATED
		//add_filter( 'woocommerce_json_search_customers_query', array( &$this, 'filter_user_search' ), 1, 4 );
		//add_filter( 'woocommerce_customer_search_customers', array( &$this, 'filter_user_search' ), 1, 4 );
		
		// This can make queries VERY slow on sites that have 10K+ users, maybe add this only via a setting
		// add_filter( 'woocommerce_json_search_customers_query', array( &$this, 'add_customer_company_search' ), 1, 4 );
		//add_filter( 'woocommerce_customer_search_customers', array( &$this, 'add_customer_company_search' ), 1, 4 );
		
		// WC 3.0 and newer (there was no 2.7 etc.)
		add_action( 'wp_ajax_opc_json_search_customers', array( &$this, 'json_search_customers' ), -1 );
		add_action( 'wp_ajax_nopriv_opc_json_search_customers', array( &$this, 'json_search_customers' ), -1 );
		
		add_action( 'admin_menu', array( &$this, 'add_new_order_link' ), 9999 );

		add_action( 'woocommerce_review_order_before_order_total', array( &$this, 'add_shipping_form' ), 1 );

		add_action( 'wp_ajax_set_custom_shipping_cost', array( &$this, 'set_custom_shipping_cost' ) );
		add_action( 'wp_ajax_nopriv_set_custom_shipping_cost', array( &$this, 'set_custom_shipping_cost' ) );

		add_action( 'woocommerce_checkout_update_order_meta', array( &$this, 'add_clerk_info' ), 5, 2 );
		add_action( 'add_meta_boxes', array( &$this, 'clerk_info_box' ), 10, 2 );

		// Prevent redirect to cart if that setting is on in WC and the popup configurator is being used
		if ( !empty( $_POST['ign_one_page_popup'] ) )
			add_filter( 'option_woocommerce_cart_redirect_after_add', array( $this, 'prevent_redirect_after_add' ) );

		$this->settings = get_option( 'woocommerce_ignitewoo_manual_order_settings' );

		add_filter( 'template_include', array( &$this, 'maybe_get_phone_manual_order_page_template' ), 99, 1 );
		
		// Add new column to orders that indicate whether the order was placed with this plugin
		add_filter( 'manage_edit-shop_order_columns', array( &$this, 'admin_order_column_header' ), 20 );
		add_action( 'manage_shop_order_posts_custom_column',  array( &$this, 'admin_order_column_content' ), 10, 2 );
		
		// HPOS Order table column
		add_filter( 'manage_woocommerce_page_wc-orders_columns', array( &$this, 'admin_order_column_header' ), 20 );
		// HPOS order table column content
		add_action( 'manage_woocommerce_page_wc-orders_custom_column',  array( &$this, 'admin_order_column_content' ), 10, 2 );
		
		add_filter( "manage_edit-shop_order_sortable_columns", array( &$this, 'admin_order_column_sortable' ) );
		add_filter( "pre_get_posts", array( &$this, 'admin_order_column_sortable_request' ), 99 );
		
		add_filter( 'wc_google_analytics_pro_do_not_track', array( &$this, 'maybe_disabled_google_analytics_pro' ), 99999, 3 );
		
		add_action('wp_print_scripts', array( &$this, 'dequeue_problem_scripts' ), 1 );		
	}
	
	public static function maybe_set_user_customer( $price = '', $_product = false ) {
		global $ign_opc;
		
		$ign_opc->switch_user_context_to_customer( 'customer' );
	//var_dump( get_current_user_id() );

		return $price; 
	}
	public static function maybe_set_user_admin( $price = '', $_product = false ) {
		global $ign_opc;
		
		$ign_opc->switch_user_context_to_customer( 'opc_user' );
		return $price; 
	}
	// Dequeue JS scripts that conflict with the order page, if the order page is being loaded
	function dequeue_problem_scripts() {         
		
		if ( empty( $this->opc ) || !$this->opc )
				return;
			
		// Checkout for WooCommerce plugin JS: 
		wp_dequeue_script( 'cfw_vendor_js' );
		wp_deregister_script( 'cfw_vendor_js' );

	}

	public function shutdown_maybe_set_user() { 
		global $ign_opc; 
			
		if ( false === apply_filters( 'ign_opc_maybe_set_user_at_shutdown', true ) ) { 
			return;
		}
		
		// Switch back
		$ign_opc->switch_user_context_to_customer( 'opc_user' );
	}

	public function maybe_set_user() { 
		global $ign_opc;

		$ign_opc->switch_user_context_to_customer( 'customer' );
	}

	// Checks product type so that JS can hide show add to order or configure buttons
	function check_product_type() {
		if ( empty( $_POST['id'] ) || absint( $_POST['id'] ) <= 0 )
			die( '-1' );

		$p = wc_get_product( absint( $_POST['id'] ) );

		if ( $p->is_type( 'simple' ) || $p->is_type( 'variation' ) )
			die( 'simple' );

		die( 'other' );
	}

	function configure_product_iframe_tag() {
		//global $post, $product, $wp_query;

		if ( empty( $_POST['items'] ) || absint( $_POST['items'] ) <= 0 )
			die( '-1' );

		if ( is_array( $_POST['items'] ) )
			$item = $_POST['items'][0];
		else
			$item = $_POST['items'];

		$link = get_permalink( absint( $item ) );

		if ( empty( $link ) )
			die( '-2' );

		if ( !empty( $_SERVER['HTTPS'] ) && 'off' !== $_SERVER['HTTPS'] )
			$link = str_replace( 'http://', 'https://', $link );

		$args = array(
			'ign_opc_pop' => 1,
			//'iframe' => 'true',
			//'width' => '100%',
			//'height' => '100%'
		);

		$link = add_query_arg( $args, $link );

		ob_start();

		?>
		<iframe id="configure_product_iframe" src="<?php echo $link ?>"></iframe> 
		<?php

		$out = ob_get_clean();

		die( json_encode( array( 'view' => $out ) ) );

	}
	
	// Prevent redirect to cart if that setting is on in WC and the popup configurator is being used
	function prevent_redirect_after_add( $val = null ) {
		return false;
	}

	// Note implemented at this time, makes queries slower
	/*
	function customer_query_meta( $wp_user_query ) {
		global $wpdb;

		if ( empty( $_GET['term'] ) )
			return;

		$term = wc_clean( stripslashes( $_GET['term'] ) );

		$wp_user_query->query_from .= ' left join ' . $wpdb->usermeta . ' ign_mm on ID=ign_mm.user_id ';

		$wp_user_query->query_where .= ' OR ( ign_mm.meta_key="billing_company" and ign_mm.meta_value LIKE "%' . $term . '%"  ) OR ( ign_mm.meta_key="billing_phone" and ign_mm.meta_value LIKE "%' . $term . '%" ) OR ( ign_mm.meta_key="billing_first_name" and ign_mm.meta_value LIKE "%' . $term . '%"  ) OR ( ign_mm.meta_key="billing_last_name" and ign_mm.meta_value LIKE "%' . $term . '%" )';

		return $wp_user_query;
	}
	*/

	function json_search_products_and_variations() {
		global $ign_opc;

		// Allow other plugins to do something before search starts.
		do_action( 'before_phone_manual_order_product_search' );
			
		ob_start();

		//$ign_opc->switch_user_context_to_customer( 'opc_user' );

		//check_ajax_referer( 'search-products', 'security' );
		
		$ign_opc->switch_user_context_to_customer( 'customer' );

		remove_all_filters( 'pre_get_posts' );

		$term    = (string) stripslashes( $_GET['term'] );
		$exclude = array();
		
		$post_types = apply_filters( 'ignitewoo_opc_search_post_types', array( 'product', 'product_variation' ) );
		
		$post_statuses = apply_filters( 'ignitewoo_opc_search_post_statuses', array( 'publish', 'private' ) );

		if ( empty( $term ) ) {
			die();
		}

		if ( ! empty( $_GET['exclude'] ) ) {
			$exclude = array_map( 'intval', explode( ',', $_GET['exclude'] ) );
		}

		/** 	Allow other code to perform the search and return results in a PHP array of post ID numbers.
			Returned array should look like this:
		 	$posts = array( 
				0 => 12639,
				1 => 42561,
				2 => 72890,
			);
		*/
		$posts = apply_filters( 'phone_manual_order_product_search', false, $term, $exclude, $post_types, $post_statuses );
		
		if ( empty( $posts ) ) {

			$args = array(
				'post_type'      => $post_types,
				'post_status'    => $post_statuses,
				'posts_per_page' => -1,
				's'              => $term,
				'fields'         => 'ids',
				'exclude'        => $exclude,
				'suppress_filters' => false,
			);

			if ( is_numeric( $term ) ) {

				if ( false === array_search( $term, $exclude ) ) {
					$posts2 = get_posts( array(
						'post_type'      => $post_types,
						'post_status'    => $post_statuses,
						'posts_per_page' => -1,
						'post__in'       => array( 0, $term ),
						'fields'         => 'ids',
						'suppress_filters' => false,
					) );
				} else {
					$posts2 = array();
				}

				$posts3 = get_posts( array(
					'post_type'      => $post_types,
					'post_status'    => $post_statuses,
					'posts_per_page' => -1,
					'post_parent'    => $term,
					'fields'         => 'ids',
					'exclude'        => $exclude,
					'suppress_filters' => false,
				) );

				$posts4 = get_posts( array(
					'post_type'      => $post_types,
					'post_status'    => $post_statuses,
					'posts_per_page' => -1,
					'meta_query'     => array(
						array(
							'key'     => '_sku',
							'value'   => $term,
							'compare' => 'LIKE'
						)
					),
					'fields'         => 'ids',
					'exclude'        => $exclude,
					'suppress_filters' => false,
				) );

				$posts = array_unique( array_merge( get_posts( $args ), $posts2, $posts3, $posts4 ) );

			} else {

				$args2 = array(
					'post_type'      => $post_types,
					'post_status'    => $post_statuses,
					'posts_per_page' => -1,
					'meta_query'     => array(
						array(
						'key'     => '_sku',
						'value'   => $term,
						'compare' => 'LIKE'
						)
					),
					'fields'         => 'ids',
					'exclude'        => $exclude,
					'suppress_filters' => false,
				);

				// Category search ONLY works on terms, not SKUs because WP_Query can't do "meta_query OR tax_query" queries
				if ( !empty( $_GET['opc_search_cat'] ) && absint( $_GET['opc_search_cat'] ) > 1 ) { 
				
					$args2['s'] = $term;
					
					unset( $args2['meta_query'] );
				
					//$args2['relation'] = 'OR';
					
					$args2['tax_query'] = array(
						array(
							'taxonomy' => 'product_cat',
							'field'	=> 'term_id',
							'terms'	=> array( $_GET['opc_search_cat'] ),
							'operator'  => 'IN',
							'suppress_filters' => false,
						)
					);
					$posts = array_unique( get_posts( $args2 ) );

				} else { 
					$posts = array_unique( array_merge( get_posts( $args ), get_posts( $args2 ) ) );
				}
			}
		}
		
		$found_products = array();
		
		if ( ! empty( $posts ) ) {
			/*
			// Switch so that the correct prices show in case pricing plugins alter prices
			if ( !empty( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ) {
				$_POST['ignitewoo_one_page_checkout_customer_id'] = $_REQUEST['ignitewoo_one_page_checkout_customer_id'];
				if ( method_exists( $ign_opc, 'switch_user_context_to_customer' ) )
					$ign_opc->switch_user_context_to_customer( 'customer' );
			}
			*/
			
			$parents = array();
			
			foreach ( $posts as $post ) {
			
				$product = wc_get_product( $post );

				if ( empty( $product ) || is_bool( $product ) )
					continue;
				
				$parent_id = $this->get_product_parent_id( $product );
								
				if ( $product->is_type( 'variation' ) && empty( $parent_id ) ) {
					continue;
				}
				
				if ( $product->is_type( 'variation' ) ) {
					if ( empty( $parents[ $product->get_parent_id() ] ) ) { 
						$parent = wc_get_product( $parent_id );
					} else { 
						$parent = $parents[ $product->get_parent_id() ];
					}
					
					if ( 'publish' !== $parent->get_status() ) { 
						continue; 
					}
				}

				$data = $this->get_product_name( $product );

				$found_products[ $post ] = rawurldecode( $data['name'] ) . $data['price'] . $data['stock'];

				// When searching with a category filter then variation children do not wind up in the results, 
				// so add them. 

				if ( !empty( $_GET['opc_search_cat'] ) && absint( $_GET['opc_search_cat'] ) > 1 ) { 
					if ( !$product->is_type( 'variable' ) )
						continue;
						
					foreach( $product->get_children() as $child_id ) {
						$product = wc_get_product( $child_id );
						$data = $this->get_product_name( $product );
						$found_products[ $child_id ] = rawurldecode( $data['name'] ) . $data['price'] . $data['stock'];
					}
				}
			}
		}

		$found_products = apply_filters( 'woocommerce_opc_json_search_found_products', $found_products, $term );

		wp_send_json( $found_products );
	}

	public function get_product_name( $product ) { 

		if ( $product->is_type( 'simple' ) )
			$price = ' &ndash; ' . strip_tags( wc_price( $product->get_price() ) );
		else
			$price = '';

		$stock = '';

		if ( !empty( $this->settings['enable_stock'] ) && 'yes' == $this->settings['enable_stock'] ) {
			if ( 'yes' == get_option( 'woocommerce_manage_stock' ) ) {
				$stock = $product->get_stock_quantity();
				if ( null !== $stock )
					$stock = ' &ndash; (' . $stock . ' ' . __( 'in stock', 'manual_phone_orders' ) . ')';
			} else { 
				$stock = $product->get_stock_status();
				if ( null !== $stock ) { 
					switch( $stock ) { 
						case 'instock': 
							$stock_status = __( 'In stock', 'manual_phone_orders' );
							break;
						case 'outofstock': 
							$stock_status = __( 'Out of stock', 'manual_phone_orders' );
							break;
						case 'onbackorder': 
							$stock_status = __( 'On backorder', 'manual_phone_orders' );
							break;
						default:
							$stock_status = '';
							break;
					}
					$stock = ' &ndash; (' . $stock_status . ')';
				}
			}
		}

		if ( version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) ) {

			if ( $product->is_type( 'variation' ) ) {
				$attrs = $product->get_variation_attributes();
				if ( !empty( $attrs ) ) {
					$a = array();
					foreach( $attrs as $key => $val ) {
						$a[] = $product->get_attribute( str_replace( 'attribute_', '', $key ) );
					}
					if ( method_exists( $product, 'get_name' ) )
						$name = $product->get_name() . ' &ndash; ' . implode( ' &ndash;', $a ) . ' (' . $product->get_sku() . ' / #' . $product->get_id() . ')';
					else 
						$name = $product->get_formatted_name();
					
				} else {
					$name = $product->get_formatted_name();
				}
			} else {
				$name = $product->get_formatted_name();
			}
		} else {
			$name = $product->get_formatted_name();
		}
		
		return array( 'name' => $name, 'price' => $price, 'stock' => $stock );
	}
	
	public function handle_custom_search_orders_query( $query, $query_vars ) {

		if ( isset($query_vars['load_orders_query'] ) ) {
			$query['meta_query']     = array(
				'relation' => 'OR',
				array(
					'key'     => '_billing_first_name',
					'value'   => $query_vars['load_orders_query'] ,
					'compare' => 'LIKE'
				),
				array(
					'key'     => '_billing_last_name',
					'value'   => $query_vars['load_orders_query'] ,
					'compare' => 'LIKE'
				),
			);
		}

		return $query;
	}
	
	function load_orders_company_query_vars( $query, $query_vars ) { 

		if ( ! empty( $query_vars['load_orders_query'] ) ) {
			$query['meta_query'] = array( 
				'relation' => 'OR', 
				array(
					'key'     => '_billing_company',
					'value'   => $query_vars['load_orders_query'],
					'compare' => 'LIKE'
				),
				
			);
		}

		return $query;
	}
	
	public function json_search_orders() {
		global $ign_opc;
		
		ob_start();

		$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		check_ajax_referer( 'search-products', 'security' );
		
		$ign_opc->switch_user_context_to_customer( 'customer' );

		//$term = (string) wc_clean( stripslashes( $_GET['term'] ) );
		$term = (string) stripslashes( $_GET['term'] );

		$posts1 = array();
		
		// Search by ID unless sequential numbers are in use:
		if ( !class_exists( 'WC_Seq_Order_Number_Pro' ) && !class_exists( 'WC_Seq_Order_Number' ) && ( !class_exists( 'WCJ_Order_Numbers' ) || 'no' === get_option( 'wcj_order_numbers_enabled') ) && !class_exists( 'Alg_WC_Custom_Order_Numbers_Core' ) ) {

			/*
			$posts1 = get_posts( array(
				'post_type'      => 'shop_order',
				'post_status'    => 'any',
				'posts_per_page' => -1,
				'include'	=> array( $term ),
				'fields'         => 'ids',
			) );
			*/
			
			if ( absint( $term ) > 0 ) { 
				$posts1 = wc_get_order( absint( $term ) );
			}

			if ( empty( $posts1 ) ) { 
				$posts1 = array();
			} else { 
				$posts1 = array( $posts1->get_id() );
			}
		}

		add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'handle_custom_search_orders_query' ), 1, 2 );
		
		$args = array( 
			'return' => 'ids',
			'load_orders_query' => $term,
			'limit' => '-1',
			// HPOS compatible customer query args:
			'field_query'     => array(
				'relation' => 'OR',
				array(
					'field'     => 'billing_first_name',
					'value'   => $term,
					'compare' => 'LIKE'
				),
				array(
					'field'     => 'billing_last_name',
					'value'   => $term,
					'compare' => 'LIKE'
				),
			),
			'orderby' => 'date',
			'order' => 'DESC',
		);
		
		$posts2 = wc_get_orders( $args );

		remove_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'handle_custom_search_orders_query' ), 1, 2 );

		/*
		// Names
		$args = array(
			'post_type'      => 'shop_order',
			'post_status'    => 'any',
			'posts_per_page' => -1,
			'meta_query'     => array(
				'relation' => 'OR',
				array(
					'key'     => '_billing_first_name',
					'value'   => $term,
					'compare' => 'LIKE'
				),
				array(
					'key'     => '_billing_last_name',
					'value'   => $term,
					'compare' => 'LIKE'
				),
			),
			'fields'         => 'ids',
		);
		
		if ( !empty( $posts1 ) ) { 
			$args['exclude'] = $posts1;
		}

		$posts2 = get_posts( $args );
		*/

		$posts = array_merge( $posts1, $posts2 );
	
		unset( $posts1, $posts2 );

		/*
		$args = array( 
			'post_type'      => 'shop_order',
			'post_status'    => 'any',
			'posts_per_page' => -1,
			'fields' => 'ids',
		);
		
		if ( !empty( $posts ) ) { 
			$args['exclude'] = $posts;
		}
		*/
			
		$args = array();
		
		// Custom order numbers: 
		if ( class_exists( 'WC_Seq_Order_Number_Pro' ) || class_exists( 'WC_Seq_Order_Number' ) ) {
			$args['meta_query'][] = array( 
				'relation' => 'OR', 
				array(
					'key'     => '_order_number',
					'value'   => $term,
					'compare' => 'LIKE'
				),
				array(
					'key'     => '_order_number_formatted',
					'value'   => $term,
					'compare' => 'LIKE'
				)
			);
			$args['load_orders_query'] = $term;
		} else if ( class_exists( 'Alg_WC_Custom_Order_Numbers_Core' ) ) {
			$args['meta_query'][]= array(
				'key'     => '_alg_wc_custom_order_number',
				'value'   => $term,
				'compare' => 'LIKE'
			);
			$args['load_orders_query'] = $term;
		} else if ( class_exists( 'WCJ_Order_Numbers' ) && 'yes' === get_option( 'wcj_order_numbers_enabled') ) {
			$args['meta_query'][]= array(
				'key'     => '_wcj_order_number',
				'value'   => $term,
				'compare' => 'LIKE'
			);
			$args['load_orders_query'] = $term;
		}

		if ( !empty( $args['meta_query'] ) ) { 
			
			$args['return'] = 'ids';
			$args['fields'] = 'ids';
			$args['limit'] = '-1';
			
			// sequential order numbers filter: 
			add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'load_orders_custom_query_vars' ), 10, 2 );
			
			$posts2 = wc_get_orders( $args );
			
			remove_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'load_orders_custom_query_vars' ), 10, 2 );
			
			$posts = array_merge( $posts, $posts2 );
		}
		
		/*
		// Do not run query if no sequential order number plugins are active on the site
		if ( !empty( $args['meta_query'] ) ) { 
			
			$posts2 = get_posts( $args );

			$posts = array_merge( $posts, $posts2 );

			unset( $posts1, $posts2 );
		}
		*/

		if ( !empty( $this->settings['search_by_company'] ) && 'yes' == $this->settings['search_by_company'] ) {
			
			/*
			$args = array(
				'post_type'      => 'shop_order',
				'post_status'    => 'any',
				'posts_per_page' => -1,
				'meta_query'     => array(
					array(
						'key'     => '_billing_company',
						'value'   => $term,
						'compare' => 'LIKE'
					)
				),
				'fields' => 'ids',
			);

			if ( !empty( $posts ) ) { 
				$args['exclude'] = $posts;
			}
			*/
			
			$args = array( 
				'fields' => 'ids',
				'return' => 'ids',
				'limit' => '-1',
				'load_orders_query' => $term,
				// HPOS compatible query vars:
				'field_query' => array(
					'field' => 'billing_company',
					'value' => $term,
					'compare' => 'LIKE',
				),
				'orderby' => 'date',
				'order' => 'DESC',
			);
			
			//$posts3 = get_posts( $args );
			add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'load_orders_company_query_vars' ), 10, 2 );
			
			$posts3 = wc_get_orders( $args );

			remove_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'load_orders_company_query_vars' ), 10, 2 );
			
			$posts = array_merge( $posts, $posts3 );
		}

		$found_orders = array();

		if ( ! empty( $posts ) ) {
			foreach ( $posts as $post_id ) {

				// Get all meta in one query: 
				// $custom_meta = get_post_custom( $post_id );
				$order = wc_get_order( $post_id );
				
				$fname = $order->get_billing_first_name();
				$lname = $order->get_billing_last_name();

				//$fname = $custom_meta['_billing_first_name'][0] ? $custom_meta['_billing_first_name'][0] : '';
				//$lname = $custom_meta['_billing_last_name'][0] ? $custom_meta['_billing_last_name'][0] : '';
				
				//$fname = get_post_meta( $post_id, '_billing_first_name', true );
				//$lname = get_post_meta( $post_id, '_billing_last_name', true );
				
				if ( class_exists( 'WC_Seq_Order_Number_Pro' ) || class_exists( 'WC_Seq_Order_Number' ) ) {
					
					$oid = $order->get_meta( '_order_number_formatted', true );
					
					//$oid = $custom_meta['_order_number_formatted'][0] ? $custom_meta['_order_number_formatted'][0] : '';
					
					// $oid = get_post_meta( $post_id, '_order_number_formatted', true );
					
					if ( empty( $oid ) ) { 
						//$oid = $custom_meta['_order_number'][0] ? $custom_meta['_order_number'][0] : '';
						//$oid = get_post_meta( $post_id, '_order_number', true );
						$oid = $order->get_meta( '_order_number', true );
					}
					
					if ( empty( $oid ) ) { 
						$oid = $post_id;
					}
				} else if ( class_exists( 'Alg_WC_Custom_Order_Numbers_Core' ) ) {
					
					//$oid = $custom_meta['_alg_wc_custom_order_number'][0] ? $custom_meta['_alg_wc_custom_order_number'][0] : '';
					//$oid = get_post_meta( $post_id, '_alg_wc_custom_order_number', true );	
					$oid = $order->get_meta( '_alg_wc_custom_order_number', true );
					if ( empty( $oid ) ) { 
						$oid = $post_id;
					}
				} else if ( class_exists( 'WCJ_Order_Numbers' ) && 'yes' === get_option( 'wcj_order_numbers_enabled' ) ) {
				
					//$oid = $custom_meta['_wcj_order_number'][0] ? $custom_meta['_wcj_order_number'][0] : '';
					//$oid = get_post_meta( $post_id, '_wcj_order_number', true );
					$oid = $order->get_meta( '_wcj_order_number', true );
					if ( empty( $oid ) ) { 
						$oid = $post_id;
					}
				} else { 
					$oid = $post_id;
				}

				$oid = (string) $oid; 

				$found_orders[ $oid ] = '#' . $oid . ' &ndash; ';
			
				$extra_data = array(); 

				if ( !empty( $this->settings['show_order_date'] ) && 'yes' == $this->settings['show_order_date'] ) { 
					$order = new WC_Order( $post_id ); 
					$extra_data[] = wc_format_datetime( $order->get_date_created() );
				}

				if ( !empty( $this->settings['show_order_total'] ) && 'yes' == $this->settings['show_order_total'] ) { 
					if ( empty( $order ) ) { 
						$order = new WC_Order( $post_id ); 
					}
					$extra_data[] = $order->get_total();
				}

				if ( !empty( $extra_data ) ) { 
					$found_orders[ $oid ] .= implode( ' &ndash; ', $extra_data ) . ' &ndash; ';
				}

				$found_orders[ $oid ] .= $fname . ' ' . $lname;
			}
		}

		arsort( $found_orders );

		$found_orders = apply_filters( 'ign_opc_json_search_found_orders', $found_orders, $term );

		wp_send_json( $found_orders );

	}

	function load_orders_custom_query_vars( $query, $query_vars ) { 
		if ( ! empty( $query_vars['load_orders_query'] ) ) {
			if ( class_exists( 'WC_Seq_Order_Number_Pro' ) || class_exists( 'WC_Seq_Order_Number' ) ) {
				$query['meta_query'] = array( 
					'relation' => 'OR', 
					array(
						'key'     => '_order_number',
						'value'   => $query_vars['load_orders_query'],
						'compare' => '='
					),
					array(
						'key'     => '_order_number_formatted',
						'value'   => $query_vars['load_orders_query'],
						'compare' => '='
					)
				);
			} else if ( class_exists( 'Alg_WC_Custom_Order_Numbers_Core' ) ) {
				$query['meta_query'][]= array(
					'key'     => '_alg_wc_custom_order_number',
					'value'   => $query_vars['load_orders_query'],
					'compare' => '='
				);
			} else if ( class_exists( 'WCJ_Order_Numbers' ) ) {
				$query['meta_query'][]= array(
					'key'     => '_wcj_order_number',
					'value'   => $query_vars['load_orders_query'],
					'compare' => '='
				);
			}
		}

		return $query;
	}
	
	function load_order() {
		global $ign_opc; 

		if ( empty( $_POST['one_page_load_order_id'] ) || empty(  $_POST['_wpnonce'] ) ) { 
			return;
		}
		
		if ( !wp_verify_nonce( $_POST['_wpnonce'], 'woocommerce-order_again' ) ) {
			//die( '-1' );
			return;
		}

		$oid = absint( $_POST['one_page_load_order_id'] );
		
		$args = array();
		
		/*
		$args = array(
			'post_type'      => 'shop_order',
			'post_status'    => 'any',
			'posts_per_page' => -1,
		);
		*/

		if ( class_exists( 'WC_Seq_Order_Number_Pro' ) || class_exists( 'WC_Seq_Order_Number' ) ) {
			$args['load_orders_query'] = $_POST['one_page_load_order_id'];
		} else if ( class_exists( 'Alg_WC_Custom_Order_Numbers_Core' ) ) {
			$args['load_orders_query'] = $_POST['one_page_load_order_id'];
		} else if ( class_exists( 'WCJ_Order_Numbers' ) ) {
			$args['load_orders_query'] = $_POST['one_page_load_order_id'];
		}
		
		if ( ! empty( $args['load_orders_query'] ) ) { 
	

			add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'load_orders_custom_query_vars' ), 10, 2 );
			
			$args['status'] = array( 'any' );
			
			$orders = wc_get_orders( $args );

			if ( ! empty( $orders ) ) {
				$oid = $orders[0]->get_id();
			}
			//$posts = get_posts( $args );
			//if ( ! empty( $posts ) ) { 
			//	$oid = $posts[0]->ID;
			//}
			
			remove_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( &$this, 'load_orders_custom_query_vars' ), 10, 2 );
		}

		$order = wc_get_order( absint( $oid ) );
		
		$order_id = $oid; // $this->get_order_id( $order );

		if ( empty( $order_id ) ) {
			return;
		}

		if ( method_exists( $order, 'get_customer_id' ) )
			$customer_id = $order->get_customer_id();
		else 
			$customer_id = $order->customer_user;
			
		$_REQUEST['one_page_customer_user'] = $customer_id; 

		//$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		$overwrite_order = ( isset( $_POST['one_page_load_order_id_as'] ) && 'overwrite' == $_POST['one_page_load_order_id_as'] ) ? true : false;
		
		$include_coupons = ( isset( $_POST['one_page_load_order_include_coupons'] ) && 'true' == $_POST['one_page_load_order_include_coupons'] ) ? true : false;

		// NOTE: State is after country in the list so the Select2 can update automatically via our JS
		// otherwise WC js resets the state when country is set when state comes before country
		if ( method_exists( $order, 'get_billing_first_name' ) ) { 
			$billing = apply_filters( 'woocommerce_order_formatted_billing_address', array(
				'first_name'    	=> $order->get_billing_first_name(),
				'last_name'     	=> $order->get_billing_last_name(),
				'company'      	=> $order->get_billing_company(),
				'address_1'    	=> $order->get_billing_address_1(),
				'address_2'     	=> $order->get_billing_address_2(),
				'city'          	=> $order->get_billing_city(),
				'postcode'      	=> $order->get_billing_postcode(),
				'country'       	=> $order->get_billing_country(),
				'state'         	=> $order->get_billing_state(),
				'email'		=> $order->get_billing_email(),
				'phone'		=> $order->get_billing_phone(),
			), $order );

			if ( $order->get_shipping_address_1() || $order->get_shipping_address_2() ) {

				// Formatted Addresses
				$shipping = apply_filters( 'woocommerce_order_formatted_shipping_address', array(
					'first_name'    => $order->get_shipping_first_name(),
					'last_name'     => $order->get_shipping_last_name(),
					'company'       => $order->get_shipping_company(),
					'address_1'     => $order->get_shipping_address_1(),
					'address_2'     => $order->get_shipping_address_2(),
					'city'          => $order->get_shipping_city(),
					'postcode'      => $order->get_shipping_postcode(),
					'country'       => $order->get_shipping_country(),
					'state'         => $order->get_shipping_state(),
				), $order );

				//$shipping = WC()->countries->get_formatted_address( $address );
				
			} else {
				$shipping = array();
			}
		} else { 
			$billing = apply_filters( 'woocommerce_order_formatted_billing_address', array(
				'first_name'    => $order->billing_first_name,
				'last_name'     => $order->billing_last_name,
				'company'       => $order->billing_company,
				'address_1'     => $order->billing_address_1,
				'address_2'     => $order->billing_address_2,
				'city'          => $order->billing_city,
				'postcode'      => $order->billing_postcode,
				'country'       => $order->billing_country,
				'state'         => $order->billing_state,
				'email'		=> $order->billing_email,
				'phone'		=> $order->billing_phone,
			), $order );

			if ( $order->shipping_address_1 || $order->shipping_address_2 ) {

				// Formatted Addresses
				$shipping = apply_filters( 'woocommerce_order_formatted_shipping_address', array(
					'first_name'    => $order->shipping_first_name,
					'last_name'     => $order->shipping_last_name,
					'company'       => $order->shipping_company,
					'address_1'     => $order->shipping_address_1,
					'address_2'     => $order->shipping_address_2,
					'city'          => $order->shipping_city,
					'postcode'      => $order->shipping_postcode,
					'country'       => $order->shipping_country,
					'state'         => $order->shipping_state,
				), $order );

				//$shipping = WC()->countries->get_formatted_address( $address );
				
			} else {
				$shipping = array();
			}
		}

		foreach( $billing as $k => $v ) { 
			$_POST[ 'billing_' . $k ] = $v;
		}
		
		foreach( $shipping as $k => $v ) { 
			$_POST[ 'shipping_' . $k ] = $v;
		}
		
		if ( method_exists( $order, 'get_customer_note' ) ) {
			$note = $order->get_customer_note();
		} else { 
			$note = $order->customer_note; 
		}
		
		WC()->cart->empty_cart();

		// Copy products from the order to the cart
		foreach ( $order->get_items() as $item ) {
			// Load all product info including variation data
			$product_id   = (int) apply_filters( 'woocommerce_add_to_cart_product_id', $item['product_id'] );
			$quantity     = (int) $item['qty'];
			$variation_id = (int) $item['variation_id'];
			$variations   = array();
			$cart_item_data = apply_filters( 'woocommerce_order_again_cart_item_data', array(), $item, $order );

			foreach ( $item['item_meta'] as $meta_name => $meta_value ) {

				if ( taxonomy_is_product_attribute( $meta_name ) ) {
					$variations[ $meta_name ] = $meta_value;
				} 
				
				if ( is_array( $meta_value ) && meta_is_product_attribute( $meta_name, $meta_value[0], $product_id ) ) { 
					$variations[ $meta_name ] = $meta_value[0];
				} else if ( meta_is_product_attribute( $meta_name, $meta_value, $product_id ) ) { 
					$variations[ $meta_name ] = $meta_value;
				}
			}

			$_POST['quantity'] = $quantity; // fix for compat with woo-min-max-quantity - 3rd party plugin 
			
			$key = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variations, $cart_item_data );

			// Set the price to be whatever is saved in the order, in case the manual/phone order user changed the price
			if ( !empty( $key ) && !$include_coupons ) { 
				$price = $order->get_item_total( $item, false, true );
				WC()->cart->cart_contents[ $key ]['_price_altered'] = $price;
				WC()->cart->set_session();		
			}
		}

		if ( $include_coupons ) { 
			foreach ( $order->get_items( 'coupon' ) as $item ) {
				$code = method_exists( $item, 'get_code' ) ? $item->get_code() : false; 
				if ( !empty( $code ) ) { 
					WC()->cart->apply_coupon( $code );
				}
			}
		}
		
		// Store this in the session when reloading an order so that the notes can be loaded too in the shipping template file.d
		WC()->session->set( 'phone_manual_order_loaded_id', $order_id );
		
		if ( $overwrite_order ) {
			/*
			WARNING: For an existing order to be loaded it must have a status of pending or failed otherwise
			WC will check the order status and empty the cart if the "order_awaiting_payment" has a status
			other than pending or failed. Therefore we set the status to pending here to ensure that the contents
			of the order can be loaded into the cart and when the order is placed that existing order can be
			overwritten.
			*/
			$order->update_status( 'pending' );
			
			WC()->session->set( 'order_awaiting_payment', $order_id );
			
			if ( method_exists( WC()->session, 'save_data' ) )
				WC()->session->save_data();
				
		} else if ( isset( WC()->session->order_awaiting_payment ) ) {
		
			WC()->session->set( 'order_awaiting_payment', false );
			
			if ( method_exists( WC()->session, 'save_data' ) )
				WC()->session->save_data();
		}

		//die( json_encode( array( 'success' => 'true', 'customer_id' => $customer_id, 'billing' => $billing, 'shipping' => $shipping, 'customer_note' => $note ) ) );
	}

	// This is used when someone loads an order and does not check the box to indicate that it should be loaded as a new order
	// Therefore the existing order status is set to pending or failed so that WC will overwrite it when the order is placed
	/* No longer necessary? 
	function maybe_alter_order_status() {

		$order_id = absint( WC()->session->get( 'order_awaiting_payment' ) );

		if ( empty( $order_id ) )
			return false; 
			
		$order = wc_get_order( $order_id );

		//$order_id = $this->get_order_id( $order );

		if ( !empty( $order ) ) {
			// Set the status to pending so that WC will overwrite it
			$order->update_status( 'pending' );
		}
	}
	*/
	
	function remove_hooks() {

		if ( isset( $_REQUEST['post_data'] ) ) {

			$args = wp_parse_args( $_REQUEST['post_data'] );

			if ( empty( $args['opc_checkout_flag'] ) && empty( $args['ignitewoo_one_page_checkout'] ) )
				return;

			remove_action( 'wc_ajax_update_order_review', array( 'WC_AJAX', 'update_order_review' ), 10 );
			remove_action( 'wc_ajax_nopriv_update_order_review', array( 'WC_AJAX', 'update_order_review' ), 10 );

			remove_action( 'wp_ajax_woocommerce_update_order_review', array( 'WC_AJAX', 'update_order_review' ), 10 );
			remove_action( 'wp_ajax_nopriv_woocommerce_update_order_review', array( 'WC_AJAX', 'update_order_review' ), 10 );
		}

		add_action( 'wp_ajax_woocommerce_update_order_review', array( &$this, 'update_order_review' ), -99 );
		add_action( 'wp_ajax_nopriv_update_order_review', array( &$this, 'update_order_review' ), -99 );
		add_action( 'wc_ajax_update_order_review', array( &$this, 'update_order_review' ), -99 );
		
	}

	function add_new_order_link() {
		global $submenu, $wpdb;

		if ( !current_user_can( 'administrator' ) && !current_user_can( 'shop_manager' ) )
			return;

		$sql = 'select ID from ' . $wpdb->posts . ' where post_type="page" and post_content like "%[woocommerce_manual_phone_order%" and post_status="publish"';

		$res = $wpdb->get_var( $sql );

		if ( empty( $res ) )
			return;

		$url = get_the_permalink( $res );

		$insert = array( __( 'Add New Order','manual_phone_orders' ) , 'edit_shop_orders', $url );

		$temp = $submenu['woocommerce'];

		if ( is_array( $temp ) )
			array_splice( $temp, 2, 0, '' );

		$submenu['woocommerce'] = $temp;

		$submenu['woocommerce'][2] = $insert;
	}

	// Helps cache plugins and browsers avoid caching the page and its elements
	public function no_cache() {

		if ( !$this->opc )
			return;

		if ( ! defined( 'DONOTCACHEPAGE' ) ) {
			define( "DONOTCACHEPAGE", true );
		}

		if ( ! defined( 'DONOTCACHEOBJECT' ) ) {
			define( "DONOTCACHEOBJECT", true );
		}

		if ( ! defined( 'DONOTCACHEDB' ) ) {
			define( "DONOTCACHEDB", true );
		}

		nocache_headers();
	}

	public function wp_head() {
		global $ign_opc; 

		if ( !$this->opc )
			return;

		$stylesheet = get_stylesheet_directory() . '/' . WC()->template_path() . '/checkout/one_page_checkout.css';

		if ( file_exists( $stylesheet ) )
			$url = get_stylesheet_uri() . '/' . WC()->template_path() . '/checkout/one_page_checkout.css';
		else
			$url = $ign_opc->plugin_url() . '/assets/one_page_checkout.css';

		if ( version_compare( WOOCOMMERCE_VERSION, '3.2', '>=' ) )
			$wc_32_or_newer = 'true';
		else 
			$wc_32_or_newer = 'false';
			
		// Make this has a value, it's a newer setting people might not have seen and adjusted yet
		if ( empty( $this->settings['disable_autorefresh'] ) )
			$this->settings['disable_autorefresh'] = 'no';
			
		$js = "
		ign_one_page_search_products_nonce = '". wp_create_nonce( 'search-products' ) . "';
		ign_one_page_search_customers_nonce = '". wp_create_nonce( 'search-customers' ) . "';
		ign_one_page_ajax_url = '" . admin_url('admin-ajax.php') . "';
		wc_version_info = '" . WOOCOMMERCE_VERSION . "';
		wc_32_or_newer = '" . $wc_32_or_newer . "';
		";

		echo '<script>'.$js.'</script>';

		wp_register_style( 'opc_styles', $url );

		wp_enqueue_style( 'opc_styles' );

		wp_register_script( 'opc-script', $ign_opc->plugin_url() . '/scripts/ignitewoo-opc.js?time=' . time(), array( 'jquery' ), time(), true );

		wp_enqueue_script( 'opc-script' );

		wp_localize_script( 'opc-script', 'opc_enhanced_select_params', array(
			'ajax_url'                  => admin_url( 'admin-ajax.php' ),
			'i18n_matches_1'            => _x( 'One result is available, press enter to select it.', 'enhanced select', 'manual_phone_orders' ),
			'i18n_matches_n'            => _x( '%qty% results are available, use up and down arrow keys to navigate.', 'enhanced select', 'manual_phone_orders' ),
			'i18n_no_matches'           => _x( 'No matches found', 'enhanced select', 'manual_phone_orders' ),
			'i18n_ajax_error'           => _x( 'Loading failed', 'enhanced select', 'manual_phone_orders' ),
			'i18n_input_too_short_1'    => _x( 'Please enter 1 or more characters', 'enhanced select', 'manual_phone_orders' ),
			'i18n_input_too_short_n'    => _x( 'Please enter %qty% or more characters', 'enhanced select', 'manual_phone_orders' ),
			'i18n_input_too_long_1'     => _x( 'Please delete 1 character', 'enhanced select', 'manual_phone_orders' ),
			'i18n_input_too_long_n'     => _x( 'Please delete %qty% characters', 'enhanced select', 'manual_phone_orders' ),
			'i18n_selection_too_long_1' => _x( 'You can only select 1 item', 'enhanced select', 'manual_phone_orders' ),
			'i18n_selection_too_long_n' => _x( 'You can only select %qty% items', 'enhanced select', 'manual_phone_orders' ),
			'i18n_load_more'            => _x( 'Loading more results&hellip;', 'enhanced select', 'manual_phone_orders' ),
			'i18n_searching'            => _x( 'Searching&hellip;', 'enhanced select', 'manual_phone_orders' ),
			'select_order'            => _x( 'Select an order first', 'enhanced select', 'manual_phone_orders' ),
			'security_error'            => _x( 'Security problem. Reload the page and try again', 'enhanced select', 'manual_phone_orders' ),
			'load_order_confirm'            => _x( 'Are you sure that want to load this order and overwrite it? Doing this will immedidately set the order status to "Pending Payment" and when you submit the order the existing order will be permanently and completely overwritten!', 'enhanced select', 'manual_phone_orders' ),
			'confirm' => __( 'Are you sure?', 'manual_phone_orders' ),
			'search_products_nonce'     => wp_create_nonce( 'search-products' ),
			'search_customers_nonce'    => wp_create_nonce( 'search-customers' ),
			'set_shipping_cost_first' => _x( 'Set a shipping cost first. Use 0 for free shipping.', 'input', 'manual_phone_orders' ),
			'yes_val' => _x( 'Yes', 'input', 'manual_phone_orders' ),
			'no_val' => _x( 'No', 'input', 'manual_phone_orders' ),
			'wc_version_gte_27' => version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ? 'true' : 'false',
			'arbitrary_product_title' => __( 'Add an arbitrary product', 'manual_phone_orders' ),
			'ajax_error_messsage' => __( 'An error occurred', 'manual_phone_orders' ),
			'disable_autorefresh' => $this->settings['disable_autorefresh'],
			'configure_product_title' => __( 'Add the item to the cart then close this window', 'manual_phone_orders' ),
		) );

		// Checkout page scripts from WC
		add_action( 'wp_head', array( $this, 'load_scripts_styles' ), 9999 );

		// Maybe load customer info it that was submitted via the form
		$this->ignitewoo_one_page_get_customer();
	}

	// Ensure context is switched so the checkout nonce is correct
	public function woocommerce_review_order_before_submit() { 
		global $ign_opc;
		//$ign_opc->switch_user_context_to_customer( 'opc_user' );

	}
	
	public function adjust_hooks() {

		if ( !$this->has_access() )
			return;

		add_action( 'woocommerce_is_checkout', array( &$this, 'is_checkout' ), 1, 999999 );
		add_action( 'woocommerce_review_order_before_submit', array( &$this, 'woocommerce_review_order_before_submit' ), 1 );
		
		if ( !empty( $_POST['ignitewoo_one_page_checkout'] ) && 'searchable' == $_POST['ignitewoo_one_page_checkout'] ) {

			remove_action( 'wc_ajax_checkout', array( 'WC_AJAX', 'checkout' ) );
			remove_action( 'wp_ajax_woocommerce_checkout', array( 'WC_AJAX', 'checkout' ) );
			remove_action( 'wp_ajax_nopriv_woocommerce_checkout', array( 'WC_AJAX', 'checkout' ) );
			remove_action( 'init', array( 'WC_Form_Handler', 'checkout_action' ), 20 );
			remove_action( 'wp_loaded', array( 'WC_Form_Handler', 'checkout_action' ), 20 );

			add_action( 'wc_ajax_checkout', array( &$this, 'checkout' ) );
			add_action( 'wp_ajax_woocommerce_checkout', array( &$this, 'checkout' ) );
			add_action( 'wp_ajax_nopriv_woocommerce_checkout', array( &$this, 'checkout' ) );
			add_action( 'wp_loaded', array( &$this, 'checkout_action' ), 20 );
		}
	}

	public function is_checkout() {

		if ( empty( $this->settings ) )
			$this->settings = get_option( 'woocommerce_ignitewoo_manual_order_settings' );

		if ( empty( $this->settings['manual_order_page'] ) )
			return false;

		$order_page = is_page( $this->settings['manual_order_page'] );

		if ( $order_page )
			return true;

		// Helpful for woocommerce_update_order_review Ajax,
		// hopefully gets required plugins to load if they need to.
		if ( !empty( $_POST['post_data'] ) ) { 
		
			$data = wp_parse_args( $_POST['post_data']);

			if ( !empty( $data['opc_checkout_flag'] ) )
				return true;
			else if ( !empty( $data['ignitewoo_one_page_checkout'] ) )
				return true;
		}
	}

	public function checkout() {

		if ( ! defined( 'WOOCOMMERCE_CHECKOUT' ) ) {
			define( 'WOOCOMMERCE_CHECKOUT', true );
		}

		$this->maybe_disable_terms();

		require_once( dirname( __FILE__ ) . '/class-ign-checkout.php' );
		
		IGN_OPC_Checkout::process_checkout();

		die(0);
	}

	public function checkout_action() {

		// For non-JS enabled order placement
		if ( isset( $_POST['woocommerce_checkout_place_order'] ) || isset( $_POST['woocommerce_checkout_update_totals'] ) ) {

			if ( sizeof( WC()->cart->get_cart() ) == 0 ) {
				wp_redirect( get_permalink( wc_get_page_id( 'cart' ) ) );
				exit;
			}

			if ( ! defined( 'WOOCOMMERCE_CHECKOUT' ) ) {
				define( 'WOOCOMMERCE_CHECKOUT', true );
			}
			
			require_once( dirname( __FILE__ ) . '/class-ign-checkout.php' );
		
			IGN_OPC_Checkout::process_checkout();

		}
	}

	function get_order_id( $order ) { 
		if ( !is_object( $order ) )
			return null;
		if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) )
			if ( method_exists( $order, 'get_id' ) )
				return $order->get_id();
		else if ( !empty( $order->id ) )
			return $order->id;
		else 
			return null;
	}
	
	function get_product_id( $product, $variation = false ) { 
		if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
			// Not getting variation ID, but the main product ID
			if ( !$variation && 'variation' == $product->get_type() )
				return $product->get_parent_id();
			else if ( method_exists( $product, 'get_id' ) )
				return $product->get_id();
		} else if ( $variation && !empty( $product->variation_id ) )
			return $product->variation_id; 
		else if ( !empty( $product->id ) )
			return $product->id;
		else 
			return null;
	}
	
	function get_product_parent_id( $product ) { 
		if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
			return $product->get_parent_id();
		}
		return $product->parent;
	}
	
	public static function is_donation( $_product ) {
		global $WC_Donations;

		if ( !is_object( $_product ) )
			return;

		$product_id = $this->get_product_id( $_product, false );

		if ( isset( $WC_Donations ) && ( $d_args = $WC_Donations->is_donation( $product_id ) ) )
			return $d_args;

	}

	// Checks if the product is a gift cert product set where the shopper can set their own price
	public static function is_gift_cert( $product ) {

		$product_id = $this->get_product_id( $_product, false );

 		if ( '1' == get_post_meta( $product_id, 'ignite_gift_enabled', true ) ) {

			if ( '1' == get_post_meta( $product_id, 'ignite_buyer_sets_price', true ) )
				return true;
			else
				return false;

		} else {

			return false;

		}

		return false;

	}

	public function check_for_shortcode( $with_filters = true )  {
		global $post;

		if ( empty( $post ) || empty( $post->post_content ) )
			return;

		if ( false === strpos( $post->post_content, '[' . $this->shortcode ) )
			return;

		if ( !$with_filters )
			return true;

		add_filter( 'woocommerce_available_variation', array( $this, 'set_min_qty' ) , 999, 3 );

		add_filter( 'woocommerce_product_single_add_to_cart_text', array( $this, 'add_to_cart_text' ) , 999, 3 );

		// Add hidden field for Ajax detection
		add_action( 'woocommerce_checkout_before_customer_details', array( &$this, 'before_customer_details' ), 1 );

		$this->opc = true;

		nocache_headers();

		return true;
	}

	public function set_min_qty( $attribs = array(), $_product = false, $_variation = false ) {
		global $product;

		$attribs['min_qty'] = 0;

		return $attribs;
	}

	public function add_to_cart_text( $text = '' ) {
		global $product;

		if ( isset( $product->opc_donation ) )
			return $text;

		return apply_filters( 'ignitewoo_one_page_update_cart_label', __( 'Update cart', 'manual_phone_orders' ) );
	}

	public function array_implode( $glue, $separator, $array ) {

		if ( ! is_array( $array ) )
			return $array;

		$string = array();

		foreach ( $array as $key => $val ) {

			if ( method_exists( $array[ $key ], 'get_options' ) ) { 
			
				$val = $array[ $key ]->get_options();

				if ( !empty( $val[0] ) ) 
					$string[] = $key . $glue . $val[0];
			
			} else { 
				
				if ( is_object( $val ) )
					$val = (array) $val;

				if ( is_array( $val ) )
					$val = implode( ',', $val );

				$string[] = "{$key}{$glue}{$val}";
			}
		}

		return implode( $separator, $string );
	}

	// Load the product template
	public function before_checkout_form() {
		global $products;

		if ( !$this->opc )
			return;

		if ( 'searchable' != $this->style && empty( $this->products ) )
			return;

		if ( empty( $this->products ) ) { 
			$this->products = array();
		}
		
		$products = $this->products;

		// unset( $this->products );
		$this->products = array();

		if ( empty( $this->style ) )
			$this->style = 'searchable';

		$template_path = $default_path = dirname( __FILE__ ) . '/phone-orders/';

		// load template via WC
		switch( $this->style ) {
			/* NOT IMPLEMENTED - POSSIBLE FUTURE LAYOUTS
			case 'list':
				$template_name = 'one_page_list.php';
				$this->load_template( $template_name );
				break;

			case 'table':
				$template_name = 'one_page_table.php';
				$this->load_template( $template_name );
				break;

			case 'single':
				$template_name = 'one_page_single.php';
				$this->load_template( $template_name );
				break;

			case 'single_consolidated':
				$template_name = 'one_page_single_consolidated.php';
				$this->load_template( $template_name );
				break;
			*/
			case 'searchable':
				$template_name = 'one_page_searchable.php';
				$this->load_template( $template_name );
				
				// Add the Configure Product DIV wrapper when the cart table loads
				?>
				<div id="configure_product_wrap" style="display:none"></div>
				<?php 
				break;
			default:
				// Load a custom template file
				//$template_name = $attrs['style'];
				//$this->load_template( $template_name );
				break;
		}
	}

	function ignitewoo_one_page_get_customer() {

		if ( empty( $_REQUEST ) || empty( $_REQUEST['one_page_customer_user'] ) )
			return;

		$usermeta = get_user_meta( absint( $_REQUEST['one_page_customer_user'] ) );

		if ( empty( $usermeta ) )
			return;

		foreach( $usermeta as $key => $data ) {

			if ( 0 === strpos( $key, 'billing_' ) || 0 === strpos( $key, 'shipping_' ) ) {
				$_POST[ $key ] = $data[0];
			}
		}

		add_action( 'woocommerce_checkout_order_review', array( &$this, 'one_page_checkout_after_order_review' ), 1 );
	}

	// Inject a hidden field with the customer ID number ( if any ) so that it is available for processing the order
	function one_page_checkout_after_order_review() {
		$customer_id = !empty( $_REQUEST['one_page_customer_user'] ) ?  $_REQUEST['one_page_customer_user'] : '';
		?>
		<input type="hidden" name="ignitewoo_one_page_checkout_customer_id" value="<?php echo absint( $customer_id ) ?>" />
		<?php
	}

	function checkout_after_order_review() {

		if ( 'searchable' !== $this->style )
			return;

		?>
		<input type="hidden" name="ignitewoo_one_page_checkout" value="searchable" />
		<?php

	}

	// Formerly checkout_before_order_review()
	function add_tax_exempt_field() {
	
		if ( 'searchable' !== $this->style ) { 
			return;
		}
		
		if ( !empty( $_REQUEST['ignitewoo_one_page_tax_exempt'] ) ) { 
			$checked = checked( $_REQUEST['ignitewoo_one_page_tax_exempt'], 'yes', false );
		} else {
			$checked = '';
		}
		
		?>
		<input class="ignitewoo_one_page_tax_exempt" style="margin-left:14px; float:none !important;vertical-align:text-top" type="checkbox" name="ignitewoo_one_page_tax_exempt" value="yes" <?php echo $checked ?>/> <?php _e( 'Tax Exempt', 'manual_phone_orders' )?>
		<?php

	}

	function is_tax_exempt() {
//var_dump( WC()->cart->get_tax_totals() ); die;
		global $ign_tax_exempt, $ign_opc;
//WC()->customer->set_is_vat_exempt( false );
//return;
		$data = array();

		if ( isset( $_POST['post_data'] ) ) {
			$data = wp_parse_args( $_POST['post_data'] );
		} else if ( empty( $_POST['opc_checkout_flag'] ) && empty( $_POST['ignitewoo_one_page_tax_exempt'] ) ) {
//WC()->customer->set_is_vat_exempt( false );
//WC()->session->save_data();
			return;
		}

		//$ign_opc->switch_user_context_to_customer( 'customer' );
		
		// If the IgniteWoo Tax Exempt plugin is in use and the user is set to exempt or and the phone/manual checkbox is checked, set exemption
		if ( ( isset( $data['ignitewoo_one_page_tax_exempt'] ) && 'yes' == $data['ignitewoo_one_page_tax_exempt'] ) || ( isset( $ign_tax_exempt ) && $ign_tax_exempt->is_user_exempt() ) ) {
			WC()->customer->set_is_vat_exempt( true );
		// Otherwise if the phone/manual order exempt box is checked the set exempt
		} else if ( ( isset( $data['ignitewoo_one_page_tax_exempt'] ) && 'yes' == $data['ignitewoo_one_page_tax_exempt'] ) ) {
			WC()->customer->set_is_vat_exempt( true );
		} else if ( ( isset( $_POST['ignitewoo_one_page_tax_exempt'] ) && 'yes' == $_POST['ignitewoo_one_page_tax_exempt'] ) ) {
			WC()->customer->set_is_vat_exempt( true );
		// No exemption set, do nothing, allow other plugins to possibly handle exemption
		} else {
			// We might need to add some code here to allow Wholesale Pro Suite to override exemption, if that plugin is active
			//if ( class_exists( 'IGN_Wholesale_Pro_Suite' ) ) { 
				// Get customer ID from $_POST
				// Get customer role
				// Check if role has_cap( 'no_tax' );
				// If so set exempt
				// If not set NOT exept 
				// 
				// if $role->has_cap( 'no_tax' ) {}
			//}
			//WC()->customer->set_is_vat_exempt( false );
		}
		
		//WC()->session->save_data();
		
		//$ign_opc->switch_user_context_to_customer( 'opc_user' );

	}

	function maybe_add_fee() {

		if ( is_admin() && ! defined( 'DOING_AJAX' ) )
			return;

		if ( isset( $_POST['post_data'] ) )
			$data = wp_parse_args( $_POST['post_data'] );
		else if ( isset( $_POST['opc_fee'] ) )
			$data = $_POST;

		if ( !empty( $data['opc_fee'] ) ) {

			for( $i = 0; $i < count( $data['opc_fee'] ); $i++ ) {
				$amount = $data['opc_fee'][ $i ];
				$label = $data['opc_fee_label'][ $i ];
				$taxable = ( 'yes' == $data['opc_fee_taxable'][ $i ] ) ? true : false;
				WC()->cart->add_fee( $label, $amount, $taxable, '' );
			}
		}
	}

	function maybe_check_fee() {

		if ( empty( $_POST['n'] ) || empty( $_POST['label'] ) || empty( $_POST['amount'] ) )
			die( __( 'Error processing fee. Please provide a label and fee.', 'manual_phone_orders' ) );

		if ( !wp_verify_nonce( $_POST['n'], 'ign-add-fees' ) )
			die( __( 'Security error processing fee', 'manual_phone_orders' ) );

		$amount = floatval( $_POST['amount'] );

		if ( !is_numeric( $amount ) )
			die( __( 'Amount must be valid number', 'manual_phone_orders' ) );

		$label = trim( $_POST['label'] );

		if ( empty( $label ) )
			die( __( 'You must set a unique label', 'manual_phone_orders' ) );

		die( 'ok' );
	}

	public function before_customer_details() {
		?>
		<input type="hidden" value="1" name="opc_checkout_flag">
		<?php

	}
	
	// Used for products that needs special parms set, such as Donations and Gift Certs
	function setup_product( $pid, $_product ) {
		global $WC_Donations, $product, $post;

		$product = $_product;

		unset( $_product );

		if ( 'yes' == get_post_meta( $pid, '_donation', true ) ) {

			$product->opc_donation = $post->opc_donation = true;

			$WC_Donations->setup_product();

		} else {
			// Unset this var in case it was set for another product in the loop
			unset( $product->opc_donation, $post->opc_donation );
		}

		if ( '1' == get_post_meta( $pid, 'ignite_gift_enabled', true ) ) {

			if ( '1' == get_post_meta( $pid, 'ignite_buyer_sets_price', true ) )
				$product->opc_gift_cert = $post->opc_gift_cert = true;
			else
				unset( $product->opc_gift_cert, $post->opc_gift_cert );

		} else {
			unset( $product->opc_gift_cert, $post->opc_gift_cert );
		}

		// Product add-on forms support
		//global $addon_fields;

		$forms = get_post_meta( $pid, '_ign_forms', true );

		if ( !empty( $forms ) )
			$post->opc_product = $product->opc_product = true;
		else {
			// unset( $addon_fields );
			$post->opc_product = $product->opc_product = false;
		}

		return $product;
	}

	public function has_access() {
		global $ign_opc;
		return $ign_opc->has_access();
	}

	// Shortcode processor, loads the products and the checkout page.
	public function one_page( $attrs = array() ) {

		// This GET var is set when editing a page in the WP admin area, so if it's set 
		// do not process the shortcode as this causes a crash when wc_notice_count() is called 
		// due to other objects being unset in WC core.
		if ( ! empty( $_GET['post'] ) ) {
			return;
		}
		
		// Fix for pagebuilders that want to render shortcodes in the admin area, so they don't crash because no frontend code is loaded from WC
		if ( !function_exists( 'wc_notice_count' ) )
			return;
			
		if ( !$this->has_access() ) {
			echo '<p class="ignitewoo_one_page_access_denied" style="font-weight:bold; font-style:italic">' . __( 'You do not have access to this page.', 'manual_phone_orders' ) . '</p>';
			return;
		}

		if ( empty( $this->settings['manual_order_page'] ) )
			return '<p style="clear:both;font-style:italic;margin:30px 20px">' . __( 'You have not finished configuring the phone/manual order settings, review and set the Order Page.', 'manual_phone_orders' ) . '</p>';
			
		if ( 'searchable' == $attrs['style'] ) {

			$this->style = $attrs['style'];

		} else {

			/* Not implemented - possible future mods
			if ( empty( $attrs['products'] ) )
				return;

			$pids = explode( ',' , $attrs['products'] );

			if ( empty( $pids ) )
				return;

			if ( empty( $attrs['style'] ) )
				$this->style = 'table';
			else
				$this->style = $attrs['style'];

			$this->products = array();


			// Load products, unless style is Single, in which case the template loads the products
			if ( 'single_consolidated' == $this->style ) {
				$this->products = $pids;
			} else
			foreach( $pids as $p ) {

				$this->load_product( $p );

			}
			*/
		}

		// Override multistep checkout in Avada theme if enabled. Only for the order page: 
		global $avada_woocommerce;
		if ( !empty( $avada_woocommerce ) ) { 
			// remove_action( 'woocommerce_before_checkout_form', array( $avada_woocommerce, 'before_checkout_form' ), 10 );
			// remove_action( 'woocommerce_after_checkout_form', array( $avada_woocommerce, 'after_checkout_form' ), 10 );
			add_filter( "avada_setting_get_woocommerce_one_page_checkout", array( &$this, 'override_avada_checkout' ), 9999, 1 );
			?>
			<script>
			jQuery( document ).ready( function($) { 
				$( 'div.col-1' ).css( 'display', 'block' ); 
				$( 'div.col-1' ).css( 'float', 'left' );
				$( 'div.col-2' ).css( 'display', 'block' ); 
				$( 'div.col-2' ).css( 'float', 'left' );
				$( '#order_review' ).css( 'display', 'block' );
				$( '.continue-checkout' ).css( 'display', 'none' );
			});
			</script>
			<?php 
		}

		// Add this into Astra child themes to prevent it from duplicating aspects of the public checkout page which affect this plugin 
		// By presenting duplicate "Order notes" fields
		/*
		add_action( 'wp', 'astra_remove_filter_test', 5 );
		function astra_remove_filter_test() { 
			if ( is_page( 'manual-order' ) ) { // use actual page slug
				add_filter( 'astra_woo_shop_product_structure_override', function() { return true; } );
			}
		}
		*/
		
		$this->maybe_disable_terms();

		if ( !empty( $this->settings['full_page'] ) && 'yes' == $this->settings['full_page'] )
			$class = 'ign_opc_full_width';
		else 
			$class = '';
			
		ob_start();

		echo '<div class="woocommerce-checkout ' . $class .'"><div class="woocommerce">';

		// Insert the checkout page
		$checkout = WC()->checkout();

		if ( empty( $_POST ) && wc_notice_count( 'error' ) > 0 ) {

			wc_get_template( 'checkout/cart-errors.php', array( 'checkout' => $checkout ) );

		} else {

			$non_js_checkout = ! empty( $_POST['woocommerce_checkout_update_totals'] ) ? true : false;

			if ( wc_notice_count( 'error' ) == 0 && $non_js_checkout )
				wc_add_notice( __( 'The order totals have been updated. Please confirm your order by pressing the Place Order button at the bottom of the page.', 'manual_phone_orders' ) );

			wc_get_template( 'checkout/form-checkout.php', array( 'checkout' => $checkout ) );

		}

		echo '</div></div>';

		return ob_get_clean();

	}

	public function override_avada_checkout( $setting ) { 
		return 1;  
	}

	
	public function maybe_disable_terms() {

		if ( empty( $this->settings ) )
			$this->settings = get_option( 'woocommerce_ignitewoo_manual_order_settings' );

		if ( isset( $this->settings['disable_terms'] ) && 'yes' == $this->settings['disable_terms'] ) {
			remove_all_filters( 'woocommerce_checkout_show_terms' );
			add_filter( 'woocommerce_checkout_show_terms', array( &$this, 'disable_terms' ), 1, 999999 );
		}

	}

	public function disable_terms( $val = null ) {
		return false;
	}

	public function get_parms( $prod ) {

		$parms = '';

		$variation_data = wc_get_product_variation_attributes( $prod->get_id() );
		
		if ( !empty( $variation_data ) )
			$parms = $this->array_implode( '=', '&', $variation_data );

		return $parms;
	}

	public function load_product( $p ) {

		if ( empty( $p ) || absint( $p ) <= 0 )
			return;

		$prod = wc_get_product( $p );

		if ( empty( $prod ) || !is_a( $prod, 'WC_Product' ) ) { 
			return;
		}
		
		$type = $prod->get_type();

		if ( empty( $type ) || is_wp_error( $prod ) )
			return;

		// "course" if for a Learndash LMS course product 
		if ( in_array( $type, array( 'simple', 'subscription', 'course' ) ) ) {

			// Any visibile attribs?
			// $attributes = $this->map_attributes( $prod );

			$this->products[ $p ] = array( 'data' => $prod, 'attrs' => array(), 'url_parms' => array() );

		} else if ( $prod->is_type( 'grouped' ) ) {

			// Grouped products should have children, in theory
			// and those children are always simple products

			$children = $prod->get_children();

			if ( !empty( $children ) )
			foreach( $children as $child ) {

				$child_product = wc_get_product( $child );

				$this->products[ $child ] = array( 'data' => $child_product, 'attrs' => '', 'url_parms' => '' );
			}

		// Variable parent
		} else if ( in_array( $type, array( 'variable', 'variable-subscription' ) ) ) {

			$children = $prod->get_children();

			if ( !empty( $children ) ) {

				foreach( $children as $child ) {

					$child_product = wc_get_product( $child );

					$attributes = $this->map_attributes( $child_product );

					$parms = $this->get_parms( $child_product );

					$this->products[ $child ] = array( 'data' => $child_product, 'attrs' => $attributes, 'url_parms' => $parms );
				}
			}

		// Variation
		} else if ( $prod->is_type( 'variation' ) ) {

			$attributes = $this->map_attributes( $prod );

			// URL params
			$parms = $this->get_parms( $prod );

			$this->products[ $p ] = array( 'data' => $prod, 'attrs' => $attributes, 'url_parms' => $parms );
		}
		/*
		else if ( $prod->is_type( 'composite' ) ) {

			$cart = WC()->cart->get_cart();

			foreach( $cart as $cart_item_key => $item_data ) {

				$pid = !empty( $item_data['variation_id'] ) ? $item_data['variation_id'] : $item_data['product_id'];
				if ( $p == $pid ) {

					$this->products[ $p ] = array( 'data' => $item_data['data'], 'attrs' => array(), 'url_parms' => array() );
					$this->products[ $p ]['_price_altered'] = $prod->get_price();
					//$this->products[ $p ]['composite_children'] = $item_data['composite_children'];
					//$this->products[ $p ]['composite_data'] = $item_data['data']->get_composite_data();
//var_dump( $item_data['composite_children'] ); die;
					//$this->products[ $p ]['composite_children'] = $prod->();
				}

			}

		}
		*/
		$cart = WC()->cart->get_cart();

		foreach( $cart as $cart_item_key => $item_data ) {

			$pid = !empty( $item_data['variation_id'] ) ? $item_data['variation_id'] : $item_data['product_id'];

			if ( $p == $pid && isset( $item_data['_price_altered'] ) && !empty( $item_data['_price_altered'] ) ) { 
				$this->products[ $p ]['_price_altered'] = $item_data['_price_altered'];
			} else { 
				unset( $this->products[ $p ]['_price_altered'] );
			}

			// Set the custom price for composite products
			/*
			if ( $item_data['data']->is_type( 'composite' ) ) {
				$this->products[ $p ]['_price_altered'] = $item_data['data']->get_price();
			}
			*/

		}

		return $prod; // useful for checking product types etc
	}

	public function load_template( $template_name = '', $template_path = '' ) {
		global $ign_opc;
		
		if ( empty( $template_name  ) )
			return;

		if ( ! $template_path ) {
			$template_path = WC()->template_path();
		}

		$template = locate_template( array( $template_path . 'phone-orders/' . $template_name ), false, false );

		if ( '' != $template )
			require( $template );
		else
			require( $ign_opc->plugin_dir . '/phone-orders/' . $template_name );
	}

	public function maybe_get_phone_manual_order_page_template( $template ) {

		if ( empty( $this->settings ) )
			return $template; 
			
		if ( !is_page( $this->settings['manual_order_page'] ) )
			return $template; 
		
		if ( empty( $this->settings['full_page'] ) || 'yes' !== $this->settings['full_page'] )
			return $template; 
			
		$page_template = $this->load_template( 'one_page_wrapper.php' );
		
		return $page_template;
	}

	public function get_refreshed_fragments() {

		if ( !class_exists( 'WC_AJAX' ) )
			return;

		$wc_ajax = new WC_AJAX();

		$wc_ajax->get_refreshed_fragments();

		unset( $wc_ajax );
	}

	public function update_items_table() {
		global $cart_products, $display_images, $ign_opc;

		//$ign_opc->switch_user_context_to_customer( 'customer' );

		$display_images = true;

		// dummy load
		$cart_products = true;

		$template_name = 'one_page_searchable_items_table.php';

		$this->load_template( $template_name );

		die;
	}

	public static function map_attributes( $prod ) {

		$attributes = array();

		//if ( $prod->is_type( 'variation' ) ) {
			//$data = $prod->variation_data;
			//var_dump( $data );
			//die;
		//}

		$attribs = $prod->get_attributes();

		if ( empty( $attribs ) )
			return $attributes;

		// Name / Value pairs
		foreach ( $attribs as $key => $attr ) {

			if ( $prod->is_type( 'variation' ) && empty( $attr['is_variation'] ) )
				continue;

			$name = wc_attribute_label( $attr['name'] );

			if ( taxonomy_exists( $attr['name'] ) )
				$key = 'attribute_pa_' . $key;
			else
				$key = 'attribute_' . $key;

			$variation_data = wc_get_product_variation_attributes( $prod->variation_id );
			
			if ( isset( $variation_data ) && isset( $attributes[ $name ] ) && isset( $variation_data[ $key ] ) && $attributes[ $name ] )
				$attributes[ $name ] = $variation_data[ $key ];

			continue;
		}

		return $attributes;
	}

	public function display_cart_contents_table( $cart_products = null, $args = null  ) {
		global $cart_products, $display_images, $ign_opc;

		$cart_products = array();

		$cart_items = WC()->cart->get_cart();

		$items = array();

		foreach( $cart_items as $key => $item ) {

			//if ( $item['data']->is_type('composite' ) )
			//	continue;

			if ( $item['data']->is_type( 'variation' ) )
				$items[] = $item['variation_id'];
			else
				$items[] = $item['product_id'];
		}

		//unset( $this->products );
		$this->products = array();
		
		if ( !empty( $items ) ) {

			// Compare to cart, if it's not in the cart then don't load it into the table yet
			// This reloads the $this->products var.
			foreach( $items as $id ) {
				$pdata = $this->load_product( $id );

			}

			if ( !empty( $this->products ) ) {

				$cart_products = $this->products;

			} else {
				$cart_products = null;
			}
		}

		$display_images = true;

		//$ign_opc->switch_user_context_to_customer( 'customer' );
		
		$template_name = 'one_page_searchable_items_table.php';

		$this->load_template( $template_name );
	}

	public function add_shipping_form() {

		$settings = get_option( 'woocommerce_phone_manual_dummy_shipping_settings' );

		if ( empty( $settings['enabled'] ) || 'yes' !== $settings['enabled'] )
			return;

		if ( !defined( 'DOING_AJAX' ) )
			return;

		if ( isset( $_POST['post_data'] )  ) {

			$args = wp_parse_args( $_POST['post_data'] );

			if ( empty( $args ) || empty( $args['opc_checkout_flag'] ) ) { 
				return;
			}
		}

		?>
		<tr>
			<th><?php _e( 'Custom Shipping Cost', 'manual_phone_orders' ) ?></th>
			<td>
				<input type="text" class="custom_shipping_cost" value="" style="width:100px">
				<button class="button set_shipping_cost"><?php _e( 'Set cost', 'manual_phone_orders' )?></button>
				<script> 
				jQuery( document ).ready( function( $ ) { 
					$( '.set_shipping_cost' ).unbind().on( 'click', function(e) {
						e.preventDefault();
						var cost = $( '.custom_shipping_cost' ).val();
						if ( null == cost || '' == cost )
							alert( opc_enhanced_select_params.set_shipping_cost_first );

						$.post( ign_one_page_ajax_url, { action: 'set_custom_shipping_cost', cost: cost }, function( data ) {
							if ( 'ok' == data ) {
								$( document.body ).trigger( 'update_checkout' );
							}

						});
						return false;
					});
				});
				</script>
			</td>
		</tr>
		<?php
	}

	public function set_custom_shipping_cost() {
		global $wpdb;

		if ( !isset( $_POST['cost'] ) )
			die;

		$sql = 'delete from ' . $wpdb->options . ' where ( option_name like "_transient_wc_ship_%" OR option_name like "_transient_timeout_wc_ship_%"  OR option_name like "_transient_shipping_%" )';

		$wpdb->query( $sql );

		$packages = WC()->cart->get_shipping_packages();

		$package_keys 		= array_keys( $packages );
		$package_keys_size 	= sizeof( $package_keys );

		for ( $i = 0; $i < $package_keys_size; $i ++ ) {
			//$this->packages[ $package_keys[ $i ] ] = $this->calculate_shipping_for_package( $packages[ $package_keys[ $i ] ] );
			$package_hash = 'wc_ship_' . md5( json_encode( $packages[ $package_keys[ $i ] ] ) . WC_Cache_Helper::get_transient_version( 'shipping' ) );

			delete_transient( $package_hash );
		}
		
		// Bust the shipping rates cache for WC 3.3.x so that the custom rate is updated properly
		for ( $i = 0; $i < $package_keys_size; $i ++ ) { 
		
			$session_key  = 'shipping_for_package_' . $packages[ $package_keys[ $i ] ];
			
			$the_key = WC()->session->get( $session_key );
			
			if ( empty( $the_key ) )
				continue;
			
			WC()->session->set( $session_key, false );
		}
		
		WC()->session->set( 'opc_custom_shipping_cost', false );
		WC()->session->set( 'opc_custom_shipping_cost', $_POST['cost'] );

		die('ok');
	}

	public function before_checkout_process() {

		if ( sizeof( WC()->cart->get_cart() ) == 0 ) {

			ob_start();

			wc_add_notice( __( 'You have not selected any items yet', 'manual_phone_orders' ), 'error' );

			wc_print_notices();

			$messages = ob_get_clean();

			// If there was an error adding to the cart, redirect to the product page to show any errors
			$data = array(
				'result' => 'failure',
				'messages' => $messages
			);

			echo '<!--WC_START-->'. json_encode( $data ) . '<!--WC_END-->';

			die;   
		}
	}

	public static function get_qty_in_cart( $pid = null, $vid = null, $cart_item_key = false ) {

		$qty = 1;

		// return an array of item quantities in cart
		if ( empty( $pid ) && empty( $vid ) ) {

			if ( sizeof( WC()->cart->get_cart() ) <= 0 )
				return array();

			foreach ( WC()->cart->get_cart() as $key => $item ) {

				if ( !empty( $item['variation_id'] ) )
					$q[ $item['variation_id'] ] = $item['quantity'];
				else
					$q[ $item['product_id'] ] = $item['quantity'];
			}
			
			return $q;

		}

		// get the individual product quantity in cart
		if ( sizeof( WC()->cart->get_cart() ) <= 0 )
			return 1;

		if ( !empty( $cart_item_key ) ) { 
			foreach ( WC()->cart->get_cart() as $key => $item ) {

				if ( $key == $cart_item_key ) { 
						
					$qty = $item['quantity'];

					break;
				}
			}
			
		} else { 
			
			foreach ( WC()->cart->get_cart() as $key => $item ) {

				if ( !empty( $pid ) && $pid !== $item['product_id'] )
					continue;

				if ( !empty( $vid ) && $vid !== $item['variation_id'] )
					continue;

				$qty = $item['quantity'];

				break;

			}
		}

		return $qty;
	}

	public function update_order_review() {
		global $ign_opc;

		if ( empty( $_REQUEST['post_data'] ) )
			return;

		$args = wp_parse_args( $_REQUEST['post_data'] );

		if ( empty( $args['opc_checkout_flag'] ) )
			return;

		$this->opc = true;

		//$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		//check_ajax_referer( 'update-order-review', 'security' );

		//$ign_opc->switch_user_context_to_customer( 'customer' );
		
		if ( isset( $args['ignitewoo_one_page_checkout_customer_id'] ) )
			$_REQUEST['ignitewoo_one_page_checkout_customer_id']  = $args['ignitewoo_one_page_checkout_customer_id'];

		$this->maybe_disable_terms();

		/*
		if ( version_compare( WOOCOMMERCE_VERSION, '2.4', '>=' ) ) {
			remove_action( 'wp_ajax_woocommerce_update_order_review', array( 'WC_AJAX', 'update_order_review' ), 10 );
			remove_action( 'wp_ajax_nopriv_update_order_review', array( 'WC_AJAX', 'update_order_review' ), 10 );
		}
		*/


		if ( 0 == sizeof( WC()->cart->get_cart() ) ) {

			//ob_start();
			//wc_add_notice( __( 'You have not selected any items yet', 'manual_phone_orders' ), 'error' );
			//echo '<div class="woocommerce-info">' . __( 'You have not selected any items yet', 'manual_phone_orders' ) . '</div>';
			//throw new Exception( __( 'You have not selected any items yet', 'manual_phone_orders' ) );
			//wc_print_notices();
			$messages = ''; //ob_get_clean();

			// Get order review fragment
			ob_start();
			woocommerce_order_review();
			$woocommerce_order_review = ob_get_clean();

			// Get checkout payment fragment
			ob_start();
			woocommerce_checkout_payment();
			$woocommerce_checkout_payment = ob_get_clean();


			$data = array(
				'result'    => empty( $messages ) ? 'success' : 'failure',
				'messages'  => $messages,
				'reload'    => isset( WC()->session->reload_checkout ) ? 'true' : 'false',
				'fragments' => apply_filters( 'woocommerce_update_order_review_fragments', array(
					'.woocommerce-checkout-review-order-table' => $woocommerce_order_review,
					'.woocommerce-checkout-payment'            => $woocommerce_checkout_payment
				) )
			);

			// Send JSON and exit
			wp_send_json( $data );

			//$ign_opc->switch_user_context_to_customer( 'opc_user' );

			//die(0);
		}
	
		$this->is_tax_exempt();
		
		foreach( WC()->cart->get_cart() as $cart_item_key => $values ) {

			$cik = $cart_item_key;

			$price = isset( $values['_price_altered'] ) ? $values['_price_altered'] : null;

			if ( null !== $price ) {
			
				WC()->cart->cart_contents[ $cik ]['data']->price_altered = $price;
				
				if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '>=' ) ) {
					WC()->cart->cart_contents[ $cik ]['data']->set_price( $price );
					WC()->cart->cart_contents[ $cik ]['data']->set_regular_price( $price );
					WC()->cart->cart_contents[ $cik ]['data']->set_sale_price( '' );

				} else {
					WC()->cart->cart_contents[ $cik ]['data']->price = $price;
					WC()->cart->cart_contents[ $cik ]['data']->regular_price = $price;
					WC()->cart->cart_contents[ $cik ]['data']->sale_price = $price;
				}

				WC()->cart->set_session();
			}
		}
		
		//$ign_opc->switch_user_context_to_customer( 'customer' );

		// For versions 2.4 and newer the JS has changed, so use this as taken from class-wc-ajax.php "function update_order_review()" to
		// avoid the "Sorry, your session has expired." message being displayed.

		if ( ! defined( 'WOOCOMMERCE_CHECKOUT' ) ) {
			define( 'WOOCOMMERCE_CHECKOUT', true );
		}

		// Unhook WooFunnels OrderBumps: WooCommerce Checkout Offers plugin function, it breaks Ajax nonce security
		if ( class_exists( 'WFOB_AJAX_Controller') ) { 
			remove_action( 'woocommerce_checkout_update_order_review', 'WFOB_AJAX_Controller::check_bump_action', -10 );
		}

		$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		do_action( 'woocommerce_checkout_update_order_review', $_POST['post_data'] );
		
		$ign_opc->switch_user_context_to_customer( 'customer' );

		$chosen_shipping_methods = WC()->session->get( 'chosen_shipping_methods' );

		if ( isset( $_POST['shipping_method'] ) && is_array( $_POST['shipping_method'] ) ) {
			foreach ( $_POST['shipping_method'] as $i => $value ) {
				$chosen_shipping_methods[ $i ] = wc_clean( $value );
			}
		}

		WC()->session->set( 'chosen_shipping_methods', $chosen_shipping_methods );
		WC()->session->set( 'chosen_payment_method', empty( $_POST['payment_method'] ) ? '' : wc_clean( wp_unslash( $_POST['payment_method'] ) ) );
		
		WC()->customer->set_props(
			array(
				'billing_country'   => isset( $_POST['country'] ) ? wc_clean( wp_unslash( $_POST['country'] ) ) : null,
				'billing_state'     => isset( $_POST['state'] ) ? wc_clean( wp_unslash( $_POST['state'] ) ) : null,
				'billing_postcode'  => isset( $_POST['postcode'] ) ? wc_clean( wp_unslash( $_POST['postcode'] ) ) : null,
				'billing_city'      => isset( $_POST['city'] ) ? wc_clean( wp_unslash( $_POST['city'] ) ) : null,
				'billing_address_1' => isset( $_POST['address'] ) ? wc_clean( wp_unslash( $_POST['address'] ) ) : null,
				'billing_address_2' => isset( $_POST['address_2'] ) ? wc_clean( wp_unslash( $_POST['address_2'] ) ) : null,
			)
		);

		if ( wc_ship_to_billing_address_only() ) {
			WC()->customer->set_props(
				array(
					'shipping_country'   => isset( $_POST['country'] ) ? wc_clean( wp_unslash( $_POST['country'] ) ) : null,
					'shipping_state'     => isset( $_POST['state'] ) ? wc_clean( wp_unslash( $_POST['state'] ) ) : null,
					'shipping_postcode'  => isset( $_POST['postcode'] ) ? wc_clean( wp_unslash( $_POST['postcode'] ) ) : null,
					'shipping_city'      => isset( $_POST['city'] ) ? wc_clean( wp_unslash( $_POST['city'] ) ) : null,
					'shipping_address_1' => isset( $_POST['address'] ) ? wc_clean( wp_unslash( $_POST['address'] ) ) : null,
					'shipping_address_2' => isset( $_POST['address_2'] ) ? wc_clean( wp_unslash( $_POST['address_2'] ) ) : null,
				)
			);
		} else {
			WC()->customer->set_props(
				array(
					'shipping_country'   => isset( $_POST['s_country'] ) ? wc_clean( wp_unslash( $_POST['s_country'] ) ) : null,
					'shipping_state'     => isset( $_POST['s_state'] ) ? wc_clean( wp_unslash( $_POST['s_state'] ) ) : null,
					'shipping_postcode'  => isset( $_POST['s_postcode'] ) ? wc_clean( wp_unslash( $_POST['s_postcode'] ) ) : null,
					'shipping_city'      => isset( $_POST['s_city'] ) ? wc_clean( wp_unslash( $_POST['s_city'] ) ) : null,
					'shipping_address_1' => isset( $_POST['s_address'] ) ? wc_clean( wp_unslash( $_POST['s_address'] ) ) : null,
					'shipping_address_2' => isset( $_POST['s_address_2'] ) ? wc_clean( wp_unslash( $_POST['s_address_2'] ) ) : null,
				)
			);
		}

		// Calculate shipping before totals. This will ensure any shipping methods that affect things like taxes are chosen prior to final totals being calculated. Ref: WC Issue #22708.
		WC()->cart->calculate_shipping();
		WC()->cart->calculate_totals();

		// Get order review fragment.
		ob_start();
		woocommerce_order_review();
		$woocommerce_order_review = ob_get_clean();

		// Get checkout payment fragment.
		ob_start();
		woocommerce_checkout_payment();
		$woocommerce_checkout_payment = ob_get_clean();

		// Get messages if reload checkout is not true.
		$reload_checkout = isset( WC()->session->reload_checkout ) ? true : false;
		if ( ! $reload_checkout ) {
			$messages = wc_print_notices( true );
		} else {
			$messages = '';
		}

		unset( WC()->session->refresh_totals, WC()->session->reload_checkout );

		wp_send_json(
			array(
				'result'    => empty( $messages ) ? 'success' : 'failure',
				'messages'  => $messages,
				'reload'    => $reload_checkout,
				'fragments' => apply_filters(
					'woocommerce_update_order_review_fragments',
					array(
						'.woocommerce-checkout-review-order-table' => $woocommerce_order_review,
						'.woocommerce-checkout-payment' => $woocommerce_checkout_payment,
					)
				),
			)
		);
	
	}

	
	function checkout_billing() {
		global $user_ID, $ign_opc, $ign_opc_core;

		if ( !$ign_opc_core->opc )
			return;

		remove_action( 'woocommerce_checkout_billing' , array( WC()->checkout, 'checkout_form_billing' ) );

		// Check theme
		$template = locate_template( array( 'phone-orders/form-billing.php' ), false, false );

		if ( '' != $template ) {
			require( $template );
		} else if ( file_exists( dirname( __FILE__ ) . '/phone-orders/form-billing.php' ) ) {
			$checkout = WC()->checkout;
			require( dirname( __FILE__ ) . '/phone-orders/form-billing.php' );
		} else { // fallback to stock
			wc_get_template( 'checkout/form-billing.php', array( 'checkout' =>  WC()->checkout ) );
		}

		echo '<input type="hidden" name="opc_admin_user_id" value="' . $ign_opc->opc_user_id . '">';
	}

	function checkout_shipping() {
		global $ign_opc_core;
		
		if ( !$ign_opc_core->opc )
			return;

		remove_action( 'woocommerce_checkout_shipping' , array( WC()->checkout, 'checkout_form_shipping' ), 10 );

		// Check theme
		$template = locate_template( array( 'phone-orders/form-shipping.php' ), false, false );

		if ( '' != $template ) {
			require( $template );
		} else if ( file_exists( dirname( __FILE__ ) . '/phone-orders/form-shipping.php' ) ) {
			$checkout = WC()->checkout;
			require( dirname( __FILE__ ) . '/phone-orders/form-shipping.php' );
		} else { // fallback to stock
			wc_get_template( 'checkout/form-shipping.php', array( 'checkout' =>  WC()->checkout ) );
		}
	}

	function thankyou_page_additions( $order_id = null ) {
		global $user_ID;

		if ( empty( $order_id ) )
			return;

		if ( !$this->has_access() )
			return;

		if ( empty( $this->settings ) )
			$this->settings = get_option( 'woocommerce_ignitewoo_manual_order_settings' );

		$url = admin_url() . '/post.php?post=' . $order_id . '&action=edit';

		if ( isset( $this->settings['manual_order_page'] ) && !empty( $this->settings['manual_order_page'] ) )
			$aurl = get_permalink( $this->settings['manual_order_page'] );
		else
			$aurl = '';

		?>

		<div class="opc_thankyou_page_additions" style="background: #fafafa none repeat scroll 0 0;border: 1px solid #ccc;font-size: 1.1em;font-weight: bold;padding: 1em;border-radius:4px">
		
			<div style="text-align:left;display:inline-block; width:49%">
				<a class="btn button" href="<?php echo $url ?>"><?php _e( 'View this order in the admin area', 'manual_phone_orders' ) ?></a>
			</div>
			<?php if ( !empty( $aurl ) ) { ?>
			<div style="text-align:right;;display:inline-block;; width:50%">
				&nbsp; &nbsp; <a class="btn button" href="<?php echo $aurl ?>"><?php _e( 'Add new manual order', 'manual_phone_orders' ) ?></a>
			</div>
			<?php } ?>
		</div>

		<?php
		
		// Look at each item in the order to see if it has meta data indicating it's an arbitrary product 
		// and if so delete the product 
		
		$order = wc_get_order( $order_id );
		
		// If the Hold Order gateway was used it puts the order on hold, so don't try to delete any arbitrary products 
		if ( 'on-hold' == $order->get_status() ) {
			return;
		}
	}

	public function wc_ajax_endpoint() {

		if ( !method_exists( 'WC_AJAX', 'get_endpoint' ) )
			return WC()->ajax_url();

		return WC_AJAX::get_endpoint( "%%endpoint%%" );

	}
	
	// Borrowed from WC core code. Because it works.
	public function load_scripts_styles() {
		global $post, $wp, $ign_opc; 

		if ( empty( $this->opc ) || !$this->opc )
			return;

		// Ensure that we're in the OPC user context so that nonces are created correctly
		$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		wp_enqueue_script( 'wc-single-product' );

		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		$assets_path = str_replace( array( 'http:', 'https:' ), '', WC()->plugin_url() ) . '/assets/';

		$frontend_script_path = $assets_path . 'js/frontend/';

		wp_enqueue_style( 'dashicons' );
		
		wp_deregister_script( 'chosen' );

		wp_register_script( 'jquery-blockui', $assets_path . 'js/jquery-blockui/jquery.blockUI' . $suffix . '.js', array( 'jquery' ), '2.60', true );

		wp_register_script( 'jquery-payment', $assets_path . 'js/jquery-payment/jquery.payment' . $suffix . '.js', array( 'jquery' ), '1.0.2', true );

		wp_register_script( 'wc-credit-card-form', $assets_path . 'js/frontend/credit-card-form' . $suffix . '.js', array( 'jquery', 'jquery-payment' ), WC_VERSION, true );

		wp_register_script( 'wc-add-to-cart-variation', $frontend_script_path . 'add-to-cart-variation' . $suffix . '.js', array( 'jquery' ), WC_VERSION, true );

		wp_register_script( 'wc-single-product', $frontend_script_path . 'single-product' . $suffix . '.js', array( 'jquery' ), WC_VERSION, true );

		wp_register_script( 'wc-country-select', $frontend_script_path . 'country-select' . $suffix . '.js', array( 'jquery' ), WC_VERSION, true );

		wp_register_script( 'wc-address-i18n', $frontend_script_path . 'address-i18n' . $suffix . '.js', array( 'jquery' ), WC_VERSION, true );

		wp_register_script( 'jquery-cookie', $assets_path . 'js/jquery-cookie/jquery.cookie' . $suffix . '.js', array( 'jquery' ), '1.3.1', true );

		wp_enqueue_script( 'wc-add-to-cart', $frontend_script_path . 'add-to-cart' . $suffix . '.js', array( 'jquery' ), WC_VERSION, true );

		// This causes timing problems because update_order_review fires off, then update_shipping_method fire off too, so 
		// we extracted the coupon related code from the cart.js and added it to the ignitewoo-opc.js file
		// wp_enqueue_script( 'wc-cart', $frontend_script_path . 'cart' . $suffix . '.js', array( 'jquery', 'wc-country-select' ), WC_VERSION, true );
		
		wp_register_script( 'jquery-tiptip', WC()->plugin_url() . '/assets/js/jquery-tiptip/jquery.tipTip' . $suffix . '.js', array( 'jquery' ), WC_VERSION, true );
		
		wp_enqueue_script( 'wc-checkout', $frontend_script_path . 'checkout' . $suffix . '.js', array( 'jquery', 'woocommerce', 'wc-country-select', 'wc-address-i18n' ), WC_VERSION, true );

		if ( is_add_payment_method_page() ) { 
			wp_enqueue_script( 'wc-add-payment-method', $frontend_script_path . 'add-payment-method' . $suffix . '.js', array( 'jquery', 'woocommerce' ), WC_VERSION, true );
		}
	
		wp_enqueue_script( 'wc-cart-fragments', $frontend_script_path . 'cart-fragments' . $suffix . '.js', array( 'jquery', 'jquery-cookie' ), WC_VERSION, true );

		// Variables for JS scripts
		wp_localize_script( 'woocommerce', 'woocommerce_params', apply_filters( 'woocommerce_params', array(
			'ajax_url'        => WC()->ajax_url(),
			'wc_ajax_url'     => WC()->ajax_url(),
			'ajax_loader_url' => $ign_opc->plugin_url() . '/assets/ajax-loader@2x.gif'
		) ) );

		wp_localize_script( 'wc-single-product', 'wc_single_product_params', apply_filters( 'wc_single_product_params', array(
			'i18n_required_rating_text' => esc_attr__( 'Please select a rating', 'manual_phone_orders' ),
			'review_rating_required'    => get_option( 'woocommerce_review_rating_required' ),
		) ) );

		wp_localize_script( 'wc-checkout', 'wc_checkout_params', apply_filters( 'wc_checkout_params', array(
			'ajax_url'                  => WC()->ajax_url(),
			'wc_ajax_url'      	    => $this->wc_ajax_endpoint(),
			'ajax_loader_url'           => $ign_opc->plugin_url() . '/assets/ajax-loader@2x.gif',
			'update_order_review_nonce' => wp_create_nonce( "update-order-review" ),
			'apply_coupon_nonce'        => wp_create_nonce( "apply-coupon" ),
			'remove_coupon_nonce'       => wp_create_nonce( 'remove-coupon' ),
			'option_guest_checkout'     => get_option( 'woocommerce_enable_guest_checkout' ),
			'checkout_url'              => add_query_arg( 'action', 'woocommerce_checkout', WC()->ajax_url() ),
			'is_checkout'               => is_page( wc_get_page_id( 'checkout' ) ) && empty( $wp->query_vars['order-pay'] ) && ! isset( $wp->query_vars['order-received'] ) ? 1 : 0
		) ) );

		wp_localize_script( 'wc-address-i18n', 'wc_address_i18n_params', apply_filters( 'wc_address_i18n_params', array(
			'locale'                    => json_encode( WC()->countries->get_country_locale() ),
			'locale_fields'             => json_encode( WC()->countries->get_country_locale_field_selectors() ),
			'i18n_required_text'        => esc_attr__( 'required', 'manual_phone_orders' ),
		) ) );

		wp_localize_script( 'wc-cart', 'wc_cart_params', apply_filters( 'wc_cart_params', array(
			'ajax_url'                     => WC()->ajax_url(),
			'wc_ajax_url'                  => WC()->ajax_url(),
			'ajax_loader_url'              => $ign_opc->plugin_url() . '/assets/ajax-loader@2x.gif',
			'update_shipping_method_nonce' => wp_create_nonce( "update-shipping-method" ),
		) ) );

		wp_localize_script( 'wc-cart-fragments', 'wc_cart_fragments_params', apply_filters( 'wc_cart_fragments_params', array(
			'ajax_url'      => WC()->ajax_url(),
			'wc_ajax_url'   => $this->wc_ajax_endpoint(),
			'fragment_name' => apply_filters( 'woocommerce_cart_fragment_name', 'wc_fragments' )
		) ) );

		wp_localize_script( 'wc-add-to-cart', 'wc_add_to_cart_params', apply_filters( 'wc_add_to_cart_params', array(
			'ajax_url'                => WC()->ajax_url(),
			'wc_ajax_url'             => WC()->ajax_url(),
			'ajax_loader_url'         => $ign_opc->plugin_url() . '/assets/ajax-loader@2x.gif',
			'i18n_view_cart'          => esc_attr__( 'View Cart', 'manual_phone_orders' ),
			'cart_url'                => get_permalink( wc_get_page_id( 'cart' ) ),
			'is_cart'                 => is_cart(),
			'cart_redirect_after_add' => get_option( 'woocommerce_cart_redirect_after_add' )
		) ) );

		wp_localize_script( 'wc-add-to-cart-variation', 'wc_add_to_cart_variation_params', apply_filters( 'wc_add_to_cart_variation_params', array(
			'i18n_no_matching_variations_text' => esc_attr__( 'Sorry, no products matched your selection. Please choose a different combination.', 'manual_phone_orders' ),
			'i18n_unavailable_text'            => esc_attr__( 'Sorry, this product is unavailable. Please choose a different combination.', 'manual_phone_orders' ),
		) ) );

		if ( version_compare( WOOCOMMERCE_VERSION, '3.2', '>=' ) ) {
			// Remove select2 in case a theme or other plugin added it

			wp_dequeue_style( 'select2' );
			wp_dequeue_script( 'select2' );
			wp_deregister_style( 'select2' );
			wp_deregister_script( 'select2' );
			
			wp_enqueue_style( 'select2', WC()->plugin_url() . '/assets/css/select2.css' );
			wp_enqueue_script( 'select2', WC()->plugin_url() . '/assets/js/selectWoo/selectWoo.full.min.js', array( 'jquery' ), WC_VERSION, true );
			
		} else if ( version_compare( WOOCOMMERCE_VERSION, '2.4', '>=' ) ) {
			// Remove select2 in case a theme or other plugin added it
			wp_dequeue_style( 'select2' );
			wp_dequeue_script( 'select2' );
			wp_deregister_style( 'select2' );
			wp_deregister_script( 'select2' );

			wp_enqueue_style( 'select2', WC()->plugin_url() . '/assets/css/select2.css' );
			wp_enqueue_script( 'select2', WC()->plugin_url() . '/assets/js/frontend/select2/select2.min.js', array( 'jquery' ), WC_VERSION, true );
			wp_enqueue_script( 'wc-country-select', WC()->plugin_url() . '/assets/js/frontend/country-select.js',  array( 'jquery' ), WC_VERSION, true );
		}
				
		wp_localize_script( 'wc-country-select', 'wc_country_select_params', apply_filters( 'wc_country_select_params', array(
			'countries'              => json_encode( array_merge( WC()->countries->get_allowed_country_states(), WC()->countries->get_shipping_country_states() ) ),
			'i18n_select_state_text' => esc_attr__( 'Select an option&hellip;', 'manual_phone_orders' ),
		) ) );

		wp_enqueue_script( 'wc-credit-card-form', array( 'jquery', 'jquery-payment' ), WC_VERSION );
		
		// CSS Styles
		$enqueue_styles = WC_Frontend_Scripts::get_styles();

		if ( $enqueue_styles )
			foreach ( $enqueue_styles as $handle => $args )
				wp_enqueue_style( $handle, $args['src'], $args['deps'], $args['version'], $args['media'] );

		$js_args = array( 
			'search_products_nonce' => wp_create_nonce('search-products'),
			'search_customers_nonce' => wp_create_nonce('search-customers'),
			'add_products_nonce' => wp_create_nonce('ign-add-products'),
			'add_fees_nonce' => wp_create_nonce('ign-add-fees'),
		);

		wp_localize_script( 'opc-script', 'ign_one_page_checkout', $js_args );
		
		/*
		$js = "ign_one_page_checkout = {
			search_products_nonce: '". wp_create_nonce('search-products') . "',
			search_customers_nonce: '". wp_create_nonce('search-customers') . "',
			add_products_nonce: '" . wp_create_nonce('ign-add-products') . "',
			add_fees_nonce: '" . wp_create_nonce('ign-add-fees') . "',
			};";

		wc_enqueue_js( $js );
		*/
	}

	function add_customer_company_search( $args, $term, $limit, $type = 'meta_query' ) { 
	
		// don't filter the query unless this plugin is running the ajax query
		if ( !isset( $_GET['opc'] ) || $_GET['opc'] != 'true' ) {
			return $args;
		}
			
		if ( 'meta_query' !== $type ) { 
			return $args;
		}
		
		$args['meta_query'][] = array(
			array(
				'key'     => 'billing_company',
				'value'   => $term,
				'compare' => 'LIKE',
			),
		);
		
		return $args; 
	}
	
	public function json_search_customers() {

		if ( !$this->has_access() )
			die( '-1' );
			
		ob_start();

		//check_ajax_referer( 'search-customers', 'security' );

		$term    = wc_clean( wp_unslash( $_GET['term'] ) );
		$exclude = array();
		$limit   = '';

		if ( empty( $term ) ) {
			die();
		}

		if ( ! empty( $_GET['exclude'] ) ) {
			$exclude = array_map( 'intval', explode( ',', $_GET['exclude'] ) );
		}

		$found_customers = array();

		$term    = wc_clean( wp_unslash( $_GET['term'] ) );
		$exclude = array();
		$limit   = '';

		if ( empty( $term ) ) {
			wp_die();
		}

		// Usernames can be numeric so we first check that no users was found by ID before searching for numeric username, this prevents performance issues with ID lookups.
		// WC 3.0 and newer
		if ( class_exists( 'WC_Data_Store' ) ) { 
			
			$ids = array();

			// Search by ID.
			if ( is_numeric( $term ) ) {
				$customer = new WC_Customer( intval( $term ) );

				// In case customer does not exists.
				if ( 0 !== $customer->get_id() ) {
					$ids = array( $customer->get_id() );
				}
			}
			
			// Always search other fields
			//if ( empty( $ids ) ) {
				$data_store = WC_Data_Store::load( 'customer' );

				// If search is smaller than 3 characters, limit result set to avoid
				// too many rows being returned.
				if ( 3 > strlen( $term ) ) {
					$limit = 20;
				}
				$other_ids = $data_store->search_customers( $term, $limit );
			//}
			
			if ( !empty( $ids ) && !empty( $other_ids ) ) { 
				$ids = array_unique( array_merge( $ids, $other_ids ) );
			}  else if ( !empty( $other_ids ) ) { 
				$ids = $other_ids;
			}
			
		// WC 2.6 and older
		} else { 
		
			add_action( 'pre_user_query', array( __CLASS__, 'json_search_customer_name' ) );

			$customers_query = new WP_User_Query( apply_filters( 'woocommerce_json_search_customers_query', array(
				'fields'         => 'all',
				'orderby'        => 'display_name',
				'search'         => '*' . $term . '*',
				'search_columns' => array( 'ID', 'user_login', 'user_email', 'user_nicename' )
			) ) );

			remove_action( 'pre_user_query', array( __CLASS__, 'json_search_customer_name' ) );

			$customers = $customers_query->get_results();

			if ( ! empty( $customers ) ) {
				foreach ( $customers as $customer ) {
					if ( ! in_array( $customer->ID, $exclude ) ) {
						$found_customers[ $customer->ID ] = $customer->display_name . ' (#' . $customer->ID . ' &ndash; ' . sanitize_email( $customer->user_email ) . ')';
					}
				}
			}
		}

		if ( ! empty( $_GET['exclude'] ) ) {
			$ids = array_diff( $ids, (array) $_GET['exclude'] );
		}

		// Do a separate customer search by company because the WC search becomes incredibly slow if we 
		// simply add a meta field to the initial query via the "woocommerce_customer_search_customers" filter hook 
		if ( !empty( $this->settings['search_by_company'] ) && 'yes' == $this->settings['search_by_company'] ) { 

			$query2 = new WP_User_Query(
				array(
					'fields'     => 'ID',
					'number'     => $limit,
					'meta_query' => array(
						'relation' => 'OR',
						array(
							'key'     => 'billing_company',
							'value'   => $term,
							'compare' => 'LIKE',
						),
						array(
							'key'     => 'shipping_company',
							'value'   => $term,
							'compare' => 'LIKE',
						),
					),
				)
			);

			$ids = wp_parse_id_list( array_merge( (array) $query2->get_results(), (array) $ids ) );
		}
		
		foreach ( $ids as $id ) {
			$customer = new WC_Customer( $id );
			/* translators: 1: user display name 2: user ID 3: user email, company name 4 */
			$co_name = $customer->get_billing_company();
			
			if ( !empty( $co_name ) ) { 
				$found_customers[ $id ] = sprintf(
					esc_html__( '%1$s (#%2$s &ndash; %3$s) &ndash; ' . $co_name, 'manual_phone_orders' ),
					$customer->get_first_name() . ' ' . $customer->get_last_name(),
					$id,
					$customer->get_email()
				);
			} else { 
				$found_customers[ $id ] = sprintf(
					esc_html__( '%1$s (#%2$s &ndash; %3$s)', 'manual_phone_orders' ),
					$customer->get_first_name() . ' ' . $customer->get_last_name(),
					$id,
					$customer->get_email()
				);
			}
		}

		$found_customers = apply_filters( 'woocommerce_json_search_found_customers', $found_customers, trim( $_GET['term'] ) );
		
		$found_customers = apply_filters( 'ignitewoo_opc_json_search_found_customers', $found_customers );
		
		wp_send_json( $found_customers );
		
	}
	
	// Backward compat for WC version before2.6
	/*
	function filter_user_search( $args = array(), $term = false, $limit = false, $type = false  ) {

		if ( !$this->has_access() )
			return $args;
		
		
		// don't filter the query unless this plugin is running the ajax query
		if ( !isset( $_GET['opc'] ) || $_GET['opc'] != 'true' )
			return $args;
		
		// admins are limited, everyone else is
		if ( current_user_can( 'administrator' ) )
			return $args;

		// no one except admins can query for admin accounts
		$admins_query = new WP_User_Query( array(
			'fields'         	=> 'ID',
			'orderby'        	=> 'ID',
			'role' 		=> 'administrator',
		) );

		// If the current user is not a shop manager then exclude shop manager accounts too
		if ( !current_user_can( 'shop_manager' ) && !current_user_can( 'administrator' ) ) {
			$shop_query = new WP_User_Query( array(
				'fields'         => 'ID',
				'orderby'        => 'ID',
				'role' => 'shop_manager',
			) );
		} else  {

			$shop_query = new stdClass();
			$shop_query->results = array();
		}

		$res = array_unique( array_merge( $admins_query->results, $shop_query->results ) );

		if ( empty( $res ) || !is_array( $res ) )
			$res = array();

		$args['exclude'] = $res;

			
		return $args;

	}
	*/

	public function get_chosen_product_input( $die = false, $return_parms = false ) {

		/* Older way where Select2 remained populated
		<input type="hidden" class="opc-product-search" data-multiple="true" style="width: 100%;" name="product_ids[]" data-placeholder="<?php esc_attr_e( 'Choose products &hellip;', 'manual_phone_orders' ); ?>" data-action="woocommerce_json_search_products_and_variations" data-selected="<?php
				//$product_ids = array_filter( array_map( 'absint', explode( ',', get_post_meta( $post->ID, 'product_ids', true ) ) ) );
				$product_ids = $items;
				$json_ids    = array();

				foreach ( $product_ids as $product_id ) {
					$product = wc_get_product( $product_id );
					if ( is_object( $product ) ) {
						$json_ids[ $product_id ] = wp_kses_post( $product->get_formatted_name() );
					}
				}

				echo esc_attr( json_encode( $json_ids ) );
			?>" value="<?php echo implode( ',', array_keys( $json_ids ) ); ?>" />
		*/
		if ( version_compare( WOOCOMMERCE_VERSION, '2.7', '<' ) ) {

			$cart_items = WC()->cart->get_cart();

			$items = array();

			foreach( $cart_items as $key => $item ) {

				if ( $item['data']->is_type( 'variation' ) ) {

					$items[] = $item['variation_id'];

				} else {

					$items[] = $item['product_id'];
				}
			}

			//if ( !empty( $_POST['product_ids'] ) )
			//	$items = array_merge( $items, $_POST['product_ids'] );

			$items = array_unique( $items, SORT_REGULAR );

			?>
			<input type="hidden" class="opc-product-search" data-multiple="false" style="width: 100%;" name="product_ids[]" data-placeholder="<?php esc_attr_e( 'Choose products &hellip;', 'manual_phone_orders' ); ?>" data-action="woocommerce_json_search_products_and_variations" data-selected="<?php
					//$product_ids = array_filter( array_map( 'absint', explode( ',', get_post_meta( $post->ID, 'product_ids', true ) ) ) );
					$product_ids = $items;
					$json_ids    = array();

					foreach ( $product_ids as $product_id ) {
						$product = wc_get_product( $product_id );
						if ( is_object( $product ) ) {
							$json_ids[ $product_id ] = wp_kses_post( $product->get_formatted_name() );
						}
					}

					echo esc_attr( json_encode( $json_ids ) );
				?>" value="" />

		<?php } else { ?>

			<div>
				<select type="text" name="product_ids[]" class="opc-product-search" data-placeholder="<?php _e( 'Choose products &hellip;', 'manual_phone_orders' ) ?>" style="width: 58% !important;" multiple="multiple">
				</select>
				
				<?php 
				
				// Added setting to select how the category list is ordered: ID, Name, Term Order
				// See this post for how to maybe do it for Term Order since WP doesn't handle it quite as expected: 
				// https://wordpress.stackexchange.com/questions/92213/order-terms-by-term-order
				// 
				// INTERESTING NOTE: If "orderby" is set to "name" then the list comes back in the term order as listed 
				// on the product cats page in the admin area. Hrrrrm. We'll use that fact.
				
				$orderby = isset( $this->settings['category_order'] ) ? $this->settings['category_order'] : 'term_name';

				$args = array(
					'show_option_all'    => '',
					'show_option_none'   => __( 'Narrow search by category', 'manual_phone_orders' ),
					'option_none_value'  => '-1',
					'orderby'            => $orderby,
					'order'              => 'ASC',
					'show_count'         => 0,
					'hide_empty'         => 1,
					'child_of'           => 0,
					'exclude'            => '',
					'include'            => '',
					'echo'               => 1,
					'selected'           => 0,
					'hierarchical'       => 1,
					'name'               => 'cat',
					'id'                 => '',
					'class'              => 'opc-search-cats',
					'depth'              => 0,
					'tab_index'          => 0,
					'taxonomy'           => 'product_cat',
					'hide_if_empty'      => false,
					'value_field'	     => 'term_id',
					);
				wp_dropdown_categories( $args );
				?>
			</div>
		<?php } ?>

		<?php

		if ( $return_parms )
			return $ids;

		if ( $die )
			die;

	}

	function user_can_change_price() {
		global $user_ID, $ign_opc;

		$can_change_price = true;

		$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		if ( current_user_can( 'administrator' ) || current_user_can( 'shop_manager' ) )
			return true;

		if ( empty( $this->settings ) )
			$this->settings = get_option( 'woocommerce_ignitewoo_manual_order_settings' );

		if ( !empty( $this->settings['change_price_users'] ) ) {
			if ( !in_array( $user_ID,  $this->settings['change_price_users'] ) )
				$can_change_price = false;
		}

		return $can_change_price;
	}

	function add_clerk_info( $order_id, $posted = null ) {
		global $current_user, $ign_opc; 

		if ( empty( $_REQUEST['opc_checkout_flag'] ) )
			return;

		/*
		if ( empty( $current_user ) && function_exists( 'wp_get_current_user' ) )
			$current_user = wp_get_current_user();
		else if ( empty( $current_user ) )
			$current_user = get_currentuserinfo();
		*/
		
		$ign_opc->switch_user_context_to_customer( 'opc_user' );
		
		$order = wc_get_order( $order_id );
		
		$order->update_meta_data( '_ign_one_page_clerk', $current_user->ID );
		$order->save();

		$ign_opc->switch_user_context_to_customer( 'customer' );
	}

	function clerk_info_box( $post_type = false, $the_post = false ) {
		global $post;

		if ( 'woocommerce_page_wc-orders' === $post_type ) {
			if ( empty( $the_post->get_id() ) ) { 
				return;
			}
			$pid = $the_post->get_id();
		} else if ( is_int( $the_post ) ) { 
			$pid = $the_post;
		} else if ( ! empty( $the_post ) && is_object( $the_post) && ! empty( $the_post->ID ) ) { 
			$pid = $post->ID;
		}

		if ( empty( $pid ) ) { 
			return;
		}

		$order = wc_get_order( $pid );
		
		if ( ! is_a( $order, 'WC_Order' ) ) { 
			return;
		}
		
		$clerk = $order->get_meta( '_ign_one_page_clerk', true );

		if ( false == $clerk ) {
			return;
		}

		add_meta_box(
			'ign-one-page-clerk',
			__( 'Manual / Phone Order Clerk','manual_phone_orders' ),
			array(&$this, 'clerk_info_box_data' ),
			'shop_order',
			'side',
			'default'
		);
		
		/**
		 * Add metabox for sites using HPOS.
		 */
		add_meta_box(
			'ign-one-page-clerk',
			__( 'Manual / Phone Order Clerk','manual_phone_orders' ),
			array(&$this, 'clerk_info_box_data' ),
			'woocommerce_page_wc-orders',
			'side',
			'default'
		);
	}

	function clerk_info_box_data( $object = false ) {
		global $post;

		// If the object is a WC_Order then HPOS is in use.
		if ( is_a( $object, 'WC_Order' ) ) {
			$id    = $object->get_id();
			$order = $object;
			$hpos  = true;
		} else {
			$id    = $post->ID;
			$order = wc_get_order( $id );
			$hpos  = false;
		}

		if ( ! is_a( $order, 'WC_Order' ) ) { 
			return;
		}
		
		$clerk = $order->get_meta( '_ign_one_page_clerk', true );

		if ( empty( $clerk ) ) { 
			echo '<p>' . __( 'No clerk found for this order.', 'manual_phone_orders' ) . '</p>';
			return;
		}
		$data = get_user_by( 'id', $clerk );

		if ( empty( $data ) || is_wp_error( $data ) ) {
			_e( 'User not found', 'manual_phone_orders' );
			return;
		}

		echo '<p>' . __( 'This order was placed manually by:', 'manual_phone_orders' ) . '</p>';

		echo '<p><a href="' . admin_url( 'user-edit.php?user_id=' . $data->ID ) . '" target="_blank">#' . $data->ID . '</a> &ndash; ' . $data->user_login . '</p>';

		echo '<p>' . get_user_meta( $data->ID, 'first_name', true ) . ' ' . get_user_meta( $data->ID, 'last_name', true ) . '</p>';
	}

	// Adds column header to 'Orders' page immediately after 'Total' column
	function admin_order_column_header( $columns ) {

		if ( empty( $this->settings['show_order_column'] ) || 'yes' !== $this->settings['show_order_column'] ) { 
			return $columns;
		}
		
		$new_columns = array();

		foreach ( $columns as $column_name => $column_info ) {

			$new_columns[ $column_name ] = $column_info;

			if ( 'order_total' === $column_name ) {
				// Filterable column name, "Via" by default:
				$new_columns['order_channel_name'] = apply_filters( 'ignitewoo_phone_manual_order_channel_column_title', __( 'Via', 'manual_phone_orders' ) );
			}
		}

		return $new_columns;
	}

	// Column content
	function admin_order_column_content( $column, $order = false ) {
		global $post;

		if ( 'order_channel_name' === $column ) {

			if ( empty( $order ) ) { 
				$order = wc_get_order( $post->ID );
			} elseif ( ! is_a( $order, 'WC_Order' ) ) { 
				// $order is $post->ID;
				$order = wc_get_order( $order );
			}

			if ( ! is_a( $order, 'WC_Order' ) ) { 
				return;
			}
			
			$clerk_id = $order->get_meta( '_ign_one_page_clerk', true );

			if ( empty( $clerk_id ) ) { 
				return;
			}
			
			$data = get_user_by( 'id', $clerk_id ); 
			
			if ( empty( $data ) || is_wp_error( $data ) ) { 
				return;
			}
			
			$fname = get_user_meta( $data->ID, 'billing_first_name', true );
			$lname = get_user_meta( $data->ID, 'billing_last_name', true );
			
			if ( empty( $fname ) || empty( $lname ) ) { 
				$fname = get_user_meta( $data->ID, 'first_name', true );
				$lname = get_user_meta( $data->ID, 'last_name', true );
			}
			
			// Filterable by 3rd party software, pass the User ID, first name, and last name: 
			echo apply_filters( 'ignitewoo_phone_manual_order_channel_text', __( 'Manual/Phone', 'manual_phone_orders' ) . '<br/>(' . $fname . ' ' . $lname . ')', $data->ID, $fname, $lname );
		}
	}

	// Column sortable 
	function admin_order_column_sortable( $columns ) {
	
		if ( empty( $this->settings['show_order_column'] ) || 'yes' !== $this->settings['show_order_column'] ) { 
			return $columns;
		}
		
		$custom = array(
			'order_channel_name'    => '_store_clerk_id', 
		);
			
		return wp_parse_args( $custom, $columns );
	}

	// Alter query for sortable column
	function admin_order_column_sortable_request( $query ) {
	
		if ( empty( $this->settings['show_order_column'] ) || 'yes' !== $this->settings['show_order_column'] ) { 
			return;
		}
		
		if ( 'shop_order' !== $query->get('post_type' ) ) { 
			return;
		}
		
		$orderby = $query->get('orderby' );
		
		if ( empty( $orderby ) || '_store_clerk_id' !== $orderby ) { 
			return;
		}
	
		if ( '_store_clerk_id' == $orderby ) {  
			$query->set('meta_key','_store_clerk_id');  
			$query->set('orderby','meta_value_num');  
		}  		
	}
	
	function maybe_disabled_google_analytics_pro( $do_not_track = false, $admin_event = false, $user_id = false ) {
		global $post; 
		
		if ( empty( $this->settings['manual_order_page'] ) || absint( $this->settings['manual_order_page'] ) <= 0 ) { 
			return $do_not_track;
		}
		
		if ( empty( $post ) || empty( $post->ID ) ) { 
			return $do_not_track;
		}
	
		if ( absint( $post->ID ) !== absint( $this->settings['manual_order_page'] ) ) { 
			return $do_not_track;
		}
		
		return true; 
	}
	
	
}
global $ign_opc_core;
$ign_opc_core = new IGN_Manual_Phone_Orders_Core();


